#ifndef __CAFE_TREE_H__
#define __CAFE_TREE_H__

#include<tree.h>
#include<birthdeath.h>
#include<pthread.h>
#include<gmatrix.h>

typedef struct 
{
	Tree	super;
	int		familysizes[2], rootfamilysizes[2];	
	double	lambda;
	double* factors[2];
	int		size_of_factor;
	int 	rfsize;
	pBirthDeathCacheArray pbdc_array;
}CafeTree;
typedef CafeTree* pCafeTree;

typedef struct
{
	PhylogenyNode super;
	double  lambda;
	double* likelihoods;		
	int*    viterbi;
	int		familysize;	
	double** bd;
}CafeNode;

typedef CafeNode*	pCafeNode;

typedef struct
{
	char* id;
	int*  count;
	char* desc;
	int   maxlh;
	int   ref;
	double*  lambda;
	pBirthDeathCacheArray pbdc_array;
	int   holder;
}CafeFamilyItem;

typedef CafeFamilyItem* pCafeFamilyItem;

typedef struct
{
	char** species;
	int   num_species;
	int*  index;
	int   max_size;
	pArrayList flist;
}CafeFamily;

typedef CafeFamily* pCafeFamily;

typedef struct tagCafeParam CafeParam;
typedef CafeParam* pCafeParam;
typedef void (*lambda_func)(pCafeParam param, double* lambda);
typedef void (*branchlength_func)(pCafeParam param, int* t);

struct tagCafeParam
{
	FILE *fout, *flog;
	pString str_fdata, str_fout, str_log;

	pCafeTree pcafe;
	pCafeFamily pfamily;
	
	// Deprecated
	double* lambda_scores;
	double* lambda_values;

	double* lambda;
	pTree lambda_tree;
	double* ML;
	int num_lambdas;
	lambda_func lambda_set_func;
	branchlength_func branchlength_update_func;
	double bl_augment;

	int* branch_range;
	int* old_branchlength;
	int blsize;
	int family_sizes[2];
	int rootfamily_sizes[2];

	double pvalue;
	
	int  num_threads;
	int  num_random_samples;

	double** viterbiPvalues;
	int** expandRemainDecrease;
	int** viterbiNodeFamilysizes;
	double* maximumPvalues;
	double* averageExpansion;
	double** cutPvalues;
	double** likelihoodRatios;
};


typedef enum 
{
	CAFE_REPORT_TEXT = 0,
	CAFE_REPORT_HTML,
	CAFE_REPORT_PDF,
}enumCafeReport;

/****************************************************************************
 * Cafe Tree
****************************************************************************/

extern double cafe_get_likelihood(pCafeParam param);
extern pCafeTree cafe_tree_new(char* sztree, int familysizes[], int rootfamilysizes[], double lambda);
extern pTreeNode cafe_tree_new_empty_node(pTree pcafe);
extern void cafe_tree_set_parameters(pCafeTree pcafe, int familysizes[], int rootfamilysizes[], double lambda);
extern pCafeTree cafe_tree_copy(pCafeTree psrc);
extern pCafeTree cafe_tree_split(pCafeTree pcafe, int idx );
extern void cafe_tree_free(pCafeTree pcafe);
extern void __cafe_tree_free_node(pTree ptree, pTreeNode ptnode, va_list ap);
extern void cafe_tree_string_name(pString pstr, pPhylogenyNode ptnode);
extern pString cafe_tree_string_with_lambda(pCafeTree pcafe);
extern pString cafe_tree_string_with_id(pCafeTree pcafe);
extern pString cafe_tree_string(pCafeTree pcafe);
extern pString cafe_tree_string_with_familysize_lambda(pCafeTree pcafe);
extern pString cafe_tree_string_with_familysize(pCafeTree pcafe);
extern void cafe_tree_string_print(pCafeTree pcafe);
extern void cafe_tree_set_birthdeath(pCafeTree pcafe);

extern double* cafe_tree_likelihood(pCafeTree pcafe);
extern void cafe_tree_viterbi(pCafeTree pcafe);

extern double* cafe_tree_random_probabilities(pCafeTree pcafe, int rootFamilysize, int trials );
extern double* cafe_tree_p_values(pCafeTree pcafe, double* p,  pArrayList pconddist, int cdlen);
extern double** cafe_tree_p_values_of_two_trees(pCafeTree pcafe1, pCafeTree pcafe2,    
		 								   double** p,
		                                   pArrayList pconddist1, pArrayList pconddist2,
										   int cdlen );

extern pCafeParam cafe_load_parameters(char* fconf);
extern void cafe_save_parameters(pCafeParam param, char* file);
extern void cafe_free_parameters(pCafeParam param);

extern pCafeParam cafe_copy_parameters(pCafeParam psrc);
extern void cafe_free_copy_parameters(pCafeParam param);

extern pArrayList cafe_tree_conditional_distribution(pCafeTree pcafe, int range[], int trial );
extern double cafe_tree_mp_remark(pString str, pTree ptree, pMetapostConfig pmc, va_list ap);
extern double cafe_tree_mp_annotation(pString str, pTreeNode pnode, pMetapostConfig pmc, va_list ap);
extern pMetapostConfig cafe_tree_get_default_mpconfig(int id, double width, double height );
extern pString cafe_tree_metapost(pCafeTree pcafe, int id, char* title, double width, double height);
extern int cafe_tree_random_familysize(pCafeTree pcafe, int rootFamilysize );


/****************************************************************************
 * Cafe Family
****************************************************************************/
extern pCafeFamily cafe_family_new(char* file, int bpatcheck);
extern void cafe_family_item_free(pCafeFamilyItem pitem );
extern void cafe_family_free(pCafeFamily pcf);
extern void cafe_family_set_size(pCafeFamily pcf, int idx, pCafeTree pcafe);
extern int cafe_family_set_species_index(pCafeFamily pcf, pCafeTree pcafe );
extern void cafe_family_set_size_for_split(pCafeFamily pcf, int idx, pCafeTree pcafe);
extern void cafe_family_set_size_with_family(pCafeFamily pcf, int idx, pCafeTree pcafe );
extern int cafe_family_get_index(pCafeFamily pcf, char* szid);
extern pCafeFamilyItem cafe_family_get_family_item(pCafeFamily pcf, char* szid );
extern void cafe_family_set_size_with_family_forced(pCafeFamily pcf, int idx, pCafeTree pcafe);
extern void cafe_family_filter( pCafeParam param );
extern void cafe_family_reset_maxlh(pCafeFamily pcf);

/****************************************************************************
 * Cafe Main
****************************************************************************/

extern void thread_run_with_arraylist(int numthreads, void* (*run)(void*), pArrayList pal );
extern void cafe_log(pCafeParam param, char* msg, ... );
extern void cafe_set_birthdeath_cache(pCafeParam param);
extern void cafe_set_birthdeath_cache_thread(pCafeParam param);
extern double* cafe_best_lambda_by_fminsearch(pCafeParam param, int lambda_len);
extern double* cafe_each_best_lambda_by_fminsearch(pCafeParam param, int lambda_len );
extern void cafe_report(pCafeParam param, int method);
extern pArrayList cafe_conditional_distribution(pCafeParam param);
//extern void cafe_birthdeath_cache_refresh( pCafeTree pcafe );
extern void cafe_lambda_set_default(pCafeParam param, double* lambda);

extern void cafe_resize_birthdeath_cache(pCafeParam param);
extern void cafe_free_birthdeath_cache(pCafeTree pcafe);
extern pArrayList cafe_viterbi(pCafeParam param, pArrayList pCD);
extern void cafe_branch_cutting(pCafeParam param);
extern void cafe_likelihood_ratio_test(pCafeParam param);
extern void cafe_lhr_for_diff_lambdas(pCafeParam param, pTree lambda_tree2, int num_lambdas, lambda_func lfunc );
extern pGMatrix cafe_lambda_distribution(pCafeParam param, int numrange, double** range );

extern int cafe_report_retrieve_data(char* file, pCafeParam param);


#endif
