#include<cafe.h>
#include<stdlib.h>
#include<stdio.h>

void __cafe_famliy_check_the_pattern(pCafeFamily pcf)
{
	int i, j, k;
	int size = pcf->flist->size;
	for ( i = 0 ; i < size ; i++ )
	{
		pCafeFamilyItem pref = (pCafeFamilyItem)pcf->flist->array[i];
		if ( pref->ref != -1 ) continue;
		pref->ref = i;
		pref->holder = 1;
		for ( j = i + 1 ; j < size ; j++ )
		{
			pCafeFamilyItem pdest = (pCafeFamilyItem)pcf->flist->array[j];
			if ( pdest->ref != -1 ) continue;
			for ( k = 0 ; k < pcf->num_species ; k++ )
			{
				if ( pref->count[k] != pdest->count[k] ) break;
			}
			if ( k == pcf->num_species )
			{
				pdest->ref = i;		
				pdest->holder = 0;
			}
		}
	}
}

pCafeFamily cafe_family_new(char* file, int bpatcheck)
{
	FILE* fp = fopen(file,"r");
	char buf[BUFSIZ];
	if ( fp == NULL )
	{
		fprintf( stderr, "Cannot open file: %s\n", file );
//		print_error(__FILE__,(char*)__FUNCTION__,__LINE__, "Cannot open file: %s ", file );
		return NULL;
	}
	pCafeFamily pcf = (pCafeFamily)memory_new( 1, sizeof(CafeFamily) );
	int i, j;
	if ( fgets(buf,BUFSIZ,fp) == NULL )
	{
		fclose(fp);
		fprintf( stderr, "Empty file: %s\n", file );
//		print_error(__FILE__,(char*)__FUNCTION__,__LINE__, "Input file is empty" );
		return NULL;
	}
	string_pchar_chomp(buf);
	pcf->flist = arraylist_new(11000);

	pArrayList data = string_pchar_split( buf, '\t');
	pcf->species = (char**)memory_new((data->size-2), sizeof(char*));
	for ( i = 2 ; i < data->size ; i++ )
	{
		pcf->species[i-2] = (char*)data->array[i];
	}
	free(data->array[0]);
	free(data->array[1]);

	pcf->num_species = data->size - 2;
	arraylist_free(data,NULL);
	pcf->max_size = 0;
	pcf->index = (int*)memory_new( pcf->num_species, sizeof(int));
	memset(pcf->index, -1, sizeof(pcf->index));

	for(  i = 0 ; fgets(buf,BUFSIZ,fp) ; i++ )	
	{
//		string_pchar_chomp(buf);
		pCafeFamilyItem pitem = (pCafeFamilyItem)memory_new(1,sizeof(CafeFamilyItem));
		data = string_pchar_split( buf, '\t');
		if ( data->size != (pcf->num_species + 2) )
		{
			for ( j = 0 ; j < data->size ; j++ )
			{
				fprintf(stderr,"%d: %s\n", j, (char*)data->array[i] );
			}
			print_error(__FILE__,(char*)__FUNCTION__,__LINE__, 
					"The number of column is not consistant(%d), but %d in line %d", 
					pcf->num_species+2, data->size,  i+2);
		}
		pitem->desc = (char*)data->array[0];
		pitem->id = (char*)data->array[1];		
		pitem->count = (int*)memory_new(pcf->num_species, sizeof(int));
		pitem->maxlh = -1;   // Maximum likelihood index
		pitem->ref = -1;
		pitem->lambda = NULL;
		pitem->pbdc_array = NULL;
		pitem->holder = 1;
		for ( j = 0 ; j < pcf->num_species ; j++ )
		{
			if ( data->array[j+2] )
			{
				pitem->count[j] = atoi((char*)data->array[j+2]);
				if ( pcf->max_size < pitem->count[j] )
				{
					pcf->max_size = pitem->count[j];
				}
				free(data->array[j+2]);
			}
			else
			{
				pitem->count[j] = 0;
			}
		}
		arraylist_free(data,NULL);
		arraylist_add(pcf->flist,pitem);
	}
	if ( bpatcheck )
	{
		__cafe_famliy_check_the_pattern(pcf);
	}
	fclose(fp);
	return pcf;
}

void cafe_family_item_free(pCafeFamilyItem pitem )
{
	free(pitem->id);
	free(pitem->count);
	if ( pitem->desc ) free(pitem->desc);
	if ( pitem->holder )
	{
		if( pitem->lambda ) memory_free(pitem->lambda);
		if( pitem->pbdc_array ) birthdeath_cache_array_free(pitem->pbdc_array);
	}
	free(pitem);
}

int cafe_family_set_species_index(pCafeFamily pcf, pCafeTree pcafe )
{
	int i,j;
	pTree ptree = (pTree)pcafe;
	for ( i = 0 ; i < pcf->num_species ;  i++ )
	{
		for( j = 0 ; j < ptree->nlist->size; j+=2 )
		{
			pPhylogenyNode pnode = (pPhylogenyNode)ptree->nlist->array[j];
			if ( pnode->name[0] & 0x80 ) continue;
			if ( string_pchar_cmp_ignore_case(pnode->name, pcf->species[i] ) )
			{
				pnode->name[0] |= 0x80;
				pcf->index[i] = j;
				break;
			}
		}
		printf("%d: %s => %d\n", i, pcf->species[i],j );
	}

	int err = 0;
	for( j = 0 ; j < ptree->nlist->size; j+=2 )
	{
		pPhylogenyNode pnode = (pPhylogenyNode)ptree->nlist->array[j];
		if ( !(pnode->name[0] & 0x80 ) )
		{
			fprintf(stderr,"Family has no %s species\n", pnode->name );
			err = -1;
		}
	}

	for( j = 0 ; j < ptree->nlist->size; j+=2 )
	{
		pPhylogenyNode pnode = (pPhylogenyNode)ptree->nlist->array[j];
		pnode->name[0] &= 0x7F;
	}
	return err;
}


void cafe_family_free(pCafeFamily pcf)
{
	int i;
	for( i = 0 ; i < pcf->num_species; i++ ) free( pcf->species[i] );
	free(pcf->species);
	free(pcf->index);
	arraylist_free( pcf->flist, (freefunc)cafe_family_item_free );	
	free(pcf);
}

void cafe_family_set_size(pCafeFamily pcf, int idx, pCafeTree pcafe)
{
	pTree ptree = (pTree)pcafe;
	pCafeFamilyItem pitem = (pCafeFamilyItem)pcf->flist->array[idx];
	int i;
	for ( i = 0 ; i < pcf->num_species ; i++ )
	{
		if( pcf->index[i] < 0 ) continue;
		pCafeNode pcnode = (pCafeNode)ptree->nlist->array[pcf->index[i]];
		pcnode->familysize = pitem->count[i];
	}
}

void cafe_family_set_size_with_family_forced(pCafeFamily pcf, int idx, pCafeTree pcafe)
{
	cafe_family_set_size(pcf, idx, pcafe);
	pCafeFamilyItem pitem = (pCafeFamilyItem)pcf->flist->array[idx];
	int i;
	int max = 0;
	for ( i = 0 ; i < pcf->num_species ; i++ )
	{
		if( pcf->index[i] < 0 ) continue;
		if ( max < pitem->count[i] )
		{
			max = pitem->count[i];
		}
	}
	pcafe->rootfamilysizes[0] = 1;
	pcafe->rootfamilysizes[1] = rint(max * 1.25);
	pcafe->familysizes[1] = max + MAX(50,max/5);
	pcafe->rfsize = pcafe->rootfamilysizes[1] - pcafe->rootfamilysizes[0] + 1;
}

void cafe_family_set_size_with_family(pCafeFamily pcf, int idx, pCafeTree pcafe )
{
	cafe_family_set_size(pcf, idx, pcafe);
	pCafeFamilyItem pitem = (pCafeFamilyItem)pcf->flist->array[idx];
	int i;
	int max = 0;
	for ( i = 0 ; i < pcf->num_species ; i++ )
	{
		if( pcf->index[i] < 0 ) continue;
		if ( max < pitem->count[i] )
		{
			max = pitem->count[i];
		}
	}
	if ( pitem->maxlh >= 0 )
	{
		pcafe->rootfamilysizes[0] = MAX(1,pitem->maxlh - 10);
		pcafe->rootfamilysizes[1] = pitem->maxlh + 20;
		pcafe->familysizes[1] = MAX( MAX(pitem->maxlh + 50, rint(pitem->maxlh * 1.2)), 
					                 MAX(max + 50, rint(max*1.2) ) );
		pcafe->rfsize = pcafe->rootfamilysizes[1] - pcafe->rootfamilysizes[0] + 1;
//		max = MAX( pcafe->rootfamilysizes[1], pcafe->familysizes[1] );
	}
}

void cafe_family_reset_maxlh(pCafeFamily pcf)
{
	int i;
	pArrayList flist = pcf->flist;
	for ( i = 0 ; i < flist->size ; i++ )
	{
		pCafeFamilyItem pitem = (pCafeFamilyItem)flist->array[i];			
		pitem->maxlh = -1;
	}
}

int cafe_family_get_index(pCafeFamily pcf, char* szid)
{
	int i;
	pArrayList flist = pcf->flist;
	for ( i = 0 ; i < flist->size ; i++ )
	{
		pCafeFamilyItem pitem = (pCafeFamilyItem)flist->array[i];			
		if ( pitem->id && strcmp( pitem->id, szid ) == 0 ) break;
	}
	return flist->size == i ? -1 : i;
}

pCafeFamilyItem cafe_family_get_family_item(pCafeFamily pcf, char* szid )
{
	int i = cafe_family_get_index( pcf, szid );	
	if ( i == -1 ) return NULL;
	return (pCafeFamilyItem)pcf->flist->array[i];
}

void cafe_family_set_size_for_split(pCafeFamily pcf, int idx, pCafeTree pcafe)
{
	int i, j;
	pCafeFamilyItem pitem = (pCafeFamilyItem)pcf->flist->array[idx];
	for ( i = 0 ; i < pcf->num_species; i++ )
	{
		for ( j = 0 ; j < pcafe->super.nlist->size ; j+=2 )
		{
			pPhylogenyNode pnode = (pPhylogenyNode)pcafe->super.nlist->array[j];
			if ( pnode->name[0] & 0x80 ) continue;
			if ( string_pchar_cmp_ignore_case(pnode->name, pcf->species[i] ) )
			{
				pnode->name[0] |= 0x80;
				((pCafeNode)pnode)->familysize = pitem->count[i];
				break;
			}
		}
	}

	for( j = 0 ; j < pcafe->super.nlist->size; j+=2 )
	{
		pPhylogenyNode pnode = (pPhylogenyNode)pcafe->super.nlist->array[j];
		pnode->name[0] &= 0x7F;
	}
}

/*
                 p      r
      0   0   => 0      0 
	  0   1   => 1/0    0 ( 0 side has child with 1, then 1 )
      0   1/0 => 0      0
      1   1/0 => 1      1
      1   1   => 1      1
      1/0 1/0 => 1/0    1
 */

#ifdef __HANDUCK__

void cafe_family_filter(pCafeParam param )
{
	int i,n;
	pCafeFamily pcf = param->pfamily;
	pCafeTree pcafe = param->pcafe;
//	pArrayList postfix = pcafe->super.postfix;
	pArrayList fflist = arraylist_new(11000);
	int max = 0;
	for ( i = 0 ; i < pcf->flist->size ; i++ )
	{
		cafe_family_set_size(pcf,i,pcafe);
		pArrayList nlist = pcafe->super.nlist;
		for ( n = 0 ; n < nlist->size; n+=2 )
		{
			pCafeNode pnode = (pCafeNode)nlist->array[n];
			if ( pnode->familysize > 0 )
			{
				pTreeNode p = ((pTreeNode)pnode);
				do 
				{
					p->reg = 1;
					p = p->parent;
				}while( p && p->reg == 0 );
			}
		}
		pTreeNode root = ((pTree)pcafe)->root;
		pTreeNode child[2] =  { tree_get_child(root,0), tree_get_child(root,1) };
		pTreeNode gchild[4] =  { tree_get_child(child[0],0), 
								tree_get_child(child[0],1), 
								tree_get_child(child[1],0), 
								tree_get_child(child[1],1) }; 
		root->reg = 0;
		if ( child[0]->reg && child[1]->reg )
		{
			root->reg = 1;
		}
		else if ( child[0]->reg || child[1]->reg )
		{
		//	if ( tree_is_leaf( child[0] ) )
			if ( child[0]->reg == 0 )
			{
				root->reg = (gchild[2] && gchild[2]->reg) && (gchild[3] && gchild[3]->reg );
			}
	//		else if ( tree_is_leaf( child[1] ) )
			else if ( child[1]->reg == 0 )
			{
				root->reg = (gchild[0] && gchild[0]->reg) && (gchild[1] && gchild[1]->reg );
			}
		}
	/*
		for ( n = 0 ; n < postfix->size - 1 ; n++ )
		{
			pTreeNode pnode = (pTreeNode)postfix->array[n];
			if ( tree_is_leaf(pnode) )
			{
				pnode->reg = ((pCafeNode)pnode)->familysize > 0 ? 1 : 0;
			}
			else
			{
				int reg[2] =  { ((pTreeNode)tree_get_child(pnode,0))->reg, 
				              ((pTreeNode)tree_get_child(pnode,1))->reg };
				if ( reg[0] * reg[1] == 0 )
				{
					pnode->reg = reg[0] + reg[1] > 0 ? -1 : 0;
				}
				else
				{
					pnode->reg = reg[0] + reg[1] >= 0 ? 1 : 0;
				}
			}
		}
		pTreeNode root = (pTreeNode)postfix->array[n];
		root->reg = 0;
		pTreeNode child[2] =  { tree_get_child(root,0), tree_get_child(root,1) };
		if ( child[0]->reg && child[1]->reg )
		{
			root->reg = 1;
		}
		else if ( child[0]->reg || child[1]->reg )
		{
			if ( child[0]->reg && !tree_is_leaf(child[0]) )
			{
				root->reg = 1;
			}
			else if ( child[1]->reg && !tree_is_leaf(child[1]) )
			{
				root->reg = 1;
			}
		}
	*/
		pCafeFamilyItem pitem = (pCafeFamilyItem)pcf->flist->array[i];
		if ( root->reg == 1 )
		{
			arraylist_add( fflist, pitem );
			for ( n = 0 ; n < pcf->num_species ; n++ )
			{
				if ( pitem->count[n] > max )
				{
					max = pitem->count[n];
				}
			}
		}
		else
		{
			cafe_family_item_free(pitem);
		}
		tree_clear_reg((pTree)pcafe);
	}
	if ( pcf->flist->size != fflist->size )
	{
		cafe_log( param, "The Number of families : %d ==> %d\n", pcf->flist->size, fflist->size );
		arraylist_free( pcf->flist , NULL );
		pcf->flist = fflist;
		for ( i = 0 ; i < fflist->size ; i++ )
		{
			pCafeFamilyItem pitem = (pCafeFamilyItem)pcf->flist->array[i];
			pitem->ref = -1;
		}
		__cafe_famliy_check_the_pattern(pcf);
	}
	else
	{
		arraylist_free( fflist, NULL );
	}
	if ( pcf->max_size != max )
	{
		pcf->max_size = max;
		param->rootfamily_sizes[1] = rint(param->pfamily->max_size * 1.25);
		param->family_sizes[1] = param->pfamily->max_size + MAX(50,param->pfamily->max_size/5);
		cafe_tree_set_parameters(pcafe, param->family_sizes, param->rootfamily_sizes, 0 );
	}
}
#endif 


void cafe_family_filter(pCafeParam param )
{
	int i,n;
	pCafeFamily pcf = param->pfamily;
	pCafeTree pcafe = param->pcafe;
	pArrayList fflist = arraylist_new(11000);
	int max = 0;
	for ( i = 0 ; i < pcf->flist->size ; i++ )
	{
		cafe_family_set_size(pcf,i,pcafe);
		pArrayList nlist = pcafe->super.nlist;
		for ( n = 0 ; n < nlist->size; n+=2 )
		{
			pCafeNode pnode = (pCafeNode)nlist->array[n];
			if ( pnode->familysize > 0 )
			{
				pTreeNode p = ((pTreeNode)pnode);
				do 
				{
					p->reg = 1;
					p = p->parent;
				}while( p && p->reg == 0 );
			}
		}
		pTreeNode root = ((pTree)pcafe)->root;
		pTreeNode child[2] =  { tree_get_child(root,0), tree_get_child(root,1) };
		pTreeNode gchild[4] =  { 
                tree_get_child(child[0],0), 
								tree_get_child(child[0],1), 
								tree_get_child(child[1],0), 
								tree_get_child(child[1],1) 
                }; 
    /*pTreeNode ggchild[8] = {
								tree_get_child(gchild[0],0), 
								tree_get_child(gchild[0],1), 
								tree_get_child(gchild[1],0), 
								tree_get_child(gchild[1],1), 
								tree_get_child(gchild[2],0), 
								tree_get_child(gchild[2],1), 
								tree_get_child(gchild[3],0), 
								tree_get_child(gchild[3],1), 
                };*/
		root->reg = 0;
		if ( child[0]->reg && child[1]->reg )
		{
			root->reg = 1;
		}
    
    // (1 and 1)
		else if ( child[0]->reg || child[1]->reg )
   		{
   	   		root->reg = 0;
    	}
    
    /* 
    // (check down level)
    else if ( child[0]->reg || child[1]->reg )
		{
			if ( child[0]->reg == 1 )
			{
        // check downto level 2 
        root->reg = 1;
        if ( gchild[0] ) { root->reg = root->reg && gchild[0]->reg; }
        if ( gchild[1] ) { root->reg = root->reg && gchild[1]->reg; }
        
        // check downto level 3
        if ( ggchild[0] ) { root->reg = root->reg && ggchild[0]->reg; }
        if ( ggchild[1] ) { root->reg = root->reg && ggchild[1]->reg; }
        if ( ggchild[2] ) { root->reg = root->reg && ggchild[2]->reg; }
        if ( ggchild[3] ) { root->reg = root->reg && ggchild[3]->reg; }
        
			}
			else if ( child[1]->reg == 1 )
			{
        // check downto level 2 
        root->reg = 1;
        if ( gchild[2] ) { root->reg = root->reg && gchild[2]->reg; }
        if ( gchild[3] ) { root->reg = root->reg && gchild[3]->reg; }
        
        // check downto level 3
        if ( ggchild[4] ) { root->reg = root->reg && ggchild[4]->reg; }
        if ( ggchild[5] ) { root->reg = root->reg && ggchild[5]->reg; }
        if ( ggchild[6] ) { root->reg = root->reg && ggchild[6]->reg; }
        if ( ggchild[7] ) { root->reg = root->reg && ggchild[7]->reg; }
        
			}
		}
    */
		pCafeFamilyItem pitem = (pCafeFamilyItem)pcf->flist->array[i];
		if ( root->reg == 1 )
		{
			arraylist_add( fflist, pitem );
			for ( n = 0 ; n < pcf->num_species ; n++ )
			{
				if ( pitem->count[n] > max )
				{
					max = pitem->count[n];
				}
			}
		}
		else
		{
      // print filtered families 
		  /*
      tree_clear_reg((pTree)pcafe);
      cafe_log( param, "%s \t", pitem->id  ); 
      pString pstr = cafe_tree_string_with_familysize(pcafe);
      cafe_log(param, "%s\n", pstr->buf );
      string_free(pstr);
      */
			cafe_family_item_free(pitem);
		}
		tree_clear_reg((pTree)pcafe);
	}
	if ( pcf->flist->size != fflist->size )
	{
		cafe_log( param, "The Number of families : %d ==> %d\n", pcf->flist->size, fflist->size );
		arraylist_free( pcf->flist , NULL );
		pcf->flist = fflist;
		for ( i = 0 ; i < fflist->size ; i++ )
		{
			pCafeFamilyItem pitem = (pCafeFamilyItem)pcf->flist->array[i];
			pitem->ref = -1;
		}
		__cafe_famliy_check_the_pattern(pcf);
	}
	else
	{
		arraylist_free( fflist, NULL );
	}

	if ( pcf->max_size != max )
	{
		pcf->max_size = max;
		param->rootfamily_sizes[1] = rint(param->pfamily->max_size * 1.25);
		param->family_sizes[1] = param->pfamily->max_size + MAX(50,param->pfamily->max_size/5);
		cafe_tree_set_parameters(pcafe, param->family_sizes, param->rootfamily_sizes, 0 );
	}
}




