#include<cafe.h>
#include<stdlib.h>
#include<math.h>
#include<stdio.h>

void cafe_log(pCafeParam param, char* msg, ... )
{
	va_list ap;
	va_start(ap, msg);

	if ( param->flog != stderr && param->flog != stdout )
	{
		vfprintf(param->flog, msg, ap);
	}
	vfprintf(stdout, msg, ap);
	fflush(param->flog);
	va_end(ap);
}

void cafe_add_birthdeath_cache(pCafeParam param, pCafeTree pcafe )
{
	int i;
	for ( i = 0 ; i < pcafe->super.nlist->size ; i++ )
	{
		pPhylogenyNode pnode = (pPhylogenyNode)pcafe->super.nlist->array[i];
		if ( pnode->branchlength > 0 )
		{
			birthdeath_cache_get_matrix(pcafe->pbdc_array, pnode->branchlength , ((pCafeNode)pnode)->lambda );
		}
	}
	cafe_tree_set_birthdeath(param->pcafe);
}


void cafe_free_birthdeath_cache(pCafeTree pcafe)
{
	birthdeath_cache_array_free(pcafe->pbdc_array);
	pcafe->pbdc_array = NULL;
}

void thread_run(int numthreads, void* (*run)(void*), void* param, int size )
{
	int i;
	pthread_t* pthreads = (pthread_t*)memory_new( numthreads, sizeof(pthread_t));
	for ( i = 0 ; i < numthreads ; i++ )
	{
		if ( pthread_create(&pthreads[i],NULL, run, (void*)((char*)param+size*i)) != 0 )			
		{
			print_error(__FILE__,(char*)__FUNCTION__,__LINE__,
					    "create %dth thread", i);		
		}
	}
	for ( i = 0 ; i < numthreads ; i++ )
	{
		pthread_join(pthreads[i], NULL);
	}
	free(pthreads);
}

void thread_run_with_arraylist(int numthreads, void* (*run)(void*), pArrayList pal )
{
	int i;
	pthread_t* pthreads = (pthread_t*)memory_new( numthreads, sizeof(pthread_t));
	for ( i = 0 ; i < numthreads ; i++ )
	{
		if ( pthread_create(&pthreads[i],NULL, run, pal->array[i] ) != 0 )			
		{
			print_error(__FILE__,(char*)__FUNCTION__,__LINE__,
					    "create %dth thread", i);		
		}
	}
	for ( i = 0 ; i < numthreads ; i++ )
	{
		pthread_join(pthreads[i], NULL);
	}
	free(pthreads);
}


void cafe_lambda_set_default(pCafeParam param, double* lambda)
{
	int i;
	pTree ptree = (pTree)param->pcafe;
	for ( i = 0 ; i < ptree->nlist->size; i++ )
	{
		((pCafeNode)ptree->nlist->array[i])->lambda = lambda[0];
	}
}

/**************************************************************************
 * Lambda 
**************************************************************************/
double __cafe_best_lambda_search(double* plambda, void* args);

pGMatrix cafe_lambda_distribution(pCafeParam param, int numrange, double** range )
{
	int i, j;
	int* size = (int*)memory_new(numrange,sizeof(int));
	double* plambda = (double*)memory_new(numrange,sizeof(double)); 
	int* idx = (int*)memory_new(numrange, sizeof(int));
	for ( i = 0 ; i < numrange; i++ )
	{
		size[i] = 1 + rint((range[i][2] - range[i][0])/range[i][1]);
	}
	pGMatrix pgm = gmatrix_double_new(numrange,size);

	for ( i = 0 ; i < pgm->num_elements; i++ )
	{
		gmatrix_dim_index(pgm,i,idx);
		for ( j = 0 ; j < numrange ; j++ )
		{
			plambda[j] = range[j][1] * idx[j] + range[j][0];
		}
		double v = -__cafe_best_lambda_search(plambda,(void*)param);
		gmatrix_double_set_with_index(pgm, v, i );
		if ( -v > 1e300 )
		{
			for ( j = 0 ; j < param->pfamily->flist->size ; j++ )
			{
				pCafeFamilyItem pitem = (pCafeFamilyItem)param->pfamily->flist->array[j];
				pitem->maxlh = -1;
			}
		}
	}

	memory_free(idx);
	memory_free(size);
	memory_free(plambda);
	return pgm;
}

double cafe_get_likelihood(pCafeParam param)
{
	int i;
	double score = 0;
	for ( i = 0 ; i < param->pfamily->flist->size ; i++ )
	{
		pCafeFamilyItem pitem = (pCafeFamilyItem)param->pfamily->flist->array[i];
		if ( pitem->ref == i ) 
		{
			cafe_family_set_size_with_family(param->pfamily,i,param->pcafe);
//			cafe_family_set_size(param->pfamily, i, pcafe);
			double* likelihood = cafe_tree_likelihood(param->pcafe);
			param->ML[i] = __max(likelihood,param->pcafe->rfsize);
			if ( pitem->maxlh < 0 )
			{
				pitem->maxlh = __maxidx(likelihood,param->pcafe->rfsize); 
			}
		}
		else
		{
			param->ML[i] = param->ML[pitem->ref];
		}
		if ( param->ML[i] == 0 )
		{ 
/*
			printf(">> %d %d\n", i, pitem->ref );
			printf("Root size: %d ~ %d , %d \n", 
					param->pcafe->rootfamilysizes[0],
					param->pcafe->rootfamilysizes[1], param->pcafe->rfsize );
			printf("Family size: %d ~ %d\n", param->pcafe->familysizes[0], param->pcafe->familysizes[1] );
			printf("Root size: %d ~ %d\n", param->rootfamily_sizes[0], param->rootfamily_sizes[1] );
			printf("Family size: %d ~ %d\n", param->family_sizes[0], param->family_sizes[1] );
			pString pstr = cafe_tree_string_with_familysize_lambda(param->pcafe);
			fprintf(stderr, "%d: %s\n", i, pstr->buf );
			string_free(pstr);
*/
			return log(0);
		}
		score += log(param->ML[i]);
	}
	return score;
}

double __cafe_best_lambda_search(double* plambda, void* args)
{
	int i;
	pCafeParam param = (pCafeParam)args;
	pCafeTree pcafe = (pCafeTree)param->pcafe;
	double score = 0;
	int skip = 0;
	for ( i = 0 ; i < param->num_lambdas ; i++ )
	{
		if ( plambda[i] < 0 ) 
		{ 
			skip  = 1;
			score = log(0);
			break;
		}
	}
	if ( !skip )
	{
		param->lambda_set_func(param,plambda);
		cafe_set_birthdeath_cache_thread(param);
		score = cafe_get_likelihood(param);
		cafe_free_birthdeath_cache(pcafe);
	}
	char buf[1024];
	buf[0] = '\0';
	string_pchar_join_double(buf,",", param->num_lambdas, plambda );
	cafe_log(param,"Lambda : %s & Score: %f\n", buf, score);
	return -score;
}

extern int chooseln_cache_size;

double* cafe_best_lambda_by_fminsearch(pCafeParam param, int lambda_len )
{
	int i;
	if ( lambda_len > 0 )
	{
		if( param->lambda ) memory_free(param->lambda );
		param->lambda = (double*)memory_new(lambda_len, sizeof(double));
		param->num_lambdas = lambda_len;
		for ( i = 0 ; i < lambda_len ; i++ )
		{
			param->lambda[i] = 0.5/param->branch_range[ param->blsize - 1 ];
//			param->lambda[i] = 1.0/param->branch_range[ param->blsize - 1 ] * unifrnd();
		}
	}

	param->pcafe->rootfamilysizes[0] = param->rootfamily_sizes[0];
	param->pcafe->rootfamilysizes[1] = param->rootfamily_sizes[1];
	param->pcafe->familysizes[0] = param->family_sizes[0];
	param->pcafe->familysizes[1] = param->family_sizes[1];
	param->pcafe->rfsize = param->rootfamily_sizes[1] - param->rootfamily_sizes[0] + 1;

	lambda_len = param->num_lambdas;
	pFMinSearch pfm = fminsearch_new_with_eq(__cafe_best_lambda_search,lambda_len,param);
	pfm->tolx = 1e-6;
	pfm->tolf = 1e-4;
	fminsearch_min(pfm, param->lambda);
	double *re = fminsearch_get_minX(pfm);
	for ( i = 0 ; i < lambda_len ; i++ ) param->lambda[i] = re[i];
	fminsearch_free(pfm);
	cafe_log(param,"Lambda Search Result: %d\n", pfm->iters );
	param->lambda_set_func(param,param->lambda);
	pString pstr = cafe_tree_string_with_lambda(param->pcafe);
	cafe_log(param,"%s\n", pstr->buf );

	param->pcafe->rootfamilysizes[0] = param->rootfamily_sizes[0];
	param->pcafe->rootfamilysizes[1] = param->rootfamily_sizes[1];
	param->pcafe->familysizes[0] = param->family_sizes[0];
	param->pcafe->familysizes[1] = param->family_sizes[1];
	param->pcafe->rfsize = param->rootfamily_sizes[1] - param->rootfamily_sizes[0] + 1;

	string_free(pstr);
	return param->lambda;
}

double __cafe_each_best_lambda_search(double* plambda, void* args)
{
	int i;
	pCafeParam param = (pCafeParam)args;
	pCafeTree pcafe = (pCafeTree)param->pcafe;
	double score = 0;
	int skip = 0;
	for ( i = 0 ; i < param->num_lambdas ; i++ )
	{
		if ( plambda[i] < 0 ) 
		{ 
			skip  = 1;
			score = log(0);
			break;
		}
	}

	if ( !skip )
	{
		param->lambda_set_func(param,plambda);
		cafe_set_birthdeath_cache_thread(param);
		double* likelihood = cafe_tree_likelihood(pcafe);
		score = log(__max(likelihood,pcafe->rfsize));
		cafe_free_birthdeath_cache(pcafe);
		pcafe->pbdc_array = NULL;
	}

	char buf[1024];
	buf[0] = '\0';
	string_pchar_join_double(buf,",", param->num_lambdas, plambda );
	cafe_log(param,"\tLambda : %s & Score: %f\n", buf, score);

	return -score;

}

double* cafe_each_best_lambda_by_fminsearch(pCafeParam param, int lambda_len )
{
	double* old_lambda = NULL;
	if( param->lambda ) 
	{
		old_lambda = param->lambda;
	}
	param->lambda = (double*)memory_new(lambda_len, sizeof(double));
	param->num_lambdas = lambda_len;

	int rootfamilysizes[2] = { param->rootfamily_sizes[0],  param->rootfamily_sizes[1] };
	int familysizes[2] = { param->family_sizes[0],  param->family_sizes[1] };
	int i, j;
	for ( i = 0 ; i < lambda_len ; i++ )
	{
		param->lambda[i] = 0.5/param->branch_range[ param->blsize - 1 ];
	}
	pFMinSearch pfm = fminsearch_new_with_eq(__cafe_each_best_lambda_search,lambda_len,param);
	pfm->tolx = 1e-4;
	pfm->tolf = 1e-3;
	int fsize = param->pfamily->flist->size;
	for ( i = 0 ; i < param->pfamily->flist->size ; i++ )
	{
		pCafeFamilyItem pitem = (pCafeFamilyItem)param->pfamily->flist->array[i];
		if ( pitem->ref != i )
		{
			pCafeFamilyItem pref = (pCafeFamilyItem)param->pfamily->flist->array[pitem->ref];
			pitem->lambda = pref->lambda;
			pitem->pbdc_array = pref->pbdc_array;

			cafe_log(param,"Lambda Search Result of %d/%d in %d iteration \n", i+1, fsize, pfm->iters );
			pString pstr = cafe_tree_string_with_familysize_lambda(param->pcafe);
			cafe_log(param,"%s\n", pstr->buf );
			string_free(pstr);
			continue;
		}

		cafe_family_set_size_with_family_forced(param->pfamily,i,param->pcafe);

		param->rootfamily_sizes[0] = param->pcafe->rootfamilysizes[0];
		param->rootfamily_sizes[1] = param->pcafe->rootfamilysizes[1];
		param->family_sizes[0] = param->pcafe->familysizes[0];
		param->family_sizes[1] = param->pcafe->familysizes[1];

		cafe_log(param,"%s:\n", pitem->id );
		
		fminsearch_min(pfm, param->lambda );

/*
		printf("%d %d:", param->rootfamily_sizes[1], param->family_sizes[1] );
		for ( j = 0 ; j < param->pcafe->super.nlist->size ; j+=2 )
		{
			pCafeNode pnode = (pCafeNode)param->pcafe->super.nlist->array[j];
			printf(" %d",  pnode->familysize );
		}
		printf("\n");
*/

		double *re = fminsearch_get_minX(pfm);
		if ( pitem->lambda ) memory_free( pitem->lambda );
		pitem->lambda = MEMORY_NEW( lambda_len, double );
		int lambda_check = 0;
		for ( j = 0 ; j < lambda_len ; j++ ) 
		{
			pitem->lambda[j] = re[j];
			double a = re[j] * param->branch_range[ param->blsize - 1 ];
//			printf("%f\n", a);
			if ( a >= 0.5 || fabs(a-0.5) < 1e-3 )
			{
				lambda_check = 1;	
			}	
		}
		param->lambda_set_func(param,re);
//		cafe_set_birthdeath_cache_thread(param);
//		pitem->pbdc_array = param->pcafe->pbdc_array;

		cafe_log(param,"Lambda Search Result of %d/%d in %d iteration \n", i+1, fsize, pfm->iters );
		if ( lambda_check )
		{
			cafe_log(param,"Caution : at least one lambda near boundary\n" );
		}
		pString pstr = cafe_tree_string_with_familysize_lambda(param->pcafe);
		if ( lambda_check )
		{
			cafe_log(param,"@@ ");
		}
		cafe_log(param,"%s\n", pstr->buf );
		string_free(pstr);
	}
	fminsearch_free(pfm);
	param->pcafe->rootfamilysizes[0] = rootfamilysizes[0];
	param->pcafe->rootfamilysizes[1] = rootfamilysizes[1];
	param->pcafe->familysizes[0] = familysizes[0];
	param->pcafe->familysizes[1] = familysizes[1];
	param->pcafe->rfsize = param->rootfamily_sizes[1] - param->rootfamily_sizes[0] + 1;
	param->rootfamily_sizes[0] = rootfamilysizes[0];
	param->rootfamily_sizes[1] = rootfamilysizes[1];
	param->family_sizes[0] = familysizes[0];
	param->family_sizes[1] = familysizes[1];
	free(param->lambda);
	param->lambda = old_lambda;
	return param->lambda;
}

/**************************************************************************
 * Contidional Distribution
**************************************************************************/

typedef struct
{
	pCafeParam cafeparam;
	int range[2];
	pArrayList pCD;
}CDParam;
typedef CDParam* pCDParam;


void* __cafe_conditional_distribution_thread_func(void* ptr)
{
	pCDParam param = (pCDParam)ptr;	
	pCafeParam cafeparam = param->cafeparam;
	pCafeTree pcafe = cafe_tree_copy(cafeparam->pcafe);
	pcafe->pbdc_array = cafeparam->pcafe->pbdc_array;
#ifdef __DEBUG__
	printf("CD: %d ~ %d\n", param->range[0], param->range[1]);
#endif
	param->pCD = cafe_tree_conditional_distribution(pcafe, param->range, cafeparam->num_random_samples);
	cafe_tree_free(pcafe);
	return (NULL);
}

pArrayList cafe_conditional_distribution(pCafeParam param)
{
	int numthreads = param->num_threads;
	int threadstep = param->pcafe->rfsize/numthreads;
	if ( threadstep == 0 )
	{
		numthreads = param->pcafe->rfsize;
	}
	else
	{
		threadstep--;
	}

	pCDParam ptparam = (pCDParam)memory_new(numthreads,sizeof(CDParam));
	int i, r = param->rootfamily_sizes[0] ;
	for ( i = 0 ; i < numthreads; i++, r+=threadstep+1 )
	{
		ptparam[i].cafeparam = param;
		ptparam[i].range[0]= r;
		ptparam[i].range[1]= r + threadstep;
	}
	ptparam[numthreads-1].range[1] = param->rootfamily_sizes[1];
	thread_run(numthreads, __cafe_conditional_distribution_thread_func, ptparam, sizeof(CDParam));
	pArrayList cdlist = ptparam[0].pCD;
	for( i = 1 ; i < numthreads ; i++ )
	{
		for ( r = 0 ; r < ptparam[i].pCD->size ; r++ )
		{
			arraylist_add(cdlist, ptparam[i].pCD->array[r]);
		}
		arraylist_free(ptparam[i].pCD, NULL);
	}
	free(ptparam);
	return cdlist;
}

/**************************************************************************
 * Viterbi
**************************************************************************/

typedef struct
{
	pCafeParam cafeparam;
	pArrayList pCD;
	int from;
}ViterbiParam;

typedef ViterbiParam*  pViterbiParam;

pthread_mutex_t mutex_cafe_viterbi = PTHREAD_MUTEX_INITIALIZER;

void* __cafe_viterbi_thread_func(void* ptr)
{
	int i, j, k, m;
	pViterbiParam pv = (pViterbiParam)ptr;
	pCafeParam param = (pCafeParam)pv->cafeparam;
	pArrayList pCD = pv->pCD;
	pCafeTree pcafe = cafe_tree_copy(param->pcafe);
	pcafe->pbdc_array = param->pcafe->pbdc_array;
	pTree ptree = (pTree)pcafe;
	int nnodes = (ptree->nlist->size-1)/2;
	int fsize = param->pfamily->flist->size;
	double* cP = (double*)memory_new( pcafe->rfsize, sizeof(double));
#ifdef __DEBUG__
	printf("VITERBI: from %d\n", pv->from );
#endif
	for ( i = pv->from; i < fsize ; i+=param->num_threads )
	{
		cafe_family_set_size_with_family_forced(param->pfamily,i, pcafe);
//		cafe_family_set_size(param->pfamily,i, pcafe);
		cafe_tree_p_values(pcafe, cP, pCD, param->num_random_samples);
		param->maximumPvalues[i] = __max(cP,pcafe->rfsize);
		cafe_tree_viterbi(pcafe);
		for ( j = 0 ; j < nnodes ; j++ )
		{
			pCafeNode pcnode = (pCafeNode)ptree->nlist->array[2*j+1];
			param->viterbiNodeFamilysizes[j][i] = pcnode->familysize;
			pCafeNode child[2] = { (pCafeNode)((pTreeNode)pcnode)->children->head->data, 
							       (pCafeNode)((pTreeNode)pcnode)->children->tail->data };
			for ( k = 0 ; k < 2 ; k++ )
			{
				m = j*2 + k;
				if ( child[k]->familysize > pcnode->familysize ) param->expandRemainDecrease[0][m]++;		
				else if ( child[k]->familysize == pcnode->familysize ) param->expandRemainDecrease[1][m]++;		
				else param->expandRemainDecrease[2][m]++;		
pthread_mutex_lock( &mutex_cafe_viterbi );
				param->averageExpansion[m] += child[k]->familysize - pcnode->familysize;
pthread_mutex_unlock( &mutex_cafe_viterbi );
			}
		}

		if ( param->maximumPvalues[i] > param->pvalue ) 
		{
			for ( j = 0 ; j < ptree->nlist->size-1 ; j++ )
			{
				param->viterbiPvalues[j][i] = -1;
			}
			continue;
		}

		for ( j = 0 ; j < nnodes ; j++ )
		{
			pCafeNode pcnode = (pCafeNode)ptree->nlist->array[2*j+1];
			pCafeNode child[2] = { (pCafeNode)((pTreeNode)pcnode)->children->head->data, 
							       (pCafeNode)((pTreeNode)pcnode)->children->tail->data };
			for ( k = 0 ; k < 2 ; k++ )
			{
				double p = child[k]->bd[pcnode->familysize][child[k]->familysize];
				double** pbdc = child[k]->bd;
				int n = 2 * j + k;
				for ( m = 0 ; m <= pcafe->familysizes[1]; m++ )
				{
					if ( pbdc[pcnode->familysize][m] == p )
					{
						param->viterbiPvalues[n][i] += pbdc[pcnode->familysize][m]/2.0;
					}
					else if ( pbdc[pcnode->familysize][m] < p )
					{
						param->viterbiPvalues[n][i] += pbdc[pcnode->familysize][m];
					}
				}
			}
		}
	}
	memory_free(cP);
	cafe_tree_free(pcafe);
	return (NULL);
}

pArrayList cafe_viterbi(pCafeParam param, pArrayList pCD)
{
	cafe_log(param,"Running Viterbi algorithm....\n");

	pViterbiParam ptparam = (pViterbiParam)memory_new(param->num_threads,sizeof(ViterbiParam));
	pTree ptree = (pTree)param->pcafe;

	if ( pCD == NULL )
	{
		pCD = cafe_conditional_distribution(param);
	}

	int nrows = param->pfamily->flist->size;
	int nnodes = ptree->nlist->size - 1;
	param->viterbiPvalues = (double**)memory_new_2dim_with_clear(nnodes,nrows,sizeof(double));
	param->expandRemainDecrease = (int**)memory_new_2dim_with_clear(3,nnodes,sizeof(int));
	param->viterbiNodeFamilysizes = (int**)memory_new_2dim_with_clear(nnodes,nrows,sizeof(int));
	param->maximumPvalues = (double*)memory_new_with_clear(nrows, sizeof(double));
	param->averageExpansion = (double*)memory_new_with_clear(nnodes, sizeof(double));

	int i;
	for ( i = 0 ; i < param->num_threads; i++ )
	{
		ptparam[i].cafeparam = param;
		ptparam[i].from = i;
		ptparam[i].pCD = pCD;
	}
	thread_run(param->num_threads, __cafe_viterbi_thread_func, ptparam, sizeof(ViterbiParam));

	for ( i = 0 ; i < ptree->nlist->size - 1; i++ )
	{
		param->averageExpansion[i] /= param->pfamily->flist->size;
	}

	memory_free(ptparam);					
	return pCD;
}

void cafe_viterbi_print(pCafeParam param)
{
	int i, j;
	int size = param->pfamily->flist->size;
	pCafeTree pcafe = param->pcafe;
	pTree ptree = (pTree)pcafe;
	for ( i = 0 ; i < size ; i++ )
	{
		cafe_family_set_size(param->pfamily,i, pcafe);
		for ( j = 1 ; j < ptree->nlist->size ; j+=2 )
		{
			pCafeNode pcnode = (pCafeNode)ptree->nlist->array[j];
			pcnode->familysize = param->viterbiNodeFamilysizes[j/2][i];
		}
		cafe_tree_string_print(pcafe);
	}
}

/**************************************************************************
 * BranchCutting
**************************************************************************/
typedef struct
{
	pCafeParam cafeparam;
	pArrayList** pCDSs;				
	int range[2];
}BranchCuttingParam;

typedef BranchCuttingParam* pBranchCuttingParam;

void* __cafe_branch_cutting_thread_func(void* ptr)
{
	int i, b;
	pBranchCuttingParam pbc = (pBranchCuttingParam)ptr;
	pCafeParam param = pbc->cafeparam;	
	pArrayList** pCDSs = pbc->pCDSs;

#ifdef __DEBUG__
	printf("Branch cutting : %d ~ %d\n", pbc->range[0], pbc->range[1] -1 );
#endif

	pTree ptree = (pTree)param->pcafe;
	int nnodes = ptree->nlist->size;
	double* p1 = (double*)memory_new( param->pcafe->rfsize, sizeof(double));
	double** p2 = (double**)memory_new_2dim( param->pcafe->rfsize, param->pcafe->rfsize, sizeof(double));

	for ( b = 0 ; b < nnodes ; b++ )
	{
		if ( tree_is_root( ptree,(pTreeNode)ptree->nlist->array[b] ) ) 
		{
			continue;
		}
		
		pCafeTree pcafe = cafe_tree_copy(param->pcafe);
		pcafe->pbdc_array = param->pcafe->pbdc_array;
		pCafeTree psub =  cafe_tree_split(pcafe,b);
		psub->pbdc_array = param->pcafe->pbdc_array;

		for ( i = pbc->range[0] ; i < pbc->range[1] ; i++ )
		{
			pCafeFamilyItem pitem = (pCafeFamilyItem)param->pfamily->flist->array[i];
			if ( pitem->ref != i ) continue;
			if ( param->maximumPvalues[i] > param->pvalue ) 
			{
				param->cutPvalues[b][i] = -1;
				continue;
			}
			if ( tree_is_leaf( psub->super.root ) || tree_is_leaf(pcafe->super.root) )
			{
				pCafeTree pct = tree_is_leaf ( psub->super.root ) ? pcafe : psub;
				cafe_family_set_size_for_split(param->pfamily,i, pct);
				cafe_tree_p_values( pct,p1,pCDSs[b][0], param->num_random_samples  );
				param->cutPvalues[b][i] = __max(p1, pcafe->rfsize );
			}
			else
			{
				cafe_family_set_size_for_split(param->pfamily,i, pcafe);
				cafe_family_set_size_for_split(param->pfamily,i, psub);
				cafe_tree_p_values_of_two_trees(pcafe,psub, p2,
										pCDSs[b][0], pCDSs[b][1], param->num_random_samples/10 );
				param->cutPvalues[b][i] = 0;
				int m,n;
				for ( m = 0 ; m < pcafe->rfsize ; m++ )
				{
					for( n = 0 ; n < pcafe->rfsize ; n++ )
					{
						if ( p2[m][n] > param->cutPvalues[b][i] )
						{
							param->cutPvalues[b][i] = p2[m][n];
						}
					}
				}
			}
		}
		cafe_tree_free(pcafe);	
		cafe_tree_free(psub);
	}
	memory_free((void**)p1);
	memory_free_2dim((void**)p2,param->pcafe->rfsize,param->pcafe->rfsize,NULL);
	return (NULL);
}

void cafe_branch_cutting(pCafeParam param)
{
	cafe_log(param,"Running Branch Cutting....\n");

	pTree ptree = (pTree)param->pcafe;
	int i,b,j;
	int nnodes = ptree->nlist->size;
	pArrayList** pCDSs = (pArrayList**)memory_new_2dim(nnodes,2,sizeof(pArrayList));

	for ( b = 0 ; b < nnodes  ; b++ )
	{
		if ( tree_is_root( ptree,(pTreeNode)ptree->nlist->array[b] ) ) 
		{
			pCDSs[b][0] = NULL;
			pCDSs[b][1] = NULL;
			continue;
		}
		pCafeTree pcafe = cafe_tree_copy(param->pcafe);
		pcafe->pbdc_array = param->pcafe->pbdc_array;
		pCafeTree psub =  cafe_tree_split(pcafe,b);
		psub->pbdc_array = param->pcafe->pbdc_array;
		
		cafe_log(param,">> %d  --------------------\n", b ); 
		pString pstr = cafe_tree_string(pcafe);
		cafe_log(param,"%s\n", pstr->buf );
		string_free(pstr);
		pstr = cafe_tree_string(psub);
		cafe_log(param,"%s\n", pstr->buf );
		string_free(pstr);

		pCafeTree porig = param->pcafe;
		if ( tree_is_leaf( psub->super.root ) )
		{
			param->pcafe = pcafe;
			pCDSs[b][0] = cafe_conditional_distribution(param);
			pCDSs[b][1] = NULL;
		}
		else if ( tree_is_leaf ( pcafe->super.root) )
		{
			param->pcafe = psub;
			pCDSs[b][0] = cafe_conditional_distribution(param);
			pCDSs[b][1] = NULL;
		}
		else
		{
			int orig_r = param->num_random_samples;
			param->num_random_samples /= 10;
			param->pcafe = pcafe;
			pCDSs[b][0] = cafe_conditional_distribution(param);
			param->pcafe = psub;
			pCDSs[b][1] = cafe_conditional_distribution(param);
			param->num_random_samples = orig_r;
		}
		param->pcafe = porig;

		cafe_tree_free(pcafe);	
		cafe_tree_free(psub);
	}

	int threadstep = param->pfamily->flist->size/param->num_threads;
	pBranchCuttingParam ptparam = (pBranchCuttingParam)memory_new(param->num_threads,sizeof(BranchCuttingParam));

	int nrows = param->pfamily->flist->size;
	param->cutPvalues = (double**)memory_new_2dim(nnodes,nrows,sizeof(double));
	int rid = ptree->root->id;
	for ( i = 0 ; i < nrows; i++ )
	{
		param->cutPvalues[rid][i] = -1;
	}

	int r = 0;
	for ( i = 0 ; i < param->num_threads; i++, r+=threadstep )
	{
		ptparam[i].cafeparam = param;
		ptparam[i].pCDSs = pCDSs;
		ptparam[i].range[0] = r;
		ptparam[i].range[1] = r + threadstep;
	}
	ptparam[param->num_threads-1].range[1]= param->pfamily->flist->size;
	thread_run(param->num_threads, __cafe_branch_cutting_thread_func, ptparam, sizeof(BranchCuttingParam));

	for ( i = 0 ; i < nrows ; i++ )
	{
		pCafeFamilyItem pitem = (pCafeFamilyItem)param->pfamily->flist->array[i];
		if ( pitem->ref == i ) continue;
		for( b = 0 ; b < nnodes ; b++ )
		{
			param->cutPvalues[b][i] = param->cutPvalues[b][pitem->ref];
		}
	}

	memory_free(ptparam);					
	for ( i = 0 ; i < nnodes; i++ )
	{
		for ( j = 0 ; j < 2 ; j++ )
		{
			if ( pCDSs[i][j] ) arraylist_free(pCDSs[i][j], free);
		}
	}
	memory_free(pCDSs);
	cafe_log( param , "Done : Branch Cutting\n" );
}

/**************************************************************************
 * Likelihood ratio test
**************************************************************************/
typedef struct
{
	pCafeParam cafeparam;
	int from;
}LRTParam;

typedef LRTParam* pLRTParam;

pthread_mutex_t mutex_cafe_likelihood = PTHREAD_MUTEX_INITIALIZER;

void* __cafe_likelihood_ratio_test_thread_func(void* ptr)
{
	int i,b;
	pLRTParam plrt = (pLRTParam)ptr;
	pCafeParam param  = plrt->cafeparam;
	pTree ptree = (pTree)param->pcafe;
	pCafeTree pcafe = cafe_tree_copy(param->pcafe);
	pcafe->pbdc_array = param->pcafe->pbdc_array;
	int nnodes = ptree->nlist->size;
	int old_bl;
	int fsize = param->pfamily->flist->size;
#ifdef __DEBUG__
	printf("Likelihood ratio test: %d\n", plrt->from );
#endif
	for ( i = plrt->from ; i < fsize ; i += param->num_threads )
	{
		pCafeFamilyItem pitem = (pCafeFamilyItem)param->pfamily->flist->array[i];
		if ( pitem->ref != i ) continue;
		if ( param->maximumPvalues[i] > param->pvalue ) 
		{
			for( b = 0 ; b < nnodes ; b++ ) param->likelihoodRatios[b][i] = -1;
			continue;
		}
		cafe_family_set_size(param->pfamily,i, pcafe);
		double maxlh = __max(cafe_tree_likelihood(pcafe), param->pcafe->rfsize );
		for( b = 0 ; b < nnodes ; b++ )
		{
			pPhylogenyNode pnode = (pPhylogenyNode)pcafe->super.nlist->array[b];
			if ( tree_is_root( (pTree)pcafe ,(pTreeNode)pnode) ) 
			{
				param->likelihoodRatios[b][i] = -1;
				continue;
			}
			old_bl = pnode->branchlength;
			double** old_bd = ((pCafeNode)pnode)->bd;
			double prevlh = -1;
			double nextlh = maxlh;
			while( prevlh < nextlh )
			{
				prevlh = nextlh;
				pnode->branchlength += rint(pnode->branchlength * 0.15);
pthread_mutex_lock( &mutex_cafe_likelihood );
				((pCafeNode)pnode)->bd = birthdeath_cache_get_matrix(pcafe->pbdc_array, pnode->branchlength, ((pCafeNode)pnode)->lambda );
pthread_mutex_unlock( &mutex_cafe_likelihood);
				nextlh = __max(cafe_tree_likelihood(pcafe), param->pcafe->rfsize );
			}
			param->likelihoodRatios[b][i] = (prevlh == maxlh) ? 1 : 1 - chi2cdf( 2*(log(prevlh) - log(maxlh)), 1);
		//	param->likelihoodRatios[b][i] = (prevlh == maxlh) ? 1 : prevlh / maxlh ;
			pnode->branchlength = old_bl;
			((pCafeNode)pnode)->bd = old_bd;
		}
	}
	cafe_tree_free(pcafe);
	return (NULL);
}

void cafe_likelihood_ratio_test(pCafeParam param)
{
	cafe_log(param,"Running Likelihood Ratio Test....\n");

	pTree ptree = (pTree)param->pcafe;
	int nnodes = ptree->nlist->size;

	pLRTParam ptparam = (pLRTParam)memory_new(param->num_threads,sizeof(LRTParam));
	int nrows = param->pfamily->flist->size;

	param->likelihoodRatios = (double**)memory_new_2dim(nnodes,nrows,sizeof(double));

	int i, b;
	for ( i = 0 ; i < param->num_threads; i++)
	{
		ptparam[i].cafeparam = param;
		ptparam[i].from = i;
	}
	thread_run(param->num_threads, __cafe_likelihood_ratio_test_thread_func, ptparam, sizeof(LRTParam));
	for( i = 0 ; i < nrows ; i++ )
	{
		pCafeFamilyItem pitem = (pCafeFamilyItem)param->pfamily->flist->array[i];
		if ( pitem->ref == i ) continue;
		for( b = 0 ; b < nnodes ; b++ )
		{
			param->likelihoodRatios[b][i] = param->likelihoodRatios[b][pitem->ref];
		}
		continue;
	}
	memory_free(ptparam);					
	cafe_log( param , "Done : Likelihood Ratio test\n" );
}

/**************************************************************************
 * Likelihood ratio test with more than one lambda
**************************************************************************/

typedef struct
{
	pCafeParam cafeparam;
	int from;
	int* lambda;
	double* pvalues;
	lambda_func lfunc;
	pTree lambda_tree;
	int    num_lambdas;
}LRT2LParam;
typedef LRT2LParam* pLRT2LParam;

double** lambda_cache;
pBirthDeathCacheArray* PBDC;

pthread_mutex_t mutex_cafe_lh2 = PTHREAD_MUTEX_INITIALIZER;

double __cafe_lhr_get_likelihood_for_diff_lambdas(pCafeParam param, int idx, int t)
{
	param->branchlength_update_func(param,&t);
pthread_mutex_lock( &mutex_cafe_lh2 );
	if ( lambda_cache[t] == NULL )
	{
		param->lambda = NULL;
		cafe_best_lambda_by_fminsearch(param,param->num_lambdas);
		lambda_cache[t] = param->lambda;
		cafe_set_birthdeath_cache(param);
		PBDC[t] = param->pcafe->pbdc_array;
	}
	else
	{
		memcpy( param->lambda, lambda_cache[t], sizeof(double)*param->num_lambdas );
		param->lambda_set_func(param,param->lambda );
		param->pcafe->pbdc_array = PBDC[t];
		cafe_tree_set_birthdeath(param->pcafe);
	}
pthread_mutex_unlock( &mutex_cafe_lh2 );
	int i;
	cafe_family_set_size(param->pfamily,idx, param->pcafe);
	double mlh = __max(cafe_tree_likelihood(param->pcafe), param->pcafe->rfsize );
	pTree ptree = (pTree)param->pcafe;
	pArrayList nlist = ptree->nlist;
	for ( i = 0 ; i < nlist->size ; i++ )
	{
		pPhylogenyNode pnode = (pPhylogenyNode) nlist->array[i];
		pnode->branchlength = param->old_branchlength[i];
	}
	return mlh;
}

void* __cafe_lhr_for_diff_lambdas_thread(void* ptr)
{
	int i, j;
	pLRT2LParam plrt = (pLRT2LParam)ptr;
	pCafeParam param = plrt->cafeparam;
	pCafeTree pcafe = cafe_tree_copy(param->pcafe);
	pcafe->pbdc_array = param->pcafe->pbdc_array;

	pCafeParam cpy_param = cafe_copy_parameters(param);	
	cpy_param->num_lambdas = plrt->num_lambdas;
	cpy_param->lambda_tree = plrt->lambda_tree;
	cpy_param->lambda_set_func = plrt->lfunc;

	int fsize = param->pfamily->flist->size;
	for( i = plrt->from; i < fsize ; i+= param->num_threads )
	{ 
		pCafeFamilyItem pitem = (pCafeFamilyItem)param->pfamily->flist->array[i];
		if ( pitem->ref != i ) continue;
		cafe_family_set_size(param->pfamily,i, pcafe);
		double maxlh1 = __max(cafe_tree_likelihood(pcafe), pcafe->rfsize );
		double prev = -1;
		double next = __cafe_lhr_get_likelihood_for_diff_lambdas(cpy_param,i,0);
		for ( j = 1 ; prev < next ; j++ )
		{
			prev = next;
			next = __cafe_lhr_get_likelihood_for_diff_lambdas(cpy_param,i,j);
		}
		plrt->pvalues[i] = (prev == maxlh1 ) ? 1 :  2*(log(prev) - log(maxlh1));
		plrt->lambda[i] = j - 2;
	}
	cafe_free_copy_parameters(cpy_param);
	cafe_tree_free(pcafe);
	return (NULL);	
}

void cafe_lhr_report(pCafeParam param, double* pvalues, int* plambda )
{
	int i;
	int fsize = param->pfamily->flist->size;
	for ( i = 0 ; i < fsize ; i++ )
	{
		pCafeFamilyItem pitem = (pCafeFamilyItem)param->pfamily->flist->array[i];
		cafe_family_set_size(param->pfamily,i, param->pcafe);
		pString pstr = cafe_tree_string(param->pcafe);
		fprintf(param->fout, "%s\t%s\t", pitem->id, pstr->buf );		
		double* l = lambda_cache[plambda[i]];
		fprintf(param->fout, "(%d, %lf,%lf)\t%g\t%f\n", plambda[i], l[0],l[1], pvalues[i], pvalues[1] == 1 ? 1 : 1-chi2cdf(pvalues[i],1));
		string_free(pstr);
	}
}

void cafe_lhr_for_diff_lambdas(pCafeParam param, pTree lambda_tree2, int num_lambdas, lambda_func lfunc )
{
	cafe_log(param,"Running Likelihood Ratio Test 2....\n");
	int i;
	lambda_cache = (double**)memory_new(100,sizeof(double*));
	PBDC = (pBirthDeathCacheArray*)memory_new(100,sizeof(pBirthDeathCacheArray));
	for ( i = 0 ; i < 100 ; i++ )
	{
		lambda_cache[i] = NULL;
		PBDC[i] = NULL;
	}
	int old_numthreads = param->num_threads;
	param->num_threads = 1;
	int nrows = param->pfamily->flist->size;
	double* pvalues = (double*)memory_new_with_clear(nrows,sizeof(double));
	int* plambda = (int*)memory_new_with_clear(nrows, sizeof(int));

	pLRT2LParam ptparam = MEMORY_NEW( param->num_threads, LRT2LParam );
	for( i = 0 ; i < param->num_threads ; i++ )
	{
		ptparam[i].cafeparam = param;
		ptparam[i].from = i;
		ptparam[i].lambda = plambda;
		ptparam[i].pvalues = pvalues;
		ptparam[i].lfunc = lfunc;
		ptparam[i].lambda_tree = lambda_tree2;
		ptparam[i].num_lambdas = num_lambdas;
	}
	lambda_func old_func = param->lambda_set_func;
	param->lambda_set_func = cafe_lambda_set_default;
	thread_run(param->num_threads, __cafe_lhr_for_diff_lambdas_thread, ptparam, sizeof(LRT2LParam));
	param->lambda_set_func = old_func;

	int fsize = param->pfamily->flist->size;
	for ( i = 0 ; i < fsize ; i++ )
	{
		pCafeFamilyItem pitem = (pCafeFamilyItem)param->pfamily->flist->array[i];
		if ( pitem->ref == i ) continue;
		pvalues[i] = pvalues[ pitem->ref ];
		plambda[i] = plambda[pitem->ref];
	}

	cafe_lhr_report(param, pvalues, plambda );

	for ( i = 0 ; i < 100 ; i++ )
	{
		if ( lambda_cache[i] ) 
		{
			free(lambda_cache[i]);
			birthdeath_cache_array_free(PBDC[i]);
		}
	}
	memory_free(pvalues);
	memory_free(plambda);
	param->num_threads = old_numthreads;
	free(lambda_cache);
	free(PBDC);
}

/*************************************************************************
 * Deprecated
 *************************************************************************/

/**************************************************************************
 * Main
**************************************************************************/

void* cafe_run(void* ptr)
{
	pCafeParam param = (pCafeParam)ptr;
	if ( param->lambda == NULL )
	{
		cafe_best_lambda_by_fminsearch(param, param->num_lambdas);
	}
	else
	{
		param->lambda_set_func(param,param->lambda);
		pString pstr = cafe_tree_string_with_lambda(param->pcafe);
		cafe_log(param, "Lambda Value: %s\n", pstr->buf );
		string_free(pstr);
	}
	cafe_set_birthdeath_cache(param);
	pArrayList pCD = cafe_viterbi(param, NULL);
//	cafe_branch_cutting(param);
	cafe_likelihood_ratio_test(param);
	cafe_free_birthdeath_cache(param->pcafe);
	cafe_report(param, CAFE_REPORT_TEXT);
	arraylist_free(pCD, free);
	return (NULL);
}

/*******************************************************************************
 *	Cafe Parameter
 *******************************************************************************/

pCafeParam cafe_copy_parameters(pCafeParam psrc)
{
	pCafeParam param = (pCafeParam)memory_new(1, sizeof(CafeParam));
	memcpy( param, psrc, sizeof(CafeParam));
	param->lambda = NULL;
	param->num_lambdas = 0;
	param->pcafe = cafe_tree_copy(psrc->pcafe);
	param->branch_range = (int*)memory_new(psrc->blsize,sizeof(int));
	memcpy( param->branch_range, psrc->branch_range, psrc->blsize * sizeof(int));

	param->viterbiPvalues = NULL;
	param->expandRemainDecrease = NULL;
	param->viterbiNodeFamilysizes = NULL;
	param->maximumPvalues = NULL;
	param->averageExpansion = NULL;
	param->cutPvalues = NULL;

	return param;
}

void cafe_free_copy_parameters(pCafeParam param)
{
	memory_free(param->branch_range);	
	cafe_tree_free(param->pcafe);
	memory_free(param);
}

pCafeParam cafe_load_parameters(char* fconf)
{
	int i, j;
	pCafeParam param = (pCafeParam)memory_new_with_clear(1, sizeof(CafeParam));
	FILE* fp = fopen(fconf,"r");
	if ( fp == NULL )
	{
		print_error(__FILE__,(char*)__FUNCTION__,__LINE__,
				    "Cannot open parameter file: %s ", fconf);
	}
	param->lambda = NULL;
	param->num_lambdas = 0;
	param->family_sizes[0] = 0;
	param->family_sizes[1] = 1;
	param->rootfamily_sizes[0] = 0;
	param->rootfamily_sizes[1] = 1;
	param->num_threads = 3;

	char buf[BUFSIZ];
	int line  = 1;
	pString pstr_tree = NULL;

	while( fgets(buf,BUFSIZ,fp) )	
	{
		char *name, *value;

		string_pchar_chomp(buf);
		if ( strlen(buf) == 0 ) continue;
		if ( buf[0] == '#' ) continue;
		name = buf;
		value = index(buf,':');
		if ( value == NULL )
		{
			print_error(__FILE__,(char*)__FUNCTION__,__LINE__,
					    "Wrong configuration in line %d: %s", line, buf);		
		}
		*value++ = '\0';
		string_pchar_chomp(name);					
		string_pchar_chomp(value);					
			
		if ( strncasecmp(name,"data",4) == 0 )
		{
			param->str_fdata = string_new_with_string(value);
		}
		else if ( strncasecmp(name,"log",3 ) == 0 )
		{
			param->str_log = string_new_with_string(value);
		}
		else if ( strncasecmp(name,"destination",10) == 0 )
		{
			if ( (param->fout = fopen(value,"w")) == NULL )		
			{
				print_error(__FILE__,(char*)__FUNCTION__,__LINE__,
					         "There is no such file, %s", value );
			}
			param->str_fout = string_new_with_string(value);
		}
		else if ( strncasecmp(name,"tree",4 ) == 0 )
		{
			pstr_tree = string_new_with_string(value);
			for ( i = 0 ; i < pstr_tree->length ; i++ )
			{
				if ( pstr_tree->buf[i] == ' ' ) pstr_tree->buf[i] = ',';
			}
		}
		else if ( strncasecmp(name,"lambda",6 ) == 0 )
		{
			pArrayList pal = string_pchar_space_split(value);
			param->num_lambdas = pal->size;
			param->lambda = MEMORY_NEW( pal->size, double);
			for( i = 0 ; i < pal->size ; i++ )
			{
				sscanf((char*)pal->array[i], "%lf", &param->lambda[i] );
			}
		}
		else if ( strncasecmp(name,"p-value",7) == 0 )
		{
			sscanf(value,"%lf", &param->pvalue );
		}
		else if ( strncasecmp(name,"number",6) == 0 )
		{
			sscanf(value,"%d", &param->num_random_samples );
		}
		else if ( strncasecmp(name,"thread",6) == 0 )
		{
			sscanf(value,"%d", &param->num_threads );
		}
		line++;
	}

	if ( param->str_fout == NULL )
	{
		param->fout = stdout;
	}

	if ( param->str_log == NULL )
	{
		param->str_log = string_new_with_string("cafe.log");
	}
	if ( string_pchar_cmp_ignore_case(param->str_log->buf,"stderr") )
	{
		param->flog = stderr;
	}
	else if ( (param->flog = fopen(param->str_log->buf,"a")) == NULL )
	{
		print_error(__FILE__,(char*)__FUNCTION__,__LINE__, "Cannot open cafe.log" );
	}

	param->pfamily = cafe_family_new( param->str_fdata->buf , 1);
	if ( param->pfamily == NULL ) 
	{
		fclose(fp);
		return NULL;
	}

	param->rootfamily_sizes[0] = 1;
	param->rootfamily_sizes[1] = rint(param->pfamily->max_size * 1.25);
	param->family_sizes[1] = param->pfamily->max_size + MAX(50,param->pfamily->max_size/5);

	param->pcafe = cafe_tree_new(pstr_tree->buf, param->family_sizes, 
				                 param->rootfamily_sizes, param->lambda ? param->lambda[0] : 0 );
	pTree ptree = (pTree)param->pcafe;

	cafe_family_set_species_index(param->pfamily, param->pcafe);

	param->blsize = ptree->nlist->size - 1;
	param->branch_range = (int*)memory_new(param->blsize, sizeof(int));
	param->old_branchlength = (int*)memory_new(param->blsize, sizeof(int));

	param->ML = (double*)memory_new( param->pfamily->flist->size, sizeof(double));

	i = 0;
	for( j = 0 ; j < ptree->nlist->size; j++ )
	{
		pPhylogenyNode pnode = (pPhylogenyNode)ptree->nlist->array[j];
		if ( pnode->branchlength > 0 )
		{
			param->branch_range[i++] = pnode->branchlength;
		}
	}
	qsort(param->branch_range, param->blsize, sizeof(int), __cmp_int );


	if ( i != ptree->nlist->size - 1 )
	{
		print_error(__FILE__,(char*)__FUNCTION__,__LINE__,
				    "Expected number of Branch length is %d\n", ptree->nlist->size - 1 );
	}
	param->lambda_set_func = cafe_lambda_set_default;

	fclose(fp);

	cafe_log(param, "===================================================\n");
	cafe_log(param, "BEGIN: %s", get_current_time() );
	cafe_log(param, "Family information: %s\n", param->str_fdata->buf );
	cafe_log(param, "Report: %s\n", param->str_fout ? param->str_fout->buf : "stdout" );
	cafe_log(param, "Log: %s\n", param->str_log->buf );
	pString pstr = cafe_tree_string(param->pcafe);
	cafe_log(param, "Tree: %s\n", pstr->buf );
	string_free(pstr);
	cafe_log(param, "The number of families is %d\n", param->pfamily->flist->size );
	cafe_log(param, "Root Family size : %d ~ %d\n", param->rootfamily_sizes[0], param->rootfamily_sizes[1]);
	cafe_log(param, "Family size : %d ~ %d\n", param->family_sizes[0], param->family_sizes[1]);

	chooseln_cache_init( MAX(param->family_sizes[1], param->rootfamily_sizes[1]) );
	printf("Done\n");
	
	return param;
}

void cafe_save_parameters(pCafeParam param, char* file)
{
	FILE* fp = fopen(file, "w");
	if ( fp == NULL )
	{
		print_error(__FILE__,(char*)__FUNCTION__,__LINE__,
				    "Cannot open file: %s ", file );
	}
	fprintf(fp, "Data file: %s\n", param->str_fdata->buf);
	fprintf(fp, "Destination file: %s\n", param->str_fout ? param->str_fout->buf : "stdout" );
	fprintf(fp, "Log file: %s\n", param->str_log->buf);

	pString tree = phylogeny_string((pTree)param->pcafe, NULL);
	fprintf(fp, "Tree structure: %s\n", tree->buf );
	string_free(tree);

	if ( param->lambda > 0 )
	{
		fprintf(fp, "Lambda: %d\n", param->num_lambdas);
	}
	else
	{
		fprintf(fp, "Lambda (value or range): %lf:%lf:%lf\n", 
				  param->lambda_values[0],
				  param->lambda_values[1] - param->lambda_values[0],
				  param->lambda_values[param->num_lambdas-1]);
	}
	fprintf(fp,"P-value threshold: %lf\n", param->pvalue);
	fprintf(fp,"Number of random samples: %d\n", param->num_random_samples);
	fprintf(fp,"Family sizes: %d:%d\n", param->family_sizes[0], param->family_sizes[1]);
	fprintf(fp,"Thread number: %d\n", param->num_threads);
	fclose(fp);
}

void cafe_free_parameters(pCafeParam param)
{
	if ( param->str_fdata ) string_free(param->str_fdata );
	if ( param->str_fout ) 
	{
		fclose(param->fout);
		string_free(param->str_fout );
	}
	if ( param->str_log ) string_free(param->str_log);

	if ( param->pcafe )
	{
		int nnodes = ((pTree)param->pcafe)->nlist->size;
		int num = (nnodes - 1 )/2;
		if ( param->viterbiPvalues ) 
		{
			memory_free_2dim((void**)param->viterbiPvalues,num,0,NULL);
			memory_free_2dim((void**)param->expandRemainDecrease,3,0,NULL);
			memory_free_2dim((void**)param->viterbiNodeFamilysizes,num, 0, NULL );
			memory_free(param->maximumPvalues);
			memory_free(param->averageExpansion);
		}
		if ( param->cutPvalues )
		{
			memory_free_2dim((void**)param->cutPvalues,nnodes,0,NULL );
		}
		if ( param->likelihoodRatios )
		{
			memory_free_2dim((void**)param->likelihoodRatios,nnodes,0,NULL);
		}
		cafe_tree_free(param->pcafe);
		free(param->branch_range);
		free(param->old_branchlength );
	}
	if ( param->pfamily ) cafe_family_free(param->pfamily);

	//free(param->lambda_values);
	//free(param->lambda_scores);	
	//
	if ( param->ML ) free(param->ML);
	
	cafe_log(param, "END: %s", get_current_time() );

	if ( !string_pchar_cmp_ignore_case(param->str_log->buf,"stderr") )
	{
		fclose(param->flog);
	}
	free(param);
	chooseln_cache_free();
}
