#include<cafe_shell.h>
#include<mathfunc.h>

int cafe_cmd_tree(int argc, char* argv[]);
int cafe_cmd_branchlength(int argc, char* argv[]);
int cafe_cmd_echo(int argc, char* argv[]);
int cafe_cmd_family(int argc, char* argv[]);
int cafe_cmd_list(int argc, char* argv[]);
int cafe_cmd_date(int argc, char* argv[]);
int cafe_cmd_viterbi(int argc, char* argv[]);
int cafe_cmd_print_param(int argc, char* argv[]);
int cafe_cmd_load(int argc, char* argv[]);
int cafe_cmd_save(int argc, char* argv[]);
int cafe_cmd_lambda(int argc, char* argv[]);
int cafe_cmd_version(int argc, char* argv[]);
int cafe_cmd_pvalue(int argc, char* argv[]);
int cafe_cmd_score(int argc, char* argv[]);
int cafe_cmd_report(int argc, char* argv[] );
int cafe_cmd_log(int argc, char* argv[] );
int cafe_cmd_cgi(int argc, char* argv[] );
int cafe_cmd_retrieve(int argc, char* argv[] );
int cafe_cmd_gainloss(int argc, char* argv[] );
int cafe_cmd_sim_extinct(int argc, char* argv[]);
int cafe_cmd_extinct(int argc, char* argv[]);
int cafe_cmd_generate_random_family(int argc, char* argv[]);
int cafe_cmd_lh_test(int argc, char* argv[]);
int __cafe_cmd_log(int argc ,char* argv[] );

pCafeParam cafe_param;
pTree tmp_lambda_tree;
pArrayList cafe_pCD;

#ifndef ASSERT_IF
	#define ASSERT_IF(a,b)	if ( a ) { fprintf(stderr,b); return -1; }
#endif

CafeShellCommand cafe_cmd[]  =
{
	{ "?", cafe_cmd_list },
	{ "branchlength", cafe_cmd_branchlength },
	{ "cgi", cafe_cmd_cgi }, 
	{ "date", cafe_cmd_date },
	{ "echo", cafe_cmd_echo },
	{ "extinct", cafe_cmd_extinct },
	{ "exit", cafe_cmd_exit },
	{ "family", cafe_cmd_family },
	{ "gainloss", cafe_cmd_gainloss },
	{ "genfamily", cafe_cmd_generate_random_family },
	{ "info", cafe_cmd_print_param },
	{ "lambda", cafe_cmd_lambda },
	{ "lhtest", cafe_cmd_lh_test },
	{ "load", cafe_cmd_load },
	{ "log", cafe_cmd_log },
	{ "pvalue", cafe_cmd_pvalue },
	{ "quit", cafe_cmd_exit },
	{ "report", cafe_cmd_report },
	{ "retrieve", cafe_cmd_retrieve },
	{ "save", cafe_cmd_save},
	{ "score", cafe_cmd_score },
	{ "simextinct", cafe_cmd_sim_extinct },
	{ "source", cafe_cmd_source },
	{ "tree", cafe_cmd_tree },
	{ "version", cafe_cmd_version },
	{ "viterbi", cafe_cmd_viterbi},
	{ NULL, NULL }
};


typedef struct
{
	char* opt;
	int   argc; 
	char** argv;
}Argument;

typedef Argument* pArgument;

pArrayList cafe_shell_build_argument(int argc, char* argv[])
{
	int i, j;
	pArrayList pal = arraylist_new(20);
	for ( i = 1 ; i < argc ; i++ )
	{
		if ( argv[i][0] == '-' )
		{
			pArgument parg = MEMORY_NEW(1,Argument);
			parg->argc = 0;
			parg->opt = argv[i];
			for ( j = i+1; j < argc; j++ )
			{
				if ( argv[j][0] == '-' ) break;
				parg->argc++;
			}
			parg->argv = parg->argc ? &argv[i+1]  : NULL;
			arraylist_add( pal, parg );
			i = j - 1;
		}
	}
	return pal;
}


pArgument cafe_shell_get_argument(char* opt, pArrayList pal)
{
	int i;
	for ( i = 0 ; i < pal->size; i++ )
	{
		pArgument parg = (pArgument)pal->array[i];
		if ( !strcasecmp(parg->opt, opt) )
		{
			return parg;
		}
	}
	return NULL;
}


void cafe_shell_set_lambda(pCafeParam param, double* lambda);
void cafe_shell_update_branchlength(pCafeParam param, int* t );

void cafe_shell_clear_interal_data()
{
	if ( cafe_param->viterbiPvalues )
	{
		int nnodes = ((pTree)cafe_param->pcafe)->nlist->size;
		int num = (nnodes - 1 )/2;
		memory_free_2dim((void**)cafe_param->viterbiPvalues,num,0,NULL);
		memory_free_2dim((void**)cafe_param->expandRemainDecrease,3,0,NULL);
		memory_free_2dim((void**)cafe_param->viterbiNodeFamilysizes,num, 0, NULL);
		memory_free(cafe_param->averageExpansion);
		if ( cafe_param->maximumPvalues )
		{
			memory_free(cafe_param->maximumPvalues);
		}
	}
	if ( cafe_param->cutPvalues )
	{
		int nnodes = ((pTree)cafe_param->pcafe)->nlist->size;
		memory_free_2dim((void**)cafe_param->cutPvalues,nnodes,0,NULL);
	}
	cafe_param->viterbiPvalues = NULL;
	cafe_param->expandRemainDecrease = NULL;
	cafe_param->viterbiNodeFamilysizes = NULL;
	cafe_param->cutPvalues = NULL;
	cafe_param->maximumPvalues = NULL;
}

void cafe_shell_clear_param(int btree_skip)
{
	if ( cafe_param->str_fdata ) 
	{
		string_free( cafe_param->str_fdata );
		cafe_param->str_fdata = NULL;
	}
	if ( cafe_param->ML )
	{
		memory_free(cafe_param->ML);
		cafe_param->ML = NULL;
	}
	cafe_shell_clear_interal_data();
	if ( !btree_skip && cafe_param->pcafe ) 
	{
		if ( cafe_param->pcafe->pbdc_array )
		{
			birthdeath_cache_array_free(cafe_param->pcafe->pbdc_array);
		}
		cafe_tree_free(cafe_param->pcafe);
		free( cafe_param->branch_range );
		free( cafe_param->old_branchlength );
		cafe_param->pcafe = NULL;
		cafe_param->branch_range = NULL;
		cafe_param->old_branchlength = NULL;
	}
	if ( cafe_param->pfamily ) 
	{
		cafe_family_free(cafe_param->pfamily);
		cafe_param->pfamily = NULL;
	}
	if ( !btree_skip && cafe_param->lambda ) 
	{
		memory_free( cafe_param->lambda );
		cafe_param->lambda = NULL;
	}
	if ( !btree_skip && cafe_param->lambda_tree ) 
	{
		phylogeny_free(cafe_param->lambda_tree);
		cafe_param->lambda_tree = NULL;
	}
	cafe_param->rootfamily_sizes[0] = 1;
	cafe_param->rootfamily_sizes[1] = 1;
	cafe_param->family_sizes[0] = 0;
	cafe_param->family_sizes[1] = 1;
	cafe_param->lambda_set_func = cafe_shell_set_lambda;
	cafe_param->branchlength_update_func = cafe_shell_update_branchlength;
	cafe_param->num_threads = 8;
	cafe_param->num_random_samples = 1000;
	cafe_param->pvalue = 0.01;
	cafe_param->bl_augment = 0.5;
}

void cafe_shell_set_sizes()
{
	pCafeTree pcafe = cafe_param->pcafe;
	pcafe->rootfamilysizes[0] = cafe_param->rootfamily_sizes[0];
	pcafe->rootfamilysizes[1] = cafe_param->rootfamily_sizes[1];
	pcafe->familysizes[0] = cafe_param->family_sizes[0];
	pcafe->familysizes[1] = cafe_param->family_sizes[1];
	cafe_tree_set_parameters(pcafe, cafe_param->family_sizes, cafe_param->rootfamily_sizes, 0 );
}

void cafe_shell_prompt(char* prompt, char* format, ... )
{
	va_list ap;
	va_start(ap, format);
	printf("%s ", prompt);
	vscanf( format, ap );
	va_end(ap);
}


int cafe_shell_dispatch_command(char* cmd)
{
	pArrayList parg = string_pchar_space_split(cmd);

	int i;
	int rtn = 0;
	
	if ( parg->size > 0 )
	{
		if ( ((char*)parg->array[0])[0] == '#' ) return 0;
	}

	if ( parg->size != 0 ) 
	{
		rtn = CAFE_SHELL_NO_COMMAND;
		for ( i = 0 ; cafe_cmd[i].command ; i++ )
		{
			if ( strcasecmp( (char*)parg->array[0],  cafe_cmd[i].command) == 0 )
			{
				rtn = cafe_cmd[i].func( parg->size, (char**)parg->array );
				break;
			}
		}
		if ( rtn == CAFE_SHELL_NO_COMMAND )
		{
			fprintf(stderr, "cafe: %s: command not found\n", (char*)parg->array[0]);		
		}
	}
	arraylist_free(parg,NULL);
	return rtn;
}

int cafe_shell_dispatch_commandf(char* format, ... )
{
	va_list ap;
	char buf[500];
	va_start(ap, format);
	vsprintf(buf, format, ap );
	int r = cafe_shell_dispatch_command(buf);
	va_end(ap);
	return r;
}

void cafe_shell_set_lambda(pCafeParam param, double* lambda)
{
	int i;
	pArrayList nlist = param->pcafe->super.nlist;
	pTree tlambda = param->lambda_tree;
	if ( tlambda == NULL )
	{
		for ( i = 0 ; i < nlist->size ; i++ )
		{
			pCafeNode pcnode = (pCafeNode)nlist->array[i];
			pcnode->lambda = lambda[0];
		}
	}
	else
	{
		pArrayList lambda_nlist = tlambda->nlist;		
		for ( i = 0 ; i < nlist->size ; i++ )
		{
			pPhylogenyNode pnode = (pPhylogenyNode)lambda_nlist->array[i];
			pCafeNode pcnode = (pCafeNode)nlist->array[i];
			pcnode->lambda = lambda[pnode->taxaid];
		}
	}
}

void cafe_shell_update_branchlength(pCafeParam param, int* t)
// t[0] is for lambda[1]
// t[1] is for lambda[2]
{
	int i;
	if ( param->lambda_tree )
	{
		pArrayList nlist = param->pcafe->super.nlist;
		pArrayList lambda_nlist = param->lambda_tree->nlist;		
		for ( i = 0 ; i < nlist->size ; i++ )
		{
			pPhylogenyNode lambda_pnode = (pPhylogenyNode)lambda_nlist->array[i];
			pPhylogenyNode pnode = (pPhylogenyNode)nlist->array[i];
			param->old_branchlength[i] = pnode->branchlength;
			if (  lambda_pnode->taxaid > 0 ) 
			{
				pnode->branchlength += pnode->branchlength * param->bl_augment 
					                   * t[lambda_pnode->taxaid-1];		
			}
		}
	}
}

int cafe_shell_parse_familysize(int argc, char* argv[])
{
	int i;
	int max = 0;
	pArrayList nlist = cafe_param->pcafe->super.nlist;
	if ( argc != (nlist->size/2+1) )
	{
		fprintf( stderr, "ERROR: There are %d species\n", nlist->size/2 + 1);
		return -1;
	}
	for ( i = 0; i < argc ; i++ )
	{
		int size = -1;
		sscanf( argv[i],"%d", &size );
		ASSERT_IF(size<0, "ERROR: You must input integers greater than or equal to 0\n");
		pCafeNode pnode = (pCafeNode)nlist->array[i*2];
		pnode->familysize = size;
		if ( size > max ) max = size;
	}
	return max;
}

int cafe_shell_set_familysize()
{
	int i;
	int max = 0;
	char buf[1024];

	ASSERT_IF( cafe_param->pcafe == NULL, "You did not specify tree: command 'tree'\n" );

	pArrayList nlist = cafe_param->pcafe->super.nlist;
	for ( i = 0; i < nlist->size ; i+=2 )
	{
		pCafeNode pnode = (pCafeNode)nlist->array[i];
		sprintf(buf, "%s: ", pnode->super.name );
		int size = -1;
		cafe_shell_prompt( buf , "%d", &size  );
		if ( size < 0 )
		{ 
			fprintf( stderr, "ERROR: You put wrong data, you must enter an integer greater than or equal to 0\n");
			cafe_shell_prompt( "Retry? [Y|N] ", "%s", buf);
			if ( buf[0] != 'Y' && buf[0] != 'y' ) return -1; 
			i -= 2;
		}
		else
		{
			pnode->familysize = size;
			if ( size > max ) max = size;
		}
	}
	return max;
}

int cafe_shell_parse_branchlength(int argc, char* argv[])
{
	int i;


	pArrayList nlist = cafe_param->pcafe->super.nlist;

	if ( argc != nlist->size )
	{
		fprintf( stderr, "ERROR: There are %d branches including the empty branch of root\n", nlist->size );
		return -1;
	}
	for ( i = 0; i < argc ; i++ )
	{
		int size = -1;
		pPhylogenyNode pnode = (pPhylogenyNode)nlist->array[i];
		sscanf( argv[i],"%d", &size );
		if ( size > 0 )
		{ 
			pnode->branchlength = size;
		}
		else
		{
			fprintf(stderr,"ERROR: the branch length of node %d is not changed\n", i);
		}
	}
	if ( cafe_param->pcafe->pbdc_array ) cafe_tree_set_birthdeath(cafe_param->pcafe);
	return 0;
}

int cafe_shell_set_branchlength()
{
	int i;
	char buf[1024];

	pArrayList nlist = cafe_param->pcafe->super.nlist;
	for ( i = 0; i < nlist->size ; i++ )
	{
		pPhylogenyNode pnode = (pPhylogenyNode)nlist->array[i];
		if ( tree_is_root( (pTree)cafe_param->pcafe, (pTreeNode)pnode) ) continue;
		printf("%d[%d]: ", i, (int)pnode->branchlength );
		fgets(buf,1024,stdin);
		int len = strlen(buf);
		buf[--len] = '\0';
		if ( len != 0 )
		{
			int size = -1;
			sscanf( buf, "%d", &size );
			if ( size > 0 )
			{ 
				pnode->branchlength = size;
			}
			else
			{
				fprintf(stderr,"ERROR: the branch length of node %d is not changed\n", i);
			}
		}
	}
	if ( cafe_param->pcafe->pbdc_array ) cafe_tree_set_birthdeath(cafe_param->pcafe);
	return 0;
}


double* cafe_shell_likelihood(int max)
{
	int rfmax = max+ MAX(50,max/4);
	int fmax = max + MAX(50,max/5);
	pCafeTree pcafe = cafe_param->pcafe;

	if ( pcafe->pbdc_array == NULL || pcafe->pbdc_array->maxFamilysize < MAX(rfmax, fmax) )
	{
		cafe_param->rootfamily_sizes[1] = rfmax;
		cafe_param->family_sizes[1] = fmax;
		cafe_tree_set_parameters(pcafe, cafe_param->family_sizes, cafe_param->rootfamily_sizes, 0 );
		if ( pcafe->pbdc_array )
		{
			cafe_resize_birthdeath_cache(cafe_param);
		}
		else
		{
			cafe_set_birthdeath_cache(cafe_param);
		}
	}
	else
	{
		pcafe->rootfamilysizes[0] = 1;
		pcafe->rootfamilysizes[1] = rfmax;
		pcafe->familysizes[1] = fmax;
		pcafe->rfsize = pcafe->rootfamilysizes[1] - pcafe->rootfamilysizes[0] + 1;
	}
	return cafe_tree_likelihood(pcafe);
}

void cafe_shell_init()
{
	cafe_param = MEMORY_NEW_WITH_CLEAR(1,CafeParam);
	cafe_param->rootfamily_sizes[0] = 1;
	cafe_param->rootfamily_sizes[1] = 1;
	cafe_param->family_sizes[0] = 0;
	cafe_param->family_sizes[1] = 1;
	cafe_param->lambda_set_func = cafe_shell_set_lambda;
	cafe_param->flog = stdout;
	cafe_param->num_threads = 8;
	cafe_param->num_random_samples = 1000;
	cafe_param->bl_augment = 0.5;
	cafe_param->pvalue = 0.01;
}

int cafe_cmd_echo(int argc, char* argv[])
{
	if ( argc == 1 ) return 0;
	int i;
	for ( i = 1 ; i < argc ; i++ )
	{
		cafe_log( cafe_param, " %s", argv[i] );
	}
	cafe_log( cafe_param, "\n" );
	return 0;
}

int cafe_cmd_date(int argc, char* argv[])
{
	cafe_log( cafe_param, "%s", get_current_time() );
	return 0;
}

int cafe_cmd_list(int argc, char* argv[])
{
	int i;
	for ( i = 0 ; cafe_cmd[i].command ; i++ )
	{
		printf("%s\n", cafe_cmd[i].command );
	}
	return 0;
}


int cafe_cmd_tree(int argc, char* argv[])
{
	char buf[BUFSIZ];
	if ( argc == 1 )
	{
		printf("Newick: ");
		fgets(buf,BUFSIZ,stdin);
	}
	else
	{
		string_pchar_join(buf,NULL,argc-1, &argv[1]);
	}
	if ( cafe_param->pcafe ) 
	{
		if ( cafe_param->pcafe->pbdc_array )
		{
			cafe_free_birthdeath_cache(cafe_param->pcafe);
		}
		cafe_tree_free(cafe_param->pcafe);
		free( cafe_param->branch_range );
		free( cafe_param->old_branchlength );
	}
	cafe_param->pcafe = cafe_tree_new(buf, cafe_param->family_sizes, 
			                          cafe_param->rootfamily_sizes, 0 );
	cafe_param->blsize = cafe_param->pcafe->super.nlist->size - 1;
	cafe_param->branch_range = (int*)memory_new(cafe_param->blsize, sizeof(int));
	cafe_param->old_branchlength = (int*)memory_new(cafe_param->blsize, sizeof(int));
	pTree ptree = (pTree)cafe_param->pcafe;
	int i = 0, j;
	for( j = 0 ; j < ptree->nlist->size; j++ )
	{
		pPhylogenyNode pnode = (pPhylogenyNode)ptree->nlist->array[j];
		if ( pnode->branchlength > 0 )
		{
			cafe_param->branch_range[i++] = (int)pnode->branchlength;
		}
	}
	qsort(cafe_param->branch_range, cafe_param->blsize, sizeof(int), __cmp_int );
	if ( i != ptree->nlist->size - 1 )
	{
		fprintf(stderr, "Expected number of Branch length is %d\n", ptree->nlist->size - 1 );
		cafe_tree_free(cafe_param->pcafe);
		memory_free( cafe_param->branch_range);
		cafe_param->pcafe = NULL;
		cafe_param->branch_range = NULL;
		return -1;
	}
	cafe_tree_string_print(cafe_param->pcafe);
	if ( cafe_param->pfamily )
	{
		cafe_family_set_species_index(cafe_param->pfamily, cafe_param->pcafe);
	}
	return 0;
}

void phylogeny_lambda_parse_func(pTree ptree, pTreeNode ptnode)
{
	pPhylogenyNode pnode = (pPhylogenyNode)ptnode;
	sscanf( pnode->name, "%d", &pnode->taxaid );	
	pnode->taxaid--;
}

void __cafe_cmd_lambda_distribution(pArgument parg, FILE* fp)
{
	double** range = MEMORY_NEW_2DIM( parg->argc, 3, double );
	int j;
	for ( j = 0 ;  j < parg->argc ; j++ )
	{
		sscanf( parg->argv[j],"%lf:%lf:%lf", &range[j][0], &range[j][1], &range[j][2] );
		cafe_log(cafe_param,"%dst Distribution: %lf : %lf : %lf\n", j+1, range[j][0], range[j][1], range[j][2] );
	}
	cafe_param->num_lambdas = parg->argc;
	pGMatrix pgm = cafe_lambda_distribution(cafe_param,parg->argc,range );
	if ( fp )
	{
		int k;
		int* idx = MEMORY_NEW( parg->argc, int );
		for ( j = 0 ; j < pgm->num_elements; j++ )
		{
			gmatrix_dim_index(pgm,j,idx);							
			fprintf( fp, "%lf",  idx[0] * range[0][1] + range[0][0] );
			for ( k = 1 ; k < parg->argc; k++ )
			{
				fprintf( fp, "\t%lf",  idx[k] * range[k][1] + range[k][0] );
			}
			fprintf( fp, "\t%lf\n", gmatrix_double_get_with_index(pgm,j) ) ;
		}
		fclose(fp);
		memory_free(idx);
	}
	memory_free_2dim( (void**)range, parg->argc, 0, NULL );
	gmatrix_free(pgm);
}

int __cafe_cmd_lambda_tree(pArgument parg)
{
	int idx = 1;
	pTree ptree;
	if ( parg->argc == 2 )
	{
		sscanf( parg->argv[0], "%d", &idx );
		ptree = phylogeny_load_from_string(parg->argv[1], tree_new, phylogeny_new_empty_node, phylogeny_lambda_parse_func );
	}
	else
	{
		ptree = phylogeny_load_from_string(parg->argv[0], tree_new, phylogeny_new_empty_node, phylogeny_lambda_parse_func );
	}
	tree_build_node_list(ptree);
	if ( ptree->nlist->size != cafe_param->pcafe->super.nlist->size )
	{
		fprintf(stderr, "Lambda has a different topology from the tree\n");
		return -1;
	}
	if ( idx == 2 ) 
	{
		if ( tmp_lambda_tree ) phylogeny_free(tmp_lambda_tree);
		tmp_lambda_tree = ptree;
		return 1;
	}
	else 
	{
		if ( cafe_param->lambda_tree ) phylogeny_free(cafe_param->lambda_tree);
		cafe_param->lambda_tree = ptree;
		int j, k, m;
		pArrayList nlist = (pArrayList)cafe_param->lambda_tree->nlist;
		memset( cafe_param->old_branchlength, 0, sizeof(int) * cafe_param->blsize );
		for ( j = k = 0 ; j < nlist->size ; j++ )
		{
			int bl = ((pPhylogenyNode)nlist->array[j])->taxaid;
			if ( bl < 0 ) continue;
			for ( m = 0 ; m < k ; m++ )
			{
				if ( cafe_param->old_branchlength[m] == bl ) break;
			}
			if ( m == k ) cafe_param->old_branchlength[k++] = bl;
		}
		cafe_param->num_lambdas = k;
		printf("The number of lambdas is %d\n", k );
	}
	return 0;
}


int cafe_cmd_lambda(int argc, char* argv[])
{
	int i,j;
	ASSERT_IF( cafe_param->pfamily == NULL, "ERROR(lambda): You must load family data first: command 'load'\n");
	ASSERT_IF( cafe_param->pcafe == NULL, "ERROR(lambda): You did not specify tree: command 'tree'\n" );

	pCafeTree pcafe = cafe_param->pcafe;
	pArrayList pargs = cafe_shell_build_argument(argc,argv);
	cafe_param->num_lambdas = -1;
	int bdone = 0;
	int bsearch = 0;
	int bprint = 0;
	int beach = 0;
	int blambda = 0;
	double vlambda = 0;
	pArgument pdist = NULL;
	pArgument pout = NULL;

	FILE* fpout = stdout;
	FILE* fmpout = NULL;
	FILE* fhttp = NULL;
	char name[1024];

	for ( i = 0 ; i < pargs->size ; i++ )
	{
		pArgument parg = (pArgument)pargs->array[i];
		if ( !strcmp( parg->opt, "-t") )
		{
			if ( (bdone = __cafe_cmd_lambda_tree(parg)) ) 
			{
				bdone = 1;
			}
			pString pstr = phylogeny_string(cafe_param->lambda_tree,NULL);
			printf("Lambda Tree: %s\n", pstr->buf );
			string_free(pstr);
		}
		else if ( !strcmp( parg->opt, "-v") )
		{
			sscanf( parg->argv[0], "%lf", &vlambda );	
			blambda = 2;
		}
		else if ( !strcmp( parg->opt, "-l") )
		{
			if ( parg->argc != cafe_param->num_lambdas )
			{
				if ( cafe_param->lambda_tree )
				{
					phylogeny_free(cafe_param->lambda_tree);
					cafe_param->lambda_tree = NULL;
				}
			}
			if ( cafe_param->lambda ) memory_free(cafe_param->lambda);
			cafe_param->num_lambdas = parg->argc;
			cafe_param->lambda = MEMORY_NEW(cafe_param->num_lambdas, double );
			for ( j = 0 ; j < parg->argc ; j++ )
			{
			 	sscanf( parg->argv[j], "%lf", &cafe_param->lambda[j] );
			}
			blambda = 1;
		}
		// Search for whole family 
		else if ( !strcmp( parg->opt, "-s" ) )
		{
			if ( cafe_param->num_lambdas == -1 )
			{
				cafe_param->num_lambdas = 1;
			}
			bsearch = 1;
		}
		// Print
		else if ( !strcmp( parg->opt, "-p" ) )
		{
			bprint = 1;	
		}
		// Lambda Distribution
		else if ( !strcmp( parg->opt, "-r" ) )
		{
			pdist = parg;
			pout = cafe_shell_get_argument("-o", pargs);
		}
		// Search for each family 
		else if ( !strcmp( parg->opt, "-e" ) )
		{
			pout = cafe_shell_get_argument("-o", pargs);
			if ( pout )
			{
				strcpy(name, pout->argv[0] ); strcat(name, ".lambda" );
				if ( (fpout = fopen( name,"w"))  == NULL )
				{
					fprintf( stderr, "Cannot open file: %s\n", name );
					return -1;
				}
				strcpy(name, pout->argv[0] ); strcat(name, ".mp" );
				if ( (fmpout = fopen( name,"w"))  == NULL )
				{
					fprintf( stderr, "Cannot open file: %s\n", name );
					fclose(fpout);
					return -1;
				}
				strcpy(name, pout->argv[0] ); strcat(name, ".html" );
				if ( (fhttp = fopen( name,"w"))  == NULL )
				{
					fprintf( stderr, "Cannot open file: %s\n", name );
					fclose(fpout);
					fclose(fmpout);
					return -1;
				}
				strcpy(name, pout->argv[0] );
			}
			beach = 1;	
		}
	}

	if ( blambda == 2 && vlambda > 0 )
	{
		if ( cafe_param->lambda ) memory_free(cafe_param->lambda);
		cafe_param->num_lambdas = cafe_param->num_lambdas < 1 ? 1 : cafe_param->num_lambdas;
		cafe_param->lambda = MEMORY_NEW( cafe_param->num_lambdas, double );
		for( j = 0 ; j < cafe_param->num_lambdas; j++ )
		{
			cafe_param->lambda[j] = vlambda;
		}
	}

	if ( pdist )
	{
		FILE* fp = NULL;
		if( pout && (fp = fopen(pout->argv[0],"w") ) == NULL )
		{
			fprintf( stderr, "ERROR(lambda): Cannot open file: %s\n", pout->argv[0] );
			return -1;
		}
		__cafe_cmd_lambda_distribution(pdist, fp);
		bdone = 1;
		fclose(fp);
	}

	arraylist_free( pargs, free );

	if ( bdone ) 
	{
		if ( bdone ) return 0;
	}
	
	ASSERT_IF(cafe_param->lambda_tree == NULL && cafe_param->num_lambdas != 1 && !bprint, "ERROR(lambda): You did not specify lambda information\n");

	if ( bsearch == 1 )
	{
		if ( cafe_param->num_lambdas > 1 && cafe_param->lambda_tree == NULL )
		{
			fprintf(stderr,"ERROR(lambda): You have to describe lambda information using option -t, because the number of lambda is more than 1\n" );
			return -1;
		}
		if ( cafe_param->pfamily && cafe_param->pcafe )
		{
			if ( beach )
			{
				cafe_each_best_lambda_by_fminsearch(cafe_param,cafe_param->num_lambdas);
			}
			else
			{
				cafe_best_lambda_by_fminsearch(cafe_param, blambda ? -1 : cafe_param->num_lambdas);
			}
		}
		else
		{
			fprintf(stderr,"ERROR(lambda): Please load family and cafe tree before\n");
			return -1;
		}
	}
	else if ( cafe_param->num_lambdas > 0 )
	{
		cafe_shell_set_lambda(cafe_param, cafe_param->lambda);
		bprint = 1;
	}

	if( beach )
	{
		if ( fhttp )
		{
			fprintf(fhttp,"<html>\n<body>\n<table border=1>\n");	
		}
		for ( i = 0 ; i < cafe_param->pfamily->flist->size ; i++ )
		{
			pCafeFamilyItem pitem = (pCafeFamilyItem)cafe_param->pfamily->flist->array[i];
			cafe_family_set_size(cafe_param->pfamily, i, pcafe);
			cafe_param->lambda_set_func(cafe_param,pitem->lambda);
			pString pstr = cafe_tree_string_with_familysize_lambda(pcafe);
			for ( j = 0 ; j < cafe_param->num_lambdas; j++ )
			{
				double a = pitem->lambda[j] * cafe_param->branch_range[ cafe_param->blsize - 1 ];
				if ( a >= 0.5 || fabs(a-0.5) < 1e-3 )
				{
					fprintf( fpout, "@@ ");
					break;
				}	
			}
			fprintf( fpout, "%s\t%s\n", pitem->id, pstr->buf );
			if ( fhttp )
			{
				fprintf(fhttp,"<tr><td rowspan=2><a href=pdf/%s-%d.pdf>%s</a></td><td>%s</td></tr>\n", 
						name, i+1, pitem->id, pitem->desc ? pitem->desc : "NONE" );	
				fprintf(fhttp,"<tr><td>%s</td></tr>\n", pstr->buf );
			}
			string_free(pstr);
			if ( fmpout )
			{
				pstr = cafe_tree_metapost( pcafe, i+1, pitem->id, 6, 6 );
				fprintf( fmpout, "%s\n", pstr->buf );
				string_free(pstr);
			}
		}
		if ( fpout != stdout ) fclose(fpout);
		if ( fmpout ) fclose(fmpout);
		if ( fhttp ) 
		{
			fprintf(fhttp,"</table>\n</body>\n</html>\n");	
			fclose(fhttp );
		}
	}
	else
	{
		if ( bprint )
		{
			pString pstr = cafe_tree_string_with_lambda(pcafe);
			printf("%s\n", pstr->buf );
			string_free(pstr);
		}
		if ( cafe_param->pfamily )
		{
			cafe_set_birthdeath_cache_thread(cafe_param);
		}
	}
	cafe_log(cafe_param,"DONE: Lamda Search or setting\n");
	return 0;
}


void __cafe_cmd_viterbi_print(int max)
{
	pCafeTree pcafe = cafe_param->pcafe;
	double* lh = cafe_shell_likelihood(max);
	double mlh =  __max(lh,pcafe->rfsize);
	cafe_tree_viterbi(pcafe);
	pString pstr = cafe_tree_string(pcafe);
	printf("%g\t%s\n", mlh , pstr->buf );
	string_free(pstr);
}

void __cafe_cmd_viterbi_family_print(int idx)
{
	pCafeTree pcafe = cafe_param->pcafe;
	cafe_family_set_size_with_family_forced(cafe_param->pfamily,idx,pcafe);
	cafe_tree_likelihood(pcafe);
	int ridx =  __maxidx(((pCafeNode)pcafe->super.root)->likelihoods,pcafe->rfsize) + pcafe->rootfamilysizes[0];
	double mlh =  __max( ((pCafeNode)pcafe->super.root)->likelihoods,pcafe->rfsize);
	cafe_tree_likelihood(pcafe);
	cafe_tree_viterbi(pcafe);
	pString pstr = cafe_tree_string(pcafe);
	printf("%g(%d)\t%s\n", mlh , ridx,  pstr->buf );
	string_free(pstr);
}

int cafe_cmd_viterbi(int argc, char* argv[])
{
	int i;
	pCafeTree pcafe = cafe_param->pcafe;
	ASSERT_IF( cafe_param->pcafe == NULL, "ERROR(viterbi): You did not specify tree: command 'tree'\n" );
	ASSERT_IF( cafe_param->lambda == NULL, "ERROR(viterbi): You have to execute 'lambda' command before\n" );
	pArrayList pargs = cafe_shell_build_argument(argc, argv);
	pArgument parg;

	if ( (parg = cafe_shell_get_argument( "-all", pargs) )  )
	{
		cafe_shell_set_sizes();
		ASSERT_IF( cafe_param->pfamily == NULL, "ERROR(viterbi): You must load family data first: command 'load'\n");
		FILE *fp = stdout;
		if  ( parg->argc )
		{
			fp = (FILE*)fopen(parg->argv[0], "w");
			if ( fp == NULL )
			{
				fprintf(stderr,"ERROR(viterbi): Cannot open %s in write mode.\n", parg->argv[0] );
				return -1;
			}
		}
		pCafeFamilyItem pitem;
/*
		int j;
	   	int	free_v;
		int nnodes = (pcafe->super.nlist->size-1)/2;
		int nrows = cafe_param->pfamily->flist->size ;
		if ( cafe_param->viterbiNodeFamilysizes == NULL )
		{
			free_v = 1;
			cafe_param->viterbiNodeFamilysizes 
				= (int**)memory_new_2dim_with_clear(nnodes,nrows,sizeof(int));
		}
		else
		{
			free_v = 0;
		}
*/
		double score = 0;
		for ( i = 0 ; i < cafe_param->pfamily->flist->size ; i++ )
		{
			pitem = (pCafeFamilyItem)cafe_param->pfamily->flist->array[i];
			pitem->maxlh = -1;
			cafe_family_set_size_with_family(cafe_param->pfamily,i, pcafe);
			cafe_tree_likelihood(pcafe);
			int ridx =  __maxidx(((pCafeNode)pcafe->super.root)->likelihoods,pcafe->rfsize) + pcafe->rootfamilysizes[0];
			double mlh =  __max( ((pCafeNode)pcafe->super.root)->likelihoods,pcafe->rfsize);
			score += log(mlh);
			cafe_tree_viterbi(pcafe);
/*
			for ( j = 0 ; j < nnodes ; j++ )
			{
				pCafeNode pcnode = (pCafeNode)pcafe->super.nlist->array[2*j+1];
				cafe_param->viterbiNodeFamilysizes[j][i] = pcnode->familysize;
			}
*/
			pString pstr = cafe_tree_string(pcafe);
			fprintf(fp,"%s\t%g\t%s\t%d\n", pitem->id,  mlh, pstr->buf, ridx );
			string_free(pstr);
		}
		cafe_log( cafe_param, "Score: %f\n", score );
		if ( fp != stdout )
		{
			fprintf(fp,"Score: %f\n", score );
		}
/*
		if ( free_v )
		{
			memory_free_2dim((void**)cafe_param->viterbiNodeFamilysizes,nnodes, 0, NULL );
			cafe_param->viterbiNodeFamilysizes = NULL;
		}
*/
		if ( fp != stdout ) fclose( fp );
	}
	else if ( (parg = cafe_shell_get_argument( "-id", pargs) )  )
	{
		int idx = cafe_family_get_index(cafe_param->pfamily, parg->argv[0] );
		if ( idx == -1 )
		{
			fprintf(stderr, "ERROR(viterbi): %s not found\n", parg->argv[0] );
			return -1;
		}
		__cafe_cmd_viterbi_family_print(idx);
	}
	else if ( (parg = cafe_shell_get_argument( "-idx", pargs) )  )
	{
		int idx = -1; sscanf( parg->argv[0], "%d", &idx );
		if ( idx == -1 )
		{
			fprintf(stderr,"ERROR(viterbi): It is not integer\n");
			return -1;
		}
		if ( idx > cafe_param->pfamily->flist->size )
		{
			fprintf(stderr, "ERROR(viterbi): Out of range[0~%d]: %s.\n", cafe_param->pfamily->flist->size, parg->argv[0] ); 
			return -1;
		}
		__cafe_cmd_viterbi_family_print(idx);
	}
	else if ( argc == 1 )
	{
		__cafe_cmd_viterbi_print( cafe_shell_set_familysize() );
	}
	else
	{
		__cafe_cmd_viterbi_print( cafe_shell_parse_familysize( argc-1, &argv[1]) );
	}
	arraylist_free(pargs, free);
	return 0;
}

int cafe_cmd_exit(int argc, char* argv[])
{
	if ( cafe_param->str_log )
	{
		string_free( cafe_param->str_log );
		fclose( cafe_param->flog );
		cafe_param->str_log = NULL;
	}
	if ( tmp_lambda_tree ) phylogeny_free( tmp_lambda_tree );
	cafe_shell_clear_param(0);
	if ( cafe_pCD ) arraylist_free( cafe_pCD, free );
	memory_free( cafe_param );
	return CAFE_SHELL_EXIT;
}

int cafe_cmd_load(int argc, char* argv[])
{
	if ( argc < 2 )
	{
		fprintf(stderr, "Usage(load): %s <famliy file>\n", argv[0] );
		return -1;
	}
	cafe_shell_clear_param(1);
	int bfilter = 0;

	pArrayList pargs = cafe_shell_build_argument(argc, argv);
	pArgument parg;

	if ((parg = cafe_shell_get_argument("-t", pargs)))
	{
		sscanf( parg->argv[0], "%d", &cafe_param->num_threads );
	}
	if ((parg = cafe_shell_get_argument("-r", pargs)))
	{
		sscanf( parg->argv[0], "%d", &cafe_param->num_random_samples );
	}
	if ((parg = cafe_shell_get_argument("-p", pargs)))
	{
		sscanf( parg->argv[0], "%lf", &cafe_param->pvalue );
	}
	if ((parg = cafe_shell_get_argument("-l", pargs)))
	{
		__cafe_cmd_log( parg->argc, parg->argv );
	}
	if ((parg = cafe_shell_get_argument("-filter", pargs)))
	{
		if ( cafe_param->pcafe == NULL )
		{
			fprintf(stderr,"Error(load): You did not specify tree. Skip filtering\n");
		}
		bfilter = 1;
	}
	if ((parg = cafe_shell_get_argument("-i", pargs)))
	{
		cafe_param->str_fdata = string_join(" ",parg->argc, parg->argv );
		cafe_param->pfamily = cafe_family_new( cafe_param->str_fdata->buf, !bfilter );
		if ( cafe_param->pfamily == NULL ) return -1;
	}
	arraylist_free(pargs, free);

	if ( cafe_param->pfamily == NULL )
	{
		fprintf(stderr,"ERROR(load): You must use -i option for input file\n");
		cafe_shell_clear_param(1);
		return -1;
	}
	cafe_param->rootfamily_sizes[0] = 1;
	cafe_param->rootfamily_sizes[1] = rint(cafe_param->pfamily->max_size * 1.25);
	cafe_param->family_sizes[1] = cafe_param->pfamily->max_size + MAX(50,cafe_param->pfamily->max_size/5);
	if ( cafe_param->pcafe )
	{
		cafe_tree_set_parameters(cafe_param->pcafe, cafe_param->family_sizes, cafe_param->rootfamily_sizes, 0 );
		cafe_family_set_species_index(cafe_param->pfamily, cafe_param->pcafe);
	}
	cafe_param->ML = (double*)memory_new( cafe_param->pfamily->flist->size, sizeof(double));
	if ( cafe_param->pcafe && bfilter )
	{
		cafe_family_filter(cafe_param);
	}
	if ( cafe_param->lambda && cafe_param->pcafe )
	{
		cafe_set_birthdeath_cache_thread(cafe_param);
	}
	cafe_cmd_print_param(0,NULL);
	return 0;
}

int cafe_cmd_save(int argc, char* argv[])
{
	if ( argc != 2 )
	{
		fprintf(stderr,"Usage(save): save filename\n");
		return -1;
	}
	FILE* fp;
	if ( ( fp = fopen( argv[1], "w" ) ) == NULL )
	{
		fprintf(stderr,"ERROR(save): Cannot open %s in write mode.\n", argv[1] );
		return -1;
	}
	pCafeFamily pcf = cafe_param->pfamily;

	int i, n;
	char buf[1024];
	string_pchar_join(buf,"\t", pcf->num_species, pcf->species );
	fprintf(fp,"Desc\tFamily ID\t%s\n", buf );	

	for ( i = 0 ; i < pcf->flist->size ; i++ )
	{
		pCafeFamilyItem pitem = (pCafeFamilyItem)pcf->flist->array[i];
		fprintf(fp,"%s\t%s\t%d", pitem->desc,pitem->id, pitem->count[0]);	
		for ( n =  1 ; n < pcf->num_species ; n++ )
		{
			fprintf(fp,"\t%d", pitem->count[n]);	
		}
		fprintf(fp,"\n");
	}
	fclose(fp);
	return 0;
}

int cafe_cmd_print_param(int argc, char* argv[])
{
	cafe_log( cafe_param, "-----------------------------------------------------------\n");
	cafe_log( cafe_param, "Family information: %s\n", cafe_param->str_fdata->buf );
	cafe_log( cafe_param, "Log: %s\n", cafe_param->flog == stdout ? "stdout" : cafe_param->str_log->buf );
	if( cafe_param->pcafe ) 
	{
		pString pstr = phylogeny_string( (pTree)cafe_param->pcafe, NULL );
		cafe_log( cafe_param, "Tree: %s\n", pstr->buf ); 
		string_free(pstr);
	}
	cafe_log( cafe_param, "The number of families is %d\n", cafe_param->pfamily->flist->size );
	cafe_log( cafe_param, "Root Family size : %d ~ %d\n", cafe_param->rootfamily_sizes[0], cafe_param->rootfamily_sizes[1]);
	cafe_log( cafe_param, "Family size : %d ~ %d\n", cafe_param->family_sizes[0], cafe_param->family_sizes[1]);
	cafe_log( cafe_param, "P-value: %f\n", cafe_param->pvalue);
	cafe_log( cafe_param, "Num of Threads: %d\n", cafe_param->num_threads );
	cafe_log( cafe_param, "Num of Random: %d\n", cafe_param->num_random_samples );
	if ( cafe_param->lambda )
	{
		pString pstr = cafe_tree_string_with_lambda(cafe_param->pcafe);
		cafe_log( cafe_param, "Lambda: %s\n", pstr->buf );
		string_free(pstr);
	}
	return 0;
}

int cafe_cmd_source(int argc, char* argv[])
{
	if ( argc != 2 )
	{
		fprintf( stderr, "Usage: %s <file>\n", argv[0]);
		return -1;
	}
	char fname[1024];
	string_pchar_join(fname, " ", argc-1, &argv[1]);
	FILE* fp = fopen( fname, "r" );
	if ( fp == NULL ) 
	{
		fprintf( stderr, "Error(source): Cannot open %s in write mode\n", argv[1] );
		return -1;
	}

	char buf[BUFSIZ];
	int rtn  = 0;
	while( fgets( buf, BUFSIZ, fp ) )
	{
		if ( (rtn = cafe_shell_dispatch_command(buf)) ) break;
	}
	fclose(fp);
	return rtn;
}

extern char* __date__;

int cafe_cmd_version(int argc, char* argv[])
{
	printf("Version: %g, at %s\n", 2.1, __date__ );
	return 0;
}

void __cafe_cmd_pvalue_print(int max)
{
	pCafeTree pcafe = cafe_param->pcafe;
	double* lh = cafe_shell_likelihood(max);
	int rfsize =  __maxidx(lh,pcafe->rfsize);
	double mlh = lh[rfsize];
   	rfsize += pcafe->rootfamilysizes[0];

	double* pcd = cafe_tree_random_probabilities(cafe_param->pcafe, rfsize, cafe_param->num_random_samples );
	double pv = pvalue( mlh, pcd, cafe_param->num_random_samples);
	pString pstr = cafe_tree_string(pcafe);
	printf("%s\n", pstr->buf );
	printf("Root size: %d with maximum likelihood : %g\n", rfsize, mlh );
	printf("p-value: %g\n", pv);
	memory_free(pcd);
	string_free(pstr);
}

int cafe_cmd_pvalue(int argc, char* argv[])
{
	int i, j;
	pArrayList pargs = cafe_shell_build_argument(argc, argv);
	pArgument parg;
	FILE* fp;
	if ( ( parg = cafe_shell_get_argument("-o", pargs) ) )
	{
		if ( cafe_param->pcafe == NULL )
		{
			fprintf(stderr, "ERROR(pvalue): You must input newick-formated tree.\n");
			return -1;
		}
		if ( cafe_param->lambda == NULL )
		{
			fprintf(stderr, "ERROR(pvalue): You must specify lambda information.\n");
			return -1;
		}
		if ( cafe_pCD ) arraylist_free( cafe_pCD, free);
		if ( (fp = fopen(parg->argv[0] ,"w") ) == NULL )
		{
			fprintf(stderr, "ERROR(pvalue): Cannot open %s in write mode\n", parg->argv[0] );
			return -1;
		}
		cafe_pCD = cafe_conditional_distribution(cafe_param);
		for ( i = 0 ; i < cafe_pCD->size ; i++ )
		{
			double* data = (double*)cafe_pCD->array[i];
			fprintf(fp, "%10.9lg", data[0] );
			for ( j = 1 ; j < cafe_param->num_random_samples ; j++ )
			{
				fprintf(fp, "\t%10.9lg", data[j] );
			}
			fprintf(fp,"\n");
		}
		fclose(fp);
	}
	else if ( ( parg = cafe_shell_get_argument("-i", pargs) ) )
	{
		if ( cafe_pCD ) arraylist_free( cafe_pCD, free);
		if ( (fp = fopen(parg->argv[0],"r") ) == NULL )
		{
			fprintf(stderr, "ERROR(pvalue): Cannot open %s in read mode\n", parg->argv[0] );
			return -1;
		}
		cafe_log(cafe_param,"Loading p-values ... \n");
		pString pstr = string_new();
//		char* buf = MEMORY_NEW(1024000, char);
		cafe_pCD = arraylist_new(10100);
//		while ( fgets(buf,1024000, fp) )
		while( file_read_line( pstr, fp ) )
		{
			char* buf = pstr->buf;
			double * data = MEMORY_NEW(cafe_param->num_random_samples, double );
			int k = 0;
			for ( i = 0, j = 0; buf[i] ; i++ )
			{
				if ( buf[i] == '\t' || buf[i] == '\n' )
				{
					buf[i] = '\0';
					sscanf(&buf[j],"%lg", &data[k++] );
					buf[i] = '\t';
					j = i + 1;
				}
			}
			arraylist_add( cafe_pCD, data );
		}
		string_free(pstr);
		fclose(fp);
		cafe_log(cafe_param,"Done Loading p-values ... \n");
	}
	else if ( ( parg = cafe_shell_get_argument("-idx", pargs) ) )
	{
		if ( cafe_pCD == NULL ) 
		{
			cafe_pCD = cafe_conditional_distribution(cafe_param);
		}
		cafe_shell_set_sizes();

		int idx = 0;

		pCafeTree pcafe = cafe_param->pcafe;
		sscanf( parg->argv[0], "%d", &idx );
		cafe_family_set_size(cafe_param->pfamily,idx,pcafe);
		double* cP = (double*)memory_new( pcafe->rfsize, sizeof(double));
		double* lh = ((pCafeNode)pcafe)->likelihoods;
		cafe_tree_p_values(pcafe, cP, cafe_pCD, cafe_param->num_random_samples);
		for (idx = 0 ; idx < pcafe->rfsize  ; idx++ )
		{
			printf("%d\t%lg\t%lg\n", idx + pcafe->rootfamilysizes[0], lh[idx] , cP[idx] );
		}
		free(cP);
	}
	else if ( pargs->size > 0 )
	{
		__cafe_cmd_pvalue_print( cafe_shell_parse_familysize(argc-1,&argv[1] ) );
	}
	else
	{
		__cafe_cmd_pvalue_print( cafe_shell_set_familysize() );
	}
	arraylist_free( pargs, free );
	return 0;
}


int cafe_cmd_branchlength(int argc, char* argv[])
{
	ASSERT_IF( cafe_param->pcafe == NULL, "ERROR(branchlength): You did not specify tree: command 'tree'\n" );

	pString pstr = cafe_tree_string_with_id( cafe_param->pcafe );
	printf("%s\n", pstr->buf );
	string_free(pstr);
	int err = 0;

	if ( argc == 1 )
	{
		err = cafe_shell_set_branchlength();	
	}
	else if ( argc > 2 )
	{
		err = cafe_shell_parse_branchlength(argc-1,&argv[1]);	
	}
	if ( !err )
	{
		cafe_tree_string_print(cafe_param->pcafe);
	}
	if ( cafe_param->pfamily )
	{
		int i;
		for ( i = 0 ; i < cafe_param->pfamily->flist->size; i++ )
		{
			pCafeFamilyItem pitem = (pCafeFamilyItem)cafe_param->pfamily->flist->array[i];
			pitem->maxlh = -1;
		}
	}
	return err;
}

int cafe_cmd_family(int argc, char* argv[])
{
	ASSERT_IF( cafe_param->pfamily == NULL, "ERROR(family): You must load family data first: command 'load'\n");

	int i, idx;
	pCafeFamilyItem pitem = NULL;
	pArrayList pargs = cafe_shell_build_argument(argc, argv);
	pArgument parg;

	if ( ( parg = cafe_shell_get_argument("-idx", pargs) ) )
	{
		idx = -1; sscanf( parg->argv[0], "%d", &idx );
	}
	else if ( ( parg = cafe_shell_get_argument("-id", pargs) ) )
	{
		idx = cafe_family_get_index( cafe_param->pfamily, parg->argv[0]);
	}
	else if ( ( parg = cafe_shell_get_argument("-add", pargs) ) )
	{
		pCafeFamily pcf = cafe_param->pfamily;
		if ( pcf == NULL )
		{
			pcf = MEMORY_NEW( 1, CafeFamily);
			pcf->flist = arraylist_new(1000);
			pArrayList nlist = cafe_param->pcafe->super.nlist;
			pcf->num_species = (nlist->size+1)/2;
			pcf->species = MEMORY_NEW( pcf->num_species, char*);
			pcf->index = MEMORY_NEW( pcf->num_species, int);
			for ( i = 0 ; i < nlist->size ; i+= 2 )
			{
				pcf->index[i] = i;
				pPhylogenyNode pnode = (pPhylogenyNode)nlist->array[i];
				pcf->species[i] = MEMORY_NEW( strlen(pnode->name)+1, char );
				strcpy( pcf->species[i], pnode->name );
			}
			cafe_param->pfamily = pcf;
		}
		pCafeFamilyItem pitem = MEMORY_NEW(1, CafeFamilyItem );
		pitem->id = MEMORY_NEW( strlen( parg->argv[0]) + 1, char );  
		pitem->ref = -1;
		pitem->count = MEMORY_NEW_WITH_CLEAR( pcf->num_species, int );
		pitem->maxlh = -1;
		pitem->desc = NULL;
		strcpy( pitem->id, parg->argv[0] );
		for ( i = 1 ; i <= pcf->num_species ; i++ )
		{
			sscanf( parg->argv[i], "%d", &pitem->count[pcf->index[i]] );
		}
		arraylist_add( pcf->flist, pitem );
	}
	else if ( ( parg = cafe_shell_get_argument("-del", pargs) ) )
	{
								
	}
	else if ( ( parg = cafe_shell_get_argument("-filter", pargs) ) )
	{
		ASSERT_IF( cafe_param->pcafe == NULL, "ERROR(family): You did not specify tree: command 'tree'\n" );
		cafe_family_filter( cafe_param );
		cafe_cmd_print_param(0,NULL);
		return 0;
	}
	if ( idx < 0  )
	{
		fprintf(stderr, "ERROR(family): your request not found\n");
		return -1;
	}
	if ( idx >= cafe_param->pfamily->flist->size )
	{
		fprintf(stderr, "ERROR(family): The index range is from 0 to %d\n", cafe_param->pfamily->flist->size );
		return -1;
	}
	pitem = (pCafeFamilyItem)cafe_param->pfamily->flist->array[idx];
	if ( pitem )
	{
		printf("ID: %s\n", pitem->id );
		printf("Desc: %s\n", pitem->desc );
		for ( i = 0 ; i < cafe_param->pfamily->num_species ; i++ )
		{
			printf("%s: %d\n", cafe_param->pfamily->species[i], pitem->count[i] );
		}
		if ( cafe_param->pcafe && cafe_param->pcafe->pbdc_array ) __cafe_cmd_viterbi_family_print(idx);
	}
	arraylist_free( pargs, free );
	return pitem ? 0 : -1;
}

extern double __cafe_best_lambda_search(double* plambda, void* args);

int cafe_cmd_score(int argc, char* argv[])
{
	cafe_log(cafe_param,"%lf\n", -__cafe_best_lambda_search(cafe_param->lambda, (void*)cafe_param) );
	cafe_shell_set_sizes();
	return 0;
}

int cafe_cmd_report(int argc, char* argv[] )
{
	int i;

	ASSERT_IF( cafe_param->pfamily == NULL, "ERROR(report): You did not load family: command 'load'\n" );
	ASSERT_IF( cafe_param->pcafe == NULL, "ERROR(report): You did not specify tree: command 'tree'\n" );
	ASSERT_IF( cafe_param->lambda == NULL, "ERROR(report): You have to execute 'lambda' command before\n" );

	char name[1024];


	strcpy( name, argv[1] ); strcat( name, ".cafe" );
	cafe_param->fout = fopen(name,"w");

	if ( cafe_param->fout == NULL )
	{
		fprintf(stderr,"ERROR(report): Cannot open %s in write mode.\n", argv[1] );
		return -1;
	}
	int bc, lh, lh2, just_save;
	bc = 0;
	lh = 0;
	lh2 = 0;
	just_save = 0;
	for (  i = 2; i < argc ; i++ )
	{
		if ( strcasecmp( argv[i],"branchcutting") == 0 ) bc = 1;
		if ( strcasecmp( argv[i],"likelihood") == 0 ) lh = 1;
		if ( strcasecmp ( argv[i], "lh2" ) == 0 ) lh2 = 1;
		if ( strcasecmp ( argv[i], "save" ) == 0 ) 
		{
			bc = 0; lh = 0; lh2 = 0;
			just_save = 1;
			break;
		}
	}
	
	if ( !just_save )
	{
		cafe_shell_set_sizes();
		cafe_shell_clear_interal_data();
	}

	if ( bc || lh )
	{
		cafe_pCD = cafe_viterbi(cafe_param, cafe_pCD );
		if ( bc ) cafe_branch_cutting(cafe_param);
		if ( lh ) cafe_likelihood_ratio_test(cafe_param);
		cafe_log( cafe_param, "Building Text report: %s\n", name);
		cafe_report(cafe_param, CAFE_REPORT_TEXT);
		fclose( cafe_param->fout );
		strcpy( name, argv[1] ); strcat( name, ".mp" );
		cafe_param->fout = fopen(name,"w");
		cafe_log( cafe_param, "Building Metapost report: %s\n", name);
		cafe_report(cafe_param, CAFE_REPORT_PDF);
	}
	else if ( lh2 )
	{
		cafe_lhr_for_diff_lambdas(cafe_param, tmp_lambda_tree, 2,cafe_shell_set_lambda );
	}
	else
	{
		if ( !just_save ) 
		{
			cafe_pCD = cafe_viterbi(cafe_param, cafe_pCD );
		}
		cafe_report(cafe_param, CAFE_REPORT_TEXT);
		fclose( cafe_param->fout );
		strcpy( name, argv[1] ); strcat( name, ".mp" );
		cafe_param->fout = fopen(name,"w");
		cafe_log( cafe_param, "Building Metapost report: %s\n", name);
		cafe_report(cafe_param, CAFE_REPORT_PDF);
	}
	fclose( cafe_param->fout );
	
// HTML
	strcpy( name, argv[1] ); strcat( name, ".html" );
	FILE* fhttp = fopen(name,"w");

	cafe_log( cafe_param, "Building HTML report: %s\n", name);
	fprintf(fhttp,"<html>\n<body>\n<table border=1>\n");	
	for ( i = 0 ; i < cafe_param->pfamily->flist->size ; i++ )
	{
		pCafeFamilyItem pitem = (pCafeFamilyItem) cafe_param->pfamily->flist->array[i];
		fprintf(fhttp,"<tr><td><a href=pdf/%s-%d.pdf>%s</a></td><td>%s</td></tr>\n", 
				argv[1], i+1, pitem->id, pitem->desc ? pitem->desc : "NONE" );	
	}
	fprintf(fhttp,"</table>\n</body>\n</html>\n");	
	fclose(fhttp);

	cafe_log( cafe_param, "Report Done\n");
	return 0;
}

int cafe_cmd_retrieve(int argc, char* argv[] )
{
	cafe_shell_clear_param(1);
	if ( cafe_report_retrieve_data(argv[1], cafe_param)	== -1 )
	{
		return -1;
	}		
	return 0;
}

int __cafe_cmd_log(int argc ,char* argv[] )
{
	if ( cafe_param->str_log )
	{
		string_free( cafe_param->str_log );
		fclose( cafe_param->flog );
		cafe_param->str_log = NULL;
	}
	if ( !strcmp( argv[0], "stdout" ) )
	{
		cafe_param->str_log = NULL;
		cafe_param->flog = stdout;
	}
	else
	{
		cafe_param->str_log = string_join(" ",argc, argv );
		if (  ( cafe_param->flog = fopen( cafe_param->str_log->buf, "a" ) ) == NULL )
		{
			fprintf(stderr, "ERROR(log): Cannot open log file: %s\n", cafe_param->str_log->buf );	
			string_free( cafe_param->str_log );
			cafe_param->flog = stdout;
			return -1;
		}
	}
	return 0;
}

int cafe_cmd_log(int argc, char* argv[] )
{
	if ( argc == 1 ) 
	{
		printf( "Log: %s\n", cafe_param->flog == stdout ? "stdout" : cafe_param->str_log->buf );
	}
	else
	{
		return __cafe_cmd_log(argc-1,&argv[1]);
	}
	return 0;
}

extern void cgi_set_type(char* type);
extern void cgi_start_html(char* title);
extern void cgi_end_html();

int cafe_cmd_cgi(int argc, char* argv[] )
{
	cgi_set_type("text/html");
	cgi_start_html("Cafe Shell test");
	printf("<body>\n");
	printf("Underconstruction\n");
	printf("</body>\n");
	cgi_end_html();
	return 0;
}

void __cafe_tree_string_gainloss(pString pstr, pPhylogenyNode ptnode)
{
	int familysize =  ((pCafeNode)ptnode)->familysize;
	if ( ptnode->name ) string_fadd( pstr, "%s", ptnode->name);
	string_fadd(pstr,"_%d", familysize );
	pCafeNode parent = (pCafeNode)ptnode->super.parent;
	if ( parent )
	{
		string_fadd(pstr,"<%d>", familysize - parent->familysize );
	}
}

void __cafe_tree_string_sum_gainloss(pString pstr, pPhylogenyNode ptnode)
{
	int familysize =  ((pCafeNode)ptnode)->familysize;
	if ( ptnode->name ) string_fadd( pstr, "%s", ptnode->name);
	pCafeNode pcnode = (pCafeNode)ptnode;
	string_fadd(pstr,"<%d/%d/%d>", pcnode->viterbi[0], pcnode->viterbi[1], familysize );
}

double __cafe_tree_gainloss_mp_annotation(pString pstr, pTreeNode pnode, pMetapostConfig pmc, va_list ap)
{
	pCafeNode pcnode = (pCafeNode)pnode;
    string_add( pstr, ";\n");
	string_fadd( pstr, "label.urt( btex \\small{%d/%d/%d} ", pcnode->viterbi[0], pcnode->viterbi[1], pcnode->familysize, pnode->id );
	string_fadd( pstr, "etex, p[%d]);\n", pnode->id );
	double last = 0;
	if ( pnode->parent )
	{
		string_fadd( pstr, "xpart mid[%d] = xpart(p[%d]);\n", pnode->id, pnode->id );
		string_fadd( pstr, "ypart mid[%d] = (ypart(p[%d])+ypart(p[%d]))/2;\n", pnode->id, pnode->id, pnode->parent->id );
		string_fadd( pstr, "label.rt( btex $l = %g$ ", ((pPhylogenyNode)pnode)->branchlength );
		string_fadd( pstr, "etex, mid[%d]);\n", pnode->id  );
		string_fadd( pstr, "label.rt( btex $\\lambda=%f$ ", pcnode->lambda );
		last -= 0.15;
		string_fadd( pstr, "etex, mid[%d] + (0,%fu));\n",  pnode->id, last );
	}
	return last;
}

extern double cafe_tree_mp_remark(pString pstr, pTree ptree, pMetapostConfig pmc, va_list ap);

pString __cafe_tree_gainloss_metapost(pCafeTree pcafe, int id, char* title, double width, double height )
{
	pMetapostConfig pmc = MEMORY_NEW(1,MetapostConfig);
	pmc->id = id;
	pmc->unit = MP_UNIT_IN;
	pmc->dir = MP_DIR_VERTICAL;
	pmc->shape = MP_SHAPE_RECT | MP_SHAPE_MOST_CENTER;
	pmc->fmod = __cafe_tree_gainloss_mp_annotation;
	pmc->fremark = cafe_tree_mp_remark;
	pmc->width = width;
	pmc->height = height;
	pString pstr = phylogeny_to_mp( (pTree)pcafe, pmc, title );
	memory_free(pmc);
	return pstr;
}

int cafe_cmd_gainloss(int argc, char* argv[] )
{
	ASSERT_IF( cafe_param->pfamily == NULL, "ERROR(gainloss): You did not load family: command 'load'\n" );
	ASSERT_IF( cafe_param->pcafe == NULL, "ERROR(gainloss): You did not specify tree: command 'tree'\n" );
	ASSERT_IF( cafe_param->lambda == NULL, "ERROR(gainloss): You have to execute 'lambda' command before\n" );

	if ( cafe_param->viterbiNodeFamilysizes == NULL )
	{
		cafe_pCD = cafe_viterbi(cafe_param, cafe_pCD );
	}
	
	char name[1024];
	strcpy( name, argv[1] ); strcat( name, ".gs" );
	FILE* fout = fopen(name,"w");

	int i,j;
	int** nodefs = cafe_param->viterbiNodeFamilysizes;
	pCafeTree pcafe = cafe_param->pcafe;
	int fsize = cafe_param->pfamily->flist->size;
	int nnodes = (pcafe->super.nlist->size-1)/2;

	pCafeTree psum = cafe_tree_copy(pcafe);

	for ( i = 0 ; i < psum->super.nlist->size ; i++ )
	{
		pCafeNode pcnode = (pCafeNode)psum->super.nlist->array[i];
		pcnode->familysize = 0;
	}

	pString pstr;
	int sum = 0;
	int totalsum = 0;
	for ( j = 0 ; j < psum->super.nlist->size ; j++ )
	{
		pCafeNode pcnode = (pCafeNode)psum->super.nlist->array[j];
		pcnode->viterbi[0] = 0;
		pcnode->viterbi[1] = 0;
	}

	for( i = 0; i < fsize ; i++ )
	{
		sum = 0;
		pCafeFamilyItem pitem = (pCafeFamilyItem)cafe_param->pfamily->flist->array[i];
		fprintf( fout,"%s\t", pitem->id );
		cafe_family_set_size(cafe_param->pfamily,i, pcafe);
		for ( j = 0 ; j < nnodes; j++ )
		{
			pCafeNode pnode = (pCafeNode)pcafe->super.nlist->array[2*j+1];
			pnode->familysize = nodefs[j][i];
		}
		for ( j = 0 ; j < pcafe->super.nlist->size ; j++ )
		{
			pCafeNode pcnode = (pCafeNode)pcafe->super.nlist->array[j];
			pCafeNode pcsum = (pCafeNode)psum->super.nlist->array[j];
			if ( tree_is_root( (pTree)pcafe, (pTreeNode)pcnode ) ) continue;
			pCafeNode parent = (pCafeNode)((pTreeNode)pcnode)->parent;
			int diff = pcnode->familysize - parent->familysize;
			sum += diff;
			pcsum->familysize += diff;
			if ( diff > 0 )
			{
				pcsum->viterbi[0] += diff;
			}
			else if ( diff <  0 )
			{
				pcsum->viterbi[1] += diff;
			}
		}
		totalsum += sum;
		fprintf( fout, "%d\t", sum );
		pstr = phylogeny_string((pTree)pcafe, __cafe_tree_string_gainloss );
		fprintf( fout, "%s\n", pstr->buf );
		string_free(pstr);
	}
	fprintf( fout,"SUM\t%d\t", totalsum );
	pstr = phylogeny_string((pTree)psum, __cafe_tree_string_sum_gainloss );
	fprintf( fout, "%s\n", pstr->buf );
	string_free(pstr);

	fclose(fout);

	strcpy( name, argv[1] ); strcat( name, ".mp" );
	fout = fopen(name,"w");
	sprintf(name,"SUM: %d", totalsum );
	pstr = __cafe_tree_gainloss_metapost(psum, 0, name,10, 8);
	fprintf(fout, "%s", pstr->buf );
	string_free(pstr);
	cafe_tree_free(psum);
	fclose(fout);
	return 0;
}

int __cafe_cmd_extinct_count_zero()
{
	int n;
	int cnt_zero = 0;
	tree_clear_reg((pTree)cafe_param->pcafe);
	pArrayList nlist = ((pTree)cafe_param->pcafe)->nlist;
	((pTree)cafe_param->pcafe)->root->reg = 1;
	for ( n = 0 ; n < nlist->size ; n+=2 )
	{
		pCafeNode pcnode = (pCafeNode)nlist->array[n];
		if ( pcnode->familysize )
		{
			pTreeNode ptnode = (pTreeNode)pcnode;
			while( ptnode )
			{
				ptnode->reg = 1;
				ptnode = ptnode->parent;
			}
		}
	}
	for ( n = 0 ; n < nlist->size ; n++ )
	{
		pTreeNode pnode = (pTreeNode)nlist->array[n];
		if ( pnode->parent == NULL ) continue;
		if ( pnode->reg == 0 &&  pnode->parent->reg == 1 )
		{
			cnt_zero++;
		}
	}
/*
	for ( n = 0 ; n < nlist->size ; n++ )
	{
		pTreeNode pnode = (pTreeNode)nlist->array[n];
		pCafeNode pcnode = (pCafeNode)nlist->array[n];
		printf("%d(%s): %d %d\n", n, n%2==0 ? pcnode->super.name : "null", pnode->reg, pcnode->familysize );
	}
	printf(  "--- : %d\n", cnt_zero );
	return 0;
*/
	return cnt_zero;
}

int cafe_cmd_extinct_old(int argc, char* argv[] )
{
	int i, j;
	ASSERT_IF( cafe_param->pfamily == NULL, "ERROR(extinct): You did not load family: command 'load'\n" );
	ASSERT_IF( cafe_param->pcafe == NULL, "ERROR(extinct): You did not specify tree: command 'tree'\n" );
	ASSERT_IF( cafe_param->lambda == NULL, "ERROR(extinct): You have to execute 'lambda' command before\n" );

	pCafeTree pcafe = cafe_param->pcafe;
	int familysize = cafe_param->pfamily->flist->size;
	double* roots_extinct = MEMORY_NEW_WITH_CLEAR(familysize,double);
	int* roots_num = MEMORY_NEW_WITH_CLEAR(pcafe->rfsize+1,int);
	int* roots_size = MEMORY_NEW_WITH_CLEAR(familysize,int);
	cafe_log( cafe_param, "Extinction count from data:\n");

	pCafeNode croot = (pCafeNode)pcafe->super.root;

	for ( i = 0 ; i < familysize ; i++ )
	{
		cafe_family_set_size(cafe_param->pfamily,i, pcafe);
		cafe_tree_viterbi(pcafe);
		roots_extinct[i] = __cafe_cmd_extinct_count_zero();
		roots_size[i] = croot->familysize;
		roots_num[croot->familysize]++;
	}

	int maxsize = roots_num[1];
	for ( i = 2 ; i <= pcafe->rfsize ; i++ )
	{
		if ( maxsize < roots_num[i] ) maxsize = roots_num[i];	
	}

	double* data = MEMORY_NEW(maxsize,double);

	pHistogram phist = histogram_new(NULL,0,0);
	pHistogram phist_accu = histogram_new( NULL, 0, 0);
	unsigned int accu_sum  = 0;
	for ( i = 1 ; i <= pcafe->rfsize; i++ )
	{
		if ( roots_num[i] )
		{
			int k = 0;
			int sum = 0;
			for ( j = 0 ; j < familysize ; j++ )
			{
				if ( roots_size[j] == i ) 
				{
					data[k++] = roots_extinct[j];
					sum += roots_extinct[j];
				}
			}
			histogram_set_sparse_data(phist,data,k);
			cafe_log( cafe_param, "--------------------------------\n");
			cafe_log( cafe_param, "Root Size: %d\n", i );
			histogram_print(phist, cafe_param->flog);
			if ( cafe_param->flog != stdout ) histogram_print(phist, NULL);
			cafe_log( cafe_param, "Sum : %d\n", sum );
			accu_sum += sum;
		}
	}
	cafe_log( cafe_param, "--------------------------------\n");
	cafe_log( cafe_param, "Total\n");
	histogram_set_sparse_data(phist_accu, roots_extinct, familysize );
	histogram_print(phist_accu, cafe_param->flog);
	if ( cafe_param->flog != stdout ) histogram_print(phist_accu, NULL);
	cafe_log( cafe_param, "Sum : %d\n", accu_sum );

	histogram_free(phist);
	histogram_free(phist_accu);
	
	memory_free(data);
	memory_free(roots_extinct);
	memory_free(roots_size);
	memory_free(roots_num);
	return 0;
}

int cafe_cmd_sim_extinct(int argc, char* argv[])
{
	ASSERT_IF( cafe_param->pcafe == NULL, "ERROR(simextinct): You did not specify tree: command 'tree'\n" );
	ASSERT_IF( cafe_param->lambda == NULL, "ERROR(simextinct): You have to execute 'lambda' command before\n" );

	pArrayList pargs = cafe_shell_build_argument(argc, argv);
	pArgument parg;
	int range[2] = { 1, cafe_param->rootfamily_sizes[1] };
	int num_trials = 10000;

/*
	if ( args->argc == 0 )
	{
		fprintf(stderr, "Usage: %s -t #[=10000]\n", argv[0] );
		fprintf(stderr, "-t : number of trials, default 10000"\n);
		arraylist_free(pargs, free);
		return -1;
	}
*/
	cafe_log( cafe_param, "Extinction count from Monte Carole:\n");
	if ( (parg = cafe_shell_get_argument( "-r", pargs) ) )
	{
		if ( index(parg->argv[0], ':' ) )	
		{
			sscanf( parg->argv[0], "%d:%d", &range[0], &range[1] );		
		}
		else
		{
			sscanf( parg->argv[0], "%d", &range[1] );
			range[0] = range[1];
		}
	}
	cafe_log( cafe_param, "root range: %d ~ %d\n", range[0], range[1] );

	if ( (parg = cafe_shell_get_argument( "-t", pargs) ) )
	{
		sscanf( parg->argv[0], "%d", &num_trials);	
	}
	cafe_log( cafe_param, "# trials: %d\n", num_trials );

	if ( range[0] > range[1] || range[1] > cafe_param->rootfamily_sizes[1] )
	{
		fprintf(stderr, "ERROR(simextinct): -r : 1 ~ %d\n", cafe_param->rootfamily_sizes[1] );
		arraylist_free(pargs, free);
		return -1;
	}
	arraylist_free(pargs, free);

	int i, r;
	unsigned int accu_sum = 0;
	pHistogram phist_sim = histogram_new(NULL,0,0);
	pHistogram phist_accu = histogram_new(NULL,0,0);
	double* data = MEMORY_NEW_WITH_CLEAR( num_trials, double );
	for ( r = range[0] ; r <= range[1] ; r++ )
	{
		int cnt_zero = 0;
		for(  i = 0 ; i < num_trials ; i++ )
		{
			cafe_tree_random_familysize(cafe_param->pcafe, r);
			data[i] = __cafe_cmd_extinct_count_zero();
			cnt_zero += data[i];
		}
		cafe_log( cafe_param, "------------------------------------------\n");
		cafe_log( cafe_param, "Root size: %d\n", r );
		histogram_set_sparse_data(phist_sim,data,num_trials);
		histogram_merge( phist_accu, phist_sim );
		histogram_print(phist_sim,cafe_param->flog );
		if ( cafe_param->flog != stdout ) histogram_print(phist_sim,NULL);
		cafe_log( cafe_param, "Sum : %d\n", cnt_zero );
		accu_sum +=  cnt_zero;
	}

	cafe_log( cafe_param, "------------------------------------------\n");
	cafe_log( cafe_param, "Total\n", r );
	histogram_print(phist_accu,cafe_param->flog );
	if ( cafe_param->flog != stdout ) histogram_print(phist_accu,NULL);
	cafe_log( cafe_param, "Sum : %d\n", accu_sum );


	histogram_free(phist_sim);
	histogram_free(phist_accu);
	tree_clear_reg( (pTree)cafe_param->pcafe );
	memory_free(data);
	return 0;
}

double __hg_norm_cdf_func(double p, double* args)
{
	return normcdf(p, args[0], args[1]);
}

void __hg_print_sim_extinct(pHistogram** phist_sim_n, pHistogram* phist_sim,  
		                    int r, pHistogram phist_tmp, double* cnt, int num_trials)
{
	int j, t;
	double args[2];
	double alpha;
	for ( j = 0 ; j < phist_sim[r]->nbins ; j++ )
	{
		for ( t = 0 ; t < num_trials ; t++ )
		{
			cnt[t] = histogram_get_count(phist_sim_n[t][r], phist_sim[r]->point[j]);
		}
		histogram_set_by_unit( phist_tmp, cnt, num_trials, 1 );
		args[0] = mean(cnt,num_trials);
		args[1] = sqrt(variance(cnt,num_trials));
		cafe_log(cafe_param, "%g\t%d\t%4.3f\t%g\t%g\t%g ~ %g\n",
				phist_sim[r]->point[j], phist_sim[r]->count[j], ((double)phist_sim[r]->count[j])/phist_sim[r]->nsamples,
				args[0], args[1], phist_tmp->min, phist_tmp->max );		
	}
	memset(cnt, 0, sizeof(double)*num_trials);
	double sum = 0;
	for ( j = 0 ; j < phist_sim[r]->nbins ; j++ )
	{
		double p = phist_sim[r]->point[j];
		if ( p == 0 ) continue;
		for ( t = 0 ; t < num_trials ; t++ )
		{
			double a = p * histogram_get_count(phist_sim_n[t][r], p );
			cnt[t] += a;
			sum += a;
		}
	}
	histogram_set_by_unit( phist_tmp, cnt, num_trials, 1);
	if ( phist_tmp->nbins > 10 )
	{
		histogram_set_by_bin( phist_tmp, cnt, num_trials, 10 );
	}
	args[0] = mean(cnt,num_trials);
	args[1] = sqrt(variance(cnt,num_trials));
	alpha = histogram_check_fitness( phist_tmp, args, __hg_norm_cdf_func );
	cafe_log(cafe_param, "Extinct: %g\t%g\t%g\t%g\t%g ~ %g\n", 
			sum, args[0], args[1], alpha,
			args[0] - 1.96 * args[1], 
			args[0] + 1.96 * args[1]);
	histogram_print( phist_tmp, cafe_param->flog );
}

int cafe_cmd_generate_random_family(int argc, char* argv[])
{
	if ( argc == 1 )
	{
		fprintf(stderr,"Usage: %s file\n", argv[0] );
		return -1;
	}
  if ( argv[1] ) {    // check directory exists
    char * dirprefix = strdup(argv[1]);
    char * pch = NULL;
    char * prevpch = NULL; 
    char directory[30];
    directory[0] = '\0';
    pch = strtok (dirprefix,"/\0");
    while (pch != NULL)
    {
      if (prevpch) {
        strcat(directory, prevpch);
      }
      prevpch = pch;
      pch = strtok (NULL, " ,.-");
    }
    if (strlen(directory) != 0) {
      struct stat st;
      if(stat(directory,&st) != 0) {
        perror(directory);
        fprintf(stderr, "Please create directory %s before running genfamily.\n", directory);
        return -1;
      }
    }
  }
	int num_trials = 1;
	pArrayList pargs = cafe_shell_build_argument(argc, argv);
	pArgument parg;
	if ( (parg = cafe_shell_get_argument( "-t", pargs) ) )
	{
		sscanf( parg->argv[0], "%d", &num_trials);	
	}
	arraylist_free(pargs, free);
	pCafeTree pcafe = cafe_param->pcafe;
	int i, j, k;
	int* roots_num = MEMORY_NEW_WITH_CLEAR(pcafe->rfsize+1,int);
	int familysize = cafe_param->pfamily->flist->size;
	pCafeNode croot = (pCafeNode)pcafe->super.root;
	printf("Viterbi\n");
	for ( i = 0 ; i < familysize; i++ )
	{
		if ( i % 1000 == 0 )
		{
			printf("%d ...\n", i );
		}
		cafe_family_set_size(cafe_param->pfamily,i, pcafe);
		cafe_tree_viterbi(pcafe);
		roots_num[croot->familysize]++;
	}

	int t;
	for ( t = 0 ; t < num_trials ; t++ )
	{
		int id = 1;	
		char buf[300];
		sprintf(buf,"%s_%d.tab",argv[1], t+1 );
		FILE* fout = fopen( buf, "w" );
		if ( fout == NULL )
		{
			perror(argv[1]);
			return -1;
		}
		fprintf(fout, "DESC\tFID" );
		pArrayList nlist = pcafe->super.nlist;
		for ( i = 0 ; i < nlist->size ; i+=2 )
		{
			pPhylogenyNode pnode = (pPhylogenyNode)nlist->array[i];
			fprintf(fout,"\t%s", pnode->name );
		}
		fprintf(fout,"\n");
		for ( i = 1 ; i <= pcafe->rfsize ; i++ )
		{
			if ( roots_num[i] ) 
			{
				for ( j = 0 ; j < roots_num[i] ; j++ )
				{
					cafe_tree_random_familysize(cafe_param->pcafe, i);
					fprintf(fout, "%d\t%d", i, id++ );
					for ( k = 0 ; k < nlist->size ; k+=2 )
					{
						pCafeNode pnode = (pCafeNode)nlist->array[k];
						fprintf(fout,"\t%d",pnode->familysize );
					}
					fprintf(fout,"\n");
				}
			}
		}
		fclose(fout);
	}
	memory_free( roots_num );
	return 0;
}

int cafe_cmd_extinct(int argc, char* argv[])
{
	ASSERT_IF( cafe_param->pfamily == NULL, "ERROR(extinct): You did not load family: command 'load'\n" );
	ASSERT_IF( cafe_param->pcafe == NULL, "ERROR(extinct): You did not specify tree: command 'tree'\n" );
	ASSERT_IF( cafe_param->lambda == NULL, "ERROR(extinct): You have to execute 'lambda' command before\n" );

	int familysize = cafe_param->pfamily->flist->size;
	pCafeTree pcafe = cafe_param->pcafe;
	double* roots_extinct = MEMORY_NEW_WITH_CLEAR(familysize,double);
	int* roots_size = MEMORY_NEW_WITH_CLEAR(familysize,int);
	int* roots_num = MEMORY_NEW_WITH_CLEAR(pcafe->rfsize+1,int);
	double* avg_extinct = MEMORY_NEW_WITH_CLEAR(pcafe->rfsize+1,double);

	pArrayList pargs = cafe_shell_build_argument(argc, argv);
	pArgument parg;
	int num_trials = 1000;

	cafe_log( cafe_param, ">> Data and Simulation for extinction:\n");

	if ( (parg = cafe_shell_get_argument( "-t", pargs) ) )
	{
		sscanf( parg->argv[0], "%d", &num_trials);	
	}
	arraylist_free(pargs, free);

	cafe_log( cafe_param, "# trials: %d\n", num_trials );

	int i,j;
	int total_extinct = 0;

	fprintf(stderr, "Data ...\n");

	pCafeNode croot = (pCafeNode)pcafe->super.root;
	for ( i = 0 ; i < familysize; i++ )
	{
		cafe_family_set_size(cafe_param->pfamily,i, pcafe);
		cafe_tree_viterbi(pcafe);
		roots_size[i] = croot->familysize;
		roots_extinct[i] = __cafe_cmd_extinct_count_zero();
		total_extinct += roots_extinct[i];
		roots_num[roots_size[i]]++;
	}

	pHistogram phist_tmp = histogram_new(NULL,0,0);
	pHistogram* phist_data = MEMORY_NEW_WITH_CLEAR(pcafe->rfsize+1,pHistogram);
	pHistogram* phist_sim = MEMORY_NEW_WITH_CLEAR(pcafe->rfsize+1,pHistogram);
	phist_sim[0] = histogram_new(NULL,0,0);
	phist_data[0] = histogram_new(NULL,0,0);
	int maxsize = roots_num[1];
	for ( i = 1 ; i <= pcafe->rfsize ; i++ ) 
	{
		if ( roots_num[i] > maxsize ) maxsize = roots_num[i];
		if ( roots_num[i] )
		{
			phist_data[i] = histogram_new(NULL,0,0);	
			phist_sim[i] = histogram_new(NULL,0,0);	
		}
	}
	histogram_set_sparse_data(phist_data[0],roots_extinct,familysize);

	double* data = MEMORY_NEW( maxsize, double );
	double* simdata = MEMORY_NEW( familysize, double );

	cafe_log(cafe_param,"*******************  DATA  *************************\n");

	for ( i = 1 ; i <= pcafe->rfsize; i++ )
	{
		if ( roots_num[i] )
		{
			int k = 0;
			int sum = 0;
			for ( j = 0 ; j < familysize ; j++ )
			{
				if ( roots_size[j] == i ) 
				{
					data[k++] = roots_extinct[j];
					sum += roots_extinct[j];
				}
			}
			histogram_set_sparse_data(phist_data[i],data,k);
			cafe_log(cafe_param,"--------------------------------\n");
			cafe_log(cafe_param,"Root Size: %d\n", i );
			histogram_print(phist_data[i], cafe_param->flog);
			if ( cafe_param->flog != stdout ) histogram_print(phist_data[i], NULL);
			cafe_log(cafe_param,"Extinct: %d\n", sum );
		}
	}

	fprintf(stderr, "Begin simulation...\n");
	cafe_log(cafe_param,"******************* SIMULATION **********************\n");

	pHistogram** phist_sim_n = MEMORY_NEW_2DIM( num_trials, pcafe->rfsize+1, pHistogram );

	int t;
	for ( t = 0 ; t < num_trials; t++ )
	{
		if ( t % 100 == 0 && t != 0 )
		{
			fprintf(stderr, "\t%d...\n", t );
		}
		int f = 0;
		for ( i = 1 ; i <= pcafe->rfsize ; i++ )
		{
			if ( roots_num[i] ) 
			{
				int k = 0;
				int sum = 0;
				for ( j = 0 ; j < roots_num[i] ; j++ )
				{
					cafe_tree_random_familysize(cafe_param->pcafe, i);
					simdata[f] = __cafe_cmd_extinct_count_zero();
					data[k++] =simdata[f];
					sum += simdata[f];
					f++;
				}
				avg_extinct[i] += sum;
				histogram_set_sparse_data(phist_tmp,data,k);
				histogram_merge( phist_sim[i], phist_tmp );
				phist_sim_n[t][i] = histogram_new(NULL,0,0);
				histogram_merge( phist_sim_n[t][i], phist_tmp );
			}
		}
		histogram_set_sparse_data(phist_tmp,simdata,familysize);
		histogram_merge( phist_sim[0], phist_tmp );
		phist_sim_n[t][0] = histogram_new(NULL,0,0);
		histogram_merge( phist_sim_n[t][0], phist_tmp);
	}

	double* cnt  = MEMORY_NEW(num_trials, double);
	double avg_total_extinct = 0;
	for ( i = 1 ; i <= pcafe->rfsize ; i++ )
	{
		if ( phist_sim[i] )
		{
			cafe_log(cafe_param,"--------------------------------\n");
			cafe_log(cafe_param,"Root Size: %d\n", i );
			__hg_print_sim_extinct(phist_sim_n,phist_sim,i,phist_tmp,cnt,num_trials);
			avg_extinct[i] /= num_trials;
			avg_total_extinct += avg_extinct[i];
		}
	}



	cafe_log(cafe_param,"*******************  ALL *************************\n");
	cafe_log(cafe_param,">> DATA\n");
	histogram_print(phist_data[0], cafe_param->flog);
	if ( cafe_param->flog != stdout) histogram_print(phist_data[0], NULL);
	cafe_log(cafe_param, "Total Extinct: %d\n", total_extinct );
	cafe_log(cafe_param,">> SIMULATION\n");
	__hg_print_sim_extinct(phist_sim_n,phist_sim,0,phist_tmp,cnt,num_trials);

	memory_free(cnt);

	for ( i = 0 ; i < pcafe->rfsize ; i++ )
	{
		if ( phist_data[i] ) 
		{
			histogram_free( phist_data[i] );
			histogram_free( phist_sim[i] );
			for ( t = 0 ; t < num_trials ; t++ )
			{
				histogram_free(phist_sim_n[t][i]);
			}
		}
	}

	histogram_free(phist_tmp);
	memory_free(phist_data);
	memory_free(phist_sim);
	memory_free_2dim((void**)phist_sim_n,num_trials, 0, NULL);

	memory_free(roots_size);
	memory_free(roots_extinct);
	memory_free(roots_num);
	memory_free(data);
	memory_free(avg_extinct);
	memory_free(simdata);
	return 0;
}

int cafe_cmd_lh_test(int argc, char* argv[])
{
	pArrayList pargs = cafe_shell_build_argument(argc, argv);
	pArgument parg;
	double lambda;
	char dir[255];
	if ( (parg = cafe_shell_get_argument( "-d", pargs) ) )
	{
		strcpy( dir, parg->argv[0] );
	}
	if ( (parg = cafe_shell_get_argument( "-l", pargs) ) )
	{
		sscanf( parg->argv[0], "%lf", &lambda );	
	}
	char tree[1024];
	if ( (parg = cafe_shell_get_argument( "-t", pargs) ) )
	{
		strcpy( tree, parg->argv[0] );
	}
	FILE* fout = stdout;
	if ( (parg = cafe_shell_get_argument( "-o", pargs) ) )
	{
		if ( (fout=fopen(parg->argv[0],"w")) == NULL )
		{
			perror( parg->argv[0]);
			return -1;
		}
	}
	arraylist_free(pargs, free);

	pArrayList files = dir_file_list(dir, NULL);
	pString pstr_cafe = phylogeny_string( (pTree)cafe_param->pcafe, NULL );
	int i, j;
	for ( i = 0 ; i < files->size ; i++ )
	{
		char* fname = (char*)files->array[i];
		if ( fname[0] == '.' ) continue;
		cafe_shell_dispatch_commandf("load -i %s/%s -p 0.01 -t 10 -l %s", 
				 dir, fname, cafe_param->str_log ? cafe_param->str_log->buf : "stdout" );
		cafe_shell_dispatch_commandf("tree %s", pstr_cafe->buf );
		cafe_shell_dispatch_commandf("lambda -s -l %lf", lambda );
		fprintf(fout,"\t%lf\t%lf", cafe_get_likelihood(cafe_param), cafe_param->lambda[0] );
		cafe_family_reset_maxlh(cafe_param->pfamily);
		cafe_shell_dispatch_commandf("lambda -s -v %lf -t %s", lambda, tree );
		fprintf(fout,"\t%lf", cafe_get_likelihood(cafe_param) );
		for ( j = 0 ; j < cafe_param->num_lambdas ; j++ )
		{
			fprintf(fout, "\t%lf", cafe_param->lambda[j] );
		}
		fprintf(fout,"\n");
		fflush(fout);
		cafe_shell_clear_param(0);
	}
	fclose(fout);
	string_free( pstr_cafe );
	arraylist_free(files, free);	
	return 0;
}
