#include<cafe.h>
#include<stdlib.h>
#include<io.h>
#include<math.h>

extern void __phylogeny_free_node(pTree ptree, pTreeNode ptnode, va_list ap);

pTreeNode cafe_tree_new_empty_node(pTree ptree)
{
	pCafeTree pcafe = (pCafeTree)ptree;
	pCafeNode pcnode = 	(pCafeNode)memory_new_with_clear(1, sizeof(CafeNode) );
	pcnode->likelihoods = (double*)memory_new_with_clear(pcafe->size_of_factor,sizeof(double));
	pcnode->viterbi = (int*)memory_new_with_clear(pcafe->size_of_factor,sizeof(int));
	pcnode->lambda = pcafe->lambda;
	pcnode->familysize = -1;
	phylogeny_clear_node((pPhylogenyNode)pcnode);
	return (pTreeNode)pcnode;		
}

void cafe_tree_set_parameters(pCafeTree pcafe, int familysizes[], 
		                int rootfamilysizes[], double lambda)
{
	int i;
	pArrayList nlist = pcafe->super.nlist;
	for ( i = 0 ; i < 2 ; i++ )
	{
		pcafe->familysizes[i] = familysizes[i];
		pcafe->rootfamilysizes[i] = rootfamilysizes[i];
	}
	pcafe->lambda = lambda;
	pcafe->rfsize = rootfamilysizes[1] - rootfamilysizes[0] + 1;

	int rsize = rootfamilysizes[1] - rootfamilysizes[0]  + 1;
	int fsize = familysizes[1] - familysizes[0]  + 1;
	int max_size = rsize > fsize ? rsize : fsize;
//	((pCafeNode)pcafe->super.root)->lambda = lambda;
	if ( pcafe->size_of_factor < max_size )
	{
		pcafe->size_of_factor = max_size;
		int idx;
		for ( idx = 0 ; idx < 2 ;  idx++ )
		{
			pcafe->factors[idx] = (double*)memory_realloc(pcafe->factors[idx], pcafe->size_of_factor,sizeof(double));
		}
		for ( i = 0;  i < nlist->size ; i++ )
		{
			pCafeNode pcnode = (pCafeNode)nlist->array[i];
			pcnode->likelihoods = (double*)memory_realloc(pcnode->likelihoods,pcafe->size_of_factor,sizeof(double));
			pcnode->viterbi = (int*)memory_realloc(pcnode->viterbi,pcafe->size_of_factor,sizeof(int));
		}
	}
}

pTree __cafe_tree_new(tree_func_node_new new_tree_node_func, va_list ap)
{
	pCafeTree pcafe = (pCafeTree)memory_new(1,sizeof(CafeTree));
	pcafe->size_of_factor = va_arg(ap,int);
	tree_new_fill((pTree)pcafe, cafe_tree_new_empty_node);
	pcafe->super.size = sizeof(CafeTree);
	return (pTree)pcafe;
}


void cafe_tree_parse_node(pTree ptree, pTreeNode ptnode)
{
	pCafeNode pcnode = (pCafeNode)ptnode;
	char* name = pcnode->super.name;
	if ( name == NULL ) return;
	char* familysize = (char*)index(name,'_');
	if ( familysize )
	{
		*familysize++ = '\0';
		pcnode->familysize = atoi(familysize);
	}
}

pCafeTree cafe_tree_new(char* sztree, int familysizes[], 
		                int rootfamilysizes[], double lambda)
{
	int i;

	int rsize = rootfamilysizes[1] - rootfamilysizes[0]  + 1;
	int fsize = familysizes[1] - familysizes[0]  + 1;

	pCafeTree pcafe = (pCafeTree)phylogeny_load_from_string(
			             		sztree, 
								__cafe_tree_new, 
			             		cafe_tree_new_empty_node, 
								cafe_tree_parse_node, 
								rsize > fsize ? rsize : fsize );
	pcafe->pbdc_array = NULL;
	for ( i = 0 ; i < 2 ; i++ )
	{
		pcafe->familysizes[i] = familysizes[i];
		pcafe->rootfamilysizes[i] = rootfamilysizes[i];
	}
	pcafe->lambda = lambda;
	pcafe->rfsize = rootfamilysizes[1] - rootfamilysizes[0] + 1;
	((pCafeNode)pcafe->super.root)->lambda = lambda;
	int idx;
	for ( idx = 0 ; idx < 2 ;  idx++ )
	{
		pcafe->factors[idx] = (double*)memory_new(pcafe->size_of_factor,sizeof(double));
	}
	tree_build_node_list((pTree)pcafe);
	return pcafe;
}

pCafeTree cafe_tree_new_from_file(char* fname, int familysizes[], 
		                          int rootfamilysizes[], double lambda )
{
	int i;
	char* sztree = file_read_all(fname);	
	for( i = 0 ; i < sztree[i]; i++ )
	{
		if ( sztree[i] == '\n' ) sztree[i] = ' ';
	}
	pCafeTree ptree = cafe_tree_new(sztree, familysizes, rootfamilysizes, lambda );
	free(sztree);
	return ptree;
}

void __cafe_tree_free_node(pTree ptree, pTreeNode ptnode, va_list ap)
{
	pCafeNode pcnode = (pCafeNode)ptnode;
	if ( pcnode->likelihoods ) free(pcnode->likelihoods);	
	if ( pcnode->viterbi ) free(pcnode->viterbi);
	__phylogeny_free_node(ptree,ptnode,ap);		
}


void cafe_tree_free(pCafeTree pcafe)
{
	free(pcafe->factors[0]); free(pcafe->factors[1]);
	if ( pcafe->super.nlist )
	{
		int i;
		for ( i = 0 ; i < pcafe->super.nlist->size ; i++ )
		{
			__cafe_tree_free_node((pTree)pcafe, (pTreeNode)pcafe->super.nlist->array[i], NULL );
		}
	}
	else
	{
		tree_traveral_prefix((pTree)pcafe,__cafe_tree_free_node);
	}
	pTree  ptree = (pTree)pcafe;
	if ( *ptree->count == 0 && ptree->data ) vector_free( ((pVector)ptree->data), free );
	tree_free(ptree);
}

/*******************************************************************************
 *	Tree Output
 *******************************************************************************/

void cafe_tree_string_name(pString pstr, pPhylogenyNode ptnode)
{
	char buf[100];
	int familysize =  ((pCafeNode)ptnode)->familysize;
	int idx = 0;
	if ( ptnode->name || familysize >= 0 )
	{
		if ( ptnode->name ) idx = sprintf(buf,"%s", ptnode->name);
		if ( familysize >= 0) sprintf(&buf[idx],"_%d", familysize );
		string_add(pstr,buf);
	}
}

void cafe_tree_string_familysize_lambda(pString pstr, pPhylogenyNode ptnode)
{
	int familysize =  ((pCafeNode)ptnode)->familysize;
	if ( ptnode->branchlength <= 0 ) return;
	if ( ptnode->name ) string_fadd(pstr,"%s", ptnode->name );
	if ( familysize >= 0) string_fadd(pstr,"<%d>", familysize );
	double lambda = ((pCafeNode)ptnode)->lambda;
	string_fadd(pstr,"_%lf", lambda );
}

void cafe_tree_string_familysize(pString pstr, pPhylogenyNode ptnode)
{
	int familysize =  ((pCafeNode)ptnode)->familysize;
	if ( ptnode->branchlength <= 0 ) return;
	if ( ptnode->name ) string_fadd(pstr,"%s", ptnode->name );
	if ( familysize >= 0) string_fadd(pstr,"< %d >", familysize );
}

void cafe_tree_string_lambda(pString pstr, pPhylogenyNode ptnode)
{
	if ( ptnode->branchlength <= 0 ) return;
	if ( ptnode->name ) string_fadd(pstr,"%s", ptnode->name );
	string_fadd(pstr,"_%lf", ((pCafeNode)ptnode)->lambda );
}

void cafe_tree_string_id(pString pstr, pPhylogenyNode pnode)
{
	if ( pnode->name ) string_fadd(pstr,"%s", pnode->name );
	string_fadd(pstr,"<%d>", pnode->super.id );
}

pString cafe_tree_string_with_familysize_lambda(pCafeTree pcafe)
{
	return phylogeny_string((pTree)pcafe,cafe_tree_string_familysize_lambda);
}

pString cafe_tree_string_with_lambda(pCafeTree pcafe)
{
	return phylogeny_string((pTree)pcafe,cafe_tree_string_lambda);
}

pString cafe_tree_string_with_familysize(pCafeTree pcafe)
{
	return phylogeny_string((pTree)pcafe,cafe_tree_string_familysize);
}

pString cafe_tree_string_with_id(pCafeTree pcafe)
{
	return phylogeny_string((pTree)pcafe,cafe_tree_string_id);
}


pString cafe_tree_string(pCafeTree pcafe)
{
	return phylogeny_string((pTree)pcafe,cafe_tree_string_name);
}

void cafe_tree_string_print(pCafeTree pcafe)
{
	pString pstr = cafe_tree_string(pcafe);	
	printf("%s\n", pstr->buf );
	string_free(pstr);
}


double cafe_tree_mp_remark(pString pstr, pTree ptree, pMetapostConfig pmc, va_list ap)
{
	char* title = va_arg(ap,char*);
	string_add( pstr, "\n% annotation\n");
	string_fadd( pstr, "label( btex  etex, (0.1u, %fu));\n", pmc->height + 0.7 ) ;
	string_fadd( pstr, "label( btex %s etex, (0.1u, %fu));\n", title, pmc->height + 0.5 ) ;
	return pmc->height + 0.5;
}

double cafe_tree_mp_annotation(pString pstr, pTreeNode pnode, pMetapostConfig pmc, va_list ap)
{
	pCafeNode pcnode = (pCafeNode)pnode;
    string_add( pstr, ";\n");
	if ( pcnode->familysize >= 0 )
	{
		char* title = va_arg(ap,char*);
		pCafeParam param = va_arg(ap,pCafeParam);
		int fid = va_arg(ap,int);
		string_fadd( pstr, "label.urt( btex %d ", pcnode->familysize, pnode->id );
		if ( param && param->viterbiPvalues  && !tree_is_leaf( pnode )  )
		{
			int bidx = 2*(pnode->id/2); 
			if(  param->viterbiPvalues[bidx][fid] != -1 )
			{
//				string_fadd( pstr,"\\small{(%4.3f\\%%, %4.3f\\%%)} ", 
				string_fadd( pstr,"\\small{(%4.3f, %4.3f)} ", 
						    param->viterbiPvalues[bidx][fid], 
							param->viterbiPvalues[bidx+1][fid] );
			}
		}
		string_fadd( pstr, "etex, p[%d]);\n", pnode->id );
	}
	double last = 0;
	if ( pnode->parent )
	{
		string_fadd( pstr, "xpart mid[%d] = xpart(p[%d]);\n", pnode->id, pnode->id );
		string_fadd( pstr, "ypart mid[%d] = (ypart(p[%d])+ypart(p[%d]))/2;\n", pnode->id, pnode->id, pnode->parent->id );
		string_fadd( pstr, "label.rt( btex $l = %g$ ", ((pPhylogenyNode)pnode)->branchlength );
		string_fadd( pstr, "etex, mid[%d]);\n", pnode->id  );
		string_fadd( pstr, "label.rt( btex $\\lambda=%f$ ", pcnode->lambda );
		last -= 0.15;
		string_fadd( pstr, "etex, mid[%d] + (0,%fu));\n",  pnode->id, last );
	}
	return last;
}

pMetapostConfig cafe_tree_get_default_mpconfig(int id, double width, double height )
{
	pMetapostConfig pmc = MEMORY_NEW(1,MetapostConfig);
	pmc->id = id;
	pmc->unit = MP_UNIT_IN;
	pmc->dir = MP_DIR_VERTICAL;
	pmc->shape = MP_SHAPE_RECT | MP_SHAPE_MOST_CENTER;
	pmc->fmod = cafe_tree_mp_annotation;
	pmc->fremark = cafe_tree_mp_remark;
	pmc->width = width;
	pmc->height = height;
	return pmc;
}

pString cafe_tree_metapost(pCafeTree pcafe, int id, char* title, double width, double height )
{
	pMetapostConfig pmc = cafe_tree_get_default_mpconfig(id,width,height);
	pString pstr = phylogeny_to_mp( (pTree)pcafe, pmc, title, NULL, 0);
	memory_free(pmc);
	return pstr;
}

/*******************************************************************************
 *	Viterbi
 *******************************************************************************/

void __cafe_tree_node_compute_viterbi(pTree ptree, pTreeNode ptnode, va_list ap )
{
	pCafeTree pcafe = (pCafeTree)ptree;
	pCafeNode pcnode = (pCafeNode)ptnode;
	//double lambda = pcafe->lambda;

	int size;
	int s,c,i,j; 
	int* rootfamilysizes;
	int* familysizes;

	if ( tree_is_leaf(ptnode) )
	{
		if ( tree_is_root(ptree, ptnode->parent) )
		{
			rootfamilysizes = pcafe->rootfamilysizes;
		}
		else
		{
			rootfamilysizes = pcafe->familysizes;
		}
		for ( s = rootfamilysizes[0], i = 0; s <= rootfamilysizes[1] ; s++, i++ )
		{
			pcnode->likelihoods[i] = pcnode->bd[s][pcnode->familysize];
		}
	}
	else
	{
		if ( tree_is_root(ptree,ptnode) )
		{
			rootfamilysizes = pcafe->rootfamilysizes;
			familysizes = pcafe->familysizes;
		}
		else
		{
			rootfamilysizes = familysizes = pcafe->familysizes;
		}
		int idx;
		double *factors[2] = { NULL, NULL };
		pCafeNode child[2] = { (pCafeNode)((pTreeNode)pcnode)->children->head->data, 
							   (pCafeNode)((pTreeNode)pcnode)->children->tail->data };
		for ( idx = 0 ; idx < 2 ; idx++ )
		{
			if ( tree_is_leaf((pTreeNode)child[idx] ) )
			{
				factors[idx] = (double*)child[idx]->likelihoods;
			}
			else
			{
				factors[idx] = pcafe->factors[idx];
				memset( factors[idx], 0, pcafe->size_of_factor*sizeof(double));
				double** bd = child[idx]->bd;
				for( s = rootfamilysizes[0], i = 0 ; s <= rootfamilysizes[1] ; s++, i++ )
				{
					double tmp = 0;
					for( c = familysizes[0], j = 0 ; c <= familysizes[1] ; c++, j++ )
					{
						tmp = bd[s][c] * child[idx]->likelihoods[j];
						if ( tmp > factors[idx][i] )
						{
							factors[idx][i] = tmp;
							child[idx]->viterbi[i] = j;
						}
					}
				}
			}
		}
		size = rootfamilysizes[1] - rootfamilysizes[0] + 1;
		for ( i = 0 ; i < size ; i++ )
		{
			pcnode->likelihoods[i] = factors[0][i] * factors[1][i];
		}
//		printf("%s : %d, %d\n", pcnode->super.name, pcnode->super.branchlength, i );
	}
}

void __cafe_tree_node_backtrack_viterbi(pTree ptree, pTreeNode ptnode, va_list ap )
{
	if ( tree_is_leaf(ptnode) ) return;
	pCafeTree pcafe = (pCafeTree)ptree;
	pCafeNode pcnode = (pCafeNode)ptnode;
	if ( ptree->root == ptnode )
	{
		pcnode->familysize = pcafe->rootfamilysizes[0] + __maxidx( pcnode->likelihoods, pcafe->rfsize );
	}
	else
	{
		pCafeNode pcparent = (pCafeNode)ptnode->parent;
		int base = tree_is_root(ptree,(pTreeNode)pcparent) ? pcafe->rootfamilysizes[0] : pcafe->familysizes[0];
		pcnode->familysize = pcnode->viterbi[pcparent->familysize - base] + pcafe->familysizes[0];
	}
}

void cafe_tree_viterbi(pCafeTree pcafe)
{
	if ( pcafe->super.postfix )
	{
		pArrayList postfix = pcafe->super.postfix;
		pArrayList prefix = pcafe->super.prefix;
		int i = 0;
		for ( i = 0 ; i < postfix->size; i++ )
		{
			__cafe_tree_node_compute_viterbi((pTree)pcafe, (pTreeNode)postfix->array[i], NULL );
		}
		for ( i = 0 ; i < prefix->size; i++ )
		{
			__cafe_tree_node_backtrack_viterbi((pTree)pcafe, (pTreeNode)prefix->array[i], NULL );
		}
	}
	else
	{
		tree_traveral_postfix((pTree)pcafe, __cafe_tree_node_compute_viterbi);
		tree_traveral_prefix((pTree)pcafe, __cafe_tree_node_backtrack_viterbi);
	}
}

/*******************************************************************************
 *	Likelihood
 *******************************************************************************/

void __cafe_tree_node_compute_likelihood(pTree ptree, pTreeNode ptnode, va_list ap )
{
	pCafeTree pcafe = (pCafeTree)ptree;
	pCafeNode pcnode = (pCafeNode)ptnode;

	int size;
	int s,c,i,j; 
	int* rootfamilysizes;
	int* familysizes;


	if ( tree_is_leaf(ptnode) )
	{
		if ( tree_is_root(ptree, ptnode->parent) )
		{
			rootfamilysizes = pcafe->rootfamilysizes;
		}
		else
		{
			rootfamilysizes = pcafe->familysizes;
		}
		for ( s = rootfamilysizes[0], i = 0; s <= rootfamilysizes[1] ; s++, i++ )
		{
			pcnode->likelihoods[i] = pcnode->bd[s][pcnode->familysize];
		}
	}
	else
	{
		if ( tree_is_root(ptree,ptnode) )
		{
			rootfamilysizes = pcafe->rootfamilysizes;
			familysizes = pcafe->familysizes;
		}
		else
		{
			rootfamilysizes = familysizes = pcafe->familysizes;
		}
		int idx;
		double *factors[2] = { NULL, NULL }; 
		pCafeNode child[2] = { (pCafeNode)((pTreeNode)pcnode)->children->head->data, 
							   (pCafeNode)((pTreeNode)pcnode)->children->tail->data };
		for ( idx = 0 ; idx < 2 ; idx++ )
		{
			if ( tree_is_leaf((pTreeNode)child[idx] ) )
			{
				factors[idx] = child[idx]->likelihoods;
			}
			else
			{	
				factors[idx] = pcafe->factors[idx];
				memset( factors[idx], 0, pcafe->size_of_factor*sizeof(double));
				double** bd = child[idx]->bd;
				for( s = rootfamilysizes[0], i = 0 ; s <= rootfamilysizes[1] ; s++, i++ )
				{
					for( c = familysizes[0], j = 0 ; c <= familysizes[1] ; c++, j++ )
					{
						factors[idx][i] += bd[s][c] * child[idx]->likelihoods[j];
					}
				}
			}
		}
		size = rootfamilysizes[1] - rootfamilysizes[0] + 1;
		for ( i = 0 ; i < size ; i++ )
		{
			pcnode->likelihoods[i] = factors[0][i] * factors[1][i];
		}
	}
}

double* cafe_tree_likelihood(pCafeTree pcafe) 
{
	if ( pcafe->super.postfix )
	{
		int i = 0;
		pArrayList postfix = ((pTree)pcafe)->postfix;
		for ( i = 0 ; i < postfix->size; i++ )
		{
			__cafe_tree_node_compute_likelihood((pTree)pcafe, (pTreeNode)postfix->array[i], NULL );
		}
	}
	else
	{
		tree_traveral_postfix((pTree)pcafe, __cafe_tree_node_compute_likelihood);
	}
	return ((pCafeNode)pcafe->super.root)->likelihoods;
}

/*******************************************************************************
 *	Copy and cut
 *******************************************************************************/

void __cafe_tree_set_birthdeath(pTree ptree, pTreeNode ptnode, va_list ap )
{
	if ( tree_is_root( ptree, ptnode ) ) return;
	pCafeNode pcnode = (pCafeNode)ptnode;
	pcnode->bd = birthdeath_cache_get_matrix(((pCafeTree)ptree)->pbdc_array, pcnode->super.branchlength, pcnode->lambda);
}

void cafe_tree_set_birthdeath(pCafeTree pcafe)
{
	if ( pcafe->super.nlist )
	{
		int i;
		pArrayList nlist = pcafe->super.nlist;
		for (  i = 0; i < nlist->size ; i++ )
		{
			pCafeNode pcnode = (pCafeNode)nlist->array[i];
			if ( pcnode->super.branchlength <= 0 ) continue;
			pcnode->bd = birthdeath_cache_get_matrix(pcafe->pbdc_array, pcnode->super.branchlength, pcnode->lambda);
		}
	}
	else
	{
		tree_traveral_prefix((pTree)pcafe, __cafe_tree_set_birthdeath );
	}
}

void cafe_tree_node_copy(pTreeNode psrc, pTreeNode pdest)
{
	pCafeNode pcsrc, pcdest;
	pcsrc = (pCafeNode)psrc;
	pcdest = (pCafeNode)pdest;
	phylogeny_node_copy(psrc,pdest);
	pcdest->lambda = pcsrc->lambda;
	pcdest->familysize = pcsrc->familysize;
	pcdest->bd = pcsrc->bd;
}

void __cafe_tree_copy_new_fill(pCafeTree psrc, pCafeTree pdest )
{
	pdest->pbdc_array = psrc->pbdc_array;
	pdest->size_of_factor = psrc->size_of_factor;
	pdest->familysizes[0] = psrc->familysizes[0];
	pdest->familysizes[1] = psrc->familysizes[1];
	pdest->rootfamilysizes[0] = psrc->rootfamilysizes[0];
	pdest->rootfamilysizes[1] = psrc->rootfamilysizes[1];
	pdest->lambda = psrc->lambda;
	pdest->rfsize = psrc->rfsize;

	int idx;
	for ( idx = 0 ; idx < 2 ;  idx++ )
	{
		pdest->factors[idx] = (double*)memory_new( pdest->size_of_factor, sizeof(double) );
	}
}

pCafeTree cafe_tree_copy(pCafeTree psrc)
{
	pCafeTree pcafe = (pCafeTree)tree_copy((pTree)psrc, 
			                    cafe_tree_new_empty_node, 
							    cafe_tree_node_copy );
	__cafe_tree_copy_new_fill(psrc,pcafe);
	tree_build_node_list((pTree)pcafe);
	return pcafe;
}

pCafeTree cafe_tree_split(pCafeTree pcafe, int idx )
{
	pCafeTree psub = (pCafeTree)phylogeny_split_tree((pTree)pcafe,idx, __cafe_tree_free_node );
	if ( psub )
	{
		__cafe_tree_copy_new_fill(pcafe,psub);
	}
	cafe_tree_set_birthdeath(pcafe);
	cafe_tree_set_birthdeath(psub);
	return psub;
}

/*******************************************************************************
 *	Random family size
 *******************************************************************************/

void __cafe_tree_node_random_familysize(pTree ptree, pTreeNode pnode, va_list ap)
{
	if ( tree_is_root(ptree,pnode) ) return;

	double rnd = unifrnd();					
	double cumul = 0;
	pCafeTree pcafe = (pCafeTree)ptree;
	pCafeNode pcnode = (pCafeNode)pnode;
	int maxFamilysize = pcafe->pbdc_array->maxFamilysize;
	int s = ((pCafeNode)pnode->parent)->familysize;
	int c;
	for ( c = 0 ; c < maxFamilysize ; c++ )
	{
		cumul += pcnode->bd[s][c];
		if ( cumul >= rnd ) break;
	}
	pcnode->familysize = c;
}

int cafe_tree_random_familysize(pCafeTree pcafe, int rootFamilysize )
{
	int max = 0;
	((pCafeNode)pcafe->super.root)->familysize = rootFamilysize;
	if ( pcafe->super.prefix )
	{
		pArrayList prefix = pcafe->super.prefix;
		int i;
		for ( i = 0 ; i < prefix->size ; i++ )
		{
			pCafeNode pnode = (pCafeNode)prefix->array[i];
			__cafe_tree_node_random_familysize((pTree)pcafe,(pTreeNode)pnode ,NULL);
			if ( max < pnode->familysize )
			{
				max = pnode->familysize;
			}
		}
	}
	else
	{
		tree_traveral_prefix( (pTree)pcafe, __cafe_tree_node_random_familysize);
	}
	return max;
}

/*
int __cmp_double(const void* a, const void* b)
{
	if ( *(double*)a > *(double*)b ) return 1;
	if ( *(double*)a == *(double*)b ) return 0;
	return -1;
}
*/

double* cafe_tree_random_probabilities(pCafeTree pcafe, int rootFamilysize, int trials )
{
	int old_rfsize = pcafe->rfsize;
	int old_rfsizes[2] = { pcafe->rootfamilysizes[0], pcafe->rootfamilysizes[1] };
	int old_fsizes[2] = { pcafe->familysizes[0], pcafe->familysizes[1] };

	pcafe->rfsize = 1;
	pcafe->rootfamilysizes[0] = rootFamilysize;
	pcafe->rootfamilysizes[1] = rootFamilysize;

	double* probs = (double*)memory_new(trials, sizeof(double));	
	int i;
	for ( i = 0 ; i < trials ; i++ )
	{
		int max = cafe_tree_random_familysize(pcafe,rootFamilysize);		
		if ( pcafe->super.nlist )
		{
			pcafe->familysizes[1] = MIN( max + MAX(50,max/5) , pcafe->familysizes[1] );
	//		printf("%d: %d(%d)\n", i, pcafe->familysizes[1], max );
		}
		cafe_tree_likelihood(pcafe);
		probs[i] = ((pCafeNode)pcafe->super.root)->likelihoods[0];
	}

	pcafe->rfsize = old_rfsize;
	pcafe->rootfamilysizes[0] = old_rfsizes[0];
	pcafe->rootfamilysizes[1] = old_rfsizes[1];
	pcafe->familysizes[0] = old_fsizes[0];
	pcafe->familysizes[1] = old_fsizes[1];

	qsort(probs, trials, sizeof(double), __cmp_double);
	return probs;
}


pArrayList cafe_tree_conditional_distribution(pCafeTree pcafe, int range[], int trial )
{
	int s;
	int size = range[1] - range[0]+1;
	pArrayList pal = arraylist_new(size);
	for ( s = range[0] ; s <= range[1]; s++ )
	{
		arraylist_add(pal, cafe_tree_random_probabilities(pcafe,s,trial) );
	}
	return pal;
}

/*
 * cdlen: length of conddist : number of trials
 */
double cafe_tree_p_value(pCafeTree pcafe, double lh, double* conddist, int cdlen)
{
	return pvalue(lh,conddist, cdlen);
}

double* cafe_tree_p_values(pCafeTree pcafe,double* pvalues, pArrayList pconddist, int cdlen)
{
	double* lh = cafe_tree_likelihood(pcafe);
	int s;
	for( s = 0 ; s < pcafe->rfsize; s++ )
	{
		pvalues[s] = pvalue( lh[s] , (double*)pconddist->array[s], cdlen);
	}
	return pvalues;
}

/*
 * cdlen: length of conddist : number of trials
 */
double* cafe_tree_p_values_without_p(pCafeTree pcafe, pArrayList pconddist, int cdlen)
{
	return cafe_tree_p_values(pcafe,
				(double*)memory_new(pcafe->rfsize, sizeof(double)), pconddist, cdlen );
}

double** cafe_tree_p_values_of_two_trees(pCafeTree pcafe1, pCafeTree pcafe2,    
										 double** pvalues,
		                                   pArrayList pconddist1, pArrayList pconddist2,
										   int cdlen )
{
	double* lh1 = cafe_tree_likelihood(pcafe1);
	double* lh2 = cafe_tree_likelihood(pcafe2);
	int s1, s2, t;
	double p;
	for( s2 = 0 ; s2 < pcafe1->rfsize ; s2++ )
	{
		for ( s1 = 0; s1 < pcafe1->rfsize; s1++ )
		{
			p = 0;
			for ( t = 0 ; t < cdlen ; t++ )
			{
				p += pvalue( lh1[s1]*lh2[s2]/((double*)pconddist2->array[s2])[t] ,
						    (double*)pconddist1->array[s1], cdlen );
			}
			pvalues[s1][s2] = p/cdlen;
		}
	}
	return pvalues;
}

double** cafe_tree_p_values_of_two_trees_without_p(pCafeTree pcafe1, pCafeTree pcafe2,
							pArrayList pconddist1, pArrayList pconddist2, int cdlen )
{
	int size = pcafe1->rfsize;
	double **pvalues = (double**)memory_new_2dim(size,size,sizeof(double));
	return cafe_tree_p_values_of_two_trees(pcafe1, pcafe2, pvalues, pconddist1, pconddist2, cdlen );
}
