#include<tree.h>
#include<io.h>

pTreeNode paml_tree_new_empty_node(pTree ptree)
{
	pPamlNode pnode = MEMORY_NEW(1,PamlNode);	
	pnode->N = -1;
	pnode->accudS = 0;
	phylogeny_clear_node((pPhylogenyNode)pnode);
	return (pTreeNode)pnode;
}

pTree __paml_tree_new(tree_func_node_new new_tree_node_func, va_list ap)
{
	pPamlTree ptree = MEMORY_NEW(1, PamlTree);
	tree_new_fill((pTree)ptree, paml_tree_new_empty_node);
	ptree->super.size = sizeof(PamlTree);
	return (pTree)ptree;
}

pPamlTree paml_tree_new(char* file)
{
	pPamlTree ptree = NULL;
	FILE* fp;
	if (  ( fp = fopen(file,"r") ) == NULL )
	{
		fprintf(stderr, "ERROR(paml): Cannot open %s\n", file );
		perror("Error\n");
		return NULL;
	}

	pString pstr = string_new();

	int num_genes = -1;
	char** gene_names = NULL;
	double lnL;
	int genecnt = 0;
	double* nei_gojobori = NULL;

	while( file_read_line(pstr, fp) > 0 )
	{
		if ( num_genes == -1 && strncmp( pstr->buf, "ns", 2 ) == 0 )
		{
			sscanf( pstr->buf, "ns = %d", &num_genes );	
			gene_names = MEMORY_NEW(num_genes, char*);
		}
		else if ( pstr->buf[0] == '#' )
		{
			char* n = index(pstr->buf, ' ');
			*index(++n,' ') = '\0';
			gene_names[genecnt] = MEMORY_NEW(strlen(n)+1, char);
			strcpy( gene_names[genecnt], n );
			genecnt++;
		}
		else if ( strncmp( pstr->buf, "lnL", 3) == 0 )
		{
			char* n = index( pstr->buf, '-' );
			*index(n,' ') = '\0';
			sscanf( n, "%lf", &lnL );
		}
		else if ( strncmp(pstr->buf, "Nei &", 5 ) == 0 )
		{
			int i,j;
			while(1)
			{
				file_read_line(pstr, fp);
				if ( pstr->buf[0] == '\n' ) break;
			}
			file_read_line(pstr, fp);
			int num_ws = (int)(genecnt * ( genecnt - 1 ) / 2.0);
			nei_gojobori = MEMORY_NEW( num_ws, double);
			int sum = 0;
			for ( i = 0 ; i < genecnt - 1; i++ )
			{
				file_read_line(pstr, fp);
				char* ptr = &pstr->buf[20];
				double w, dN, dS;
				for ( j = 0 ; j <= i ; j++, sum++ )
				{
				 	sscanf( ptr, "%lf (%lf %lf)", &w, &dN, &dS );
					nei_gojobori[sum] = w;
					ptr = index( ptr, ')');
					if ( ptr ) ptr++;
				}	
			}
		}
		else if ( ptree == NULL && strncmp( pstr->buf, "tree len", 8 ) == 0 )
		{
			file_read_line(pstr, fp);
			file_read_line(pstr, fp);
			ptree = (pPamlTree)phylogeny_load_from_string(pstr->buf,__paml_tree_new, paml_tree_new_empty_node, NULL );
			if ( ptree == NULL ) return NULL;
			tree_build_node_list((pTree)ptree);
			ptree->num_genes = num_genes;
			ptree->gene_names = gene_names;
			ptree->lnL = lnL;
			ptree->nei_gojobori = nei_gojobori;
			pArrayList nlist = ptree->super.prefix;
			ptree->nodelist = MEMORY_NEW( nlist->size, pPamlNode );
			int id = num_genes;
			int i;
			for ( i = 0 ; i < nlist->size; i++ )
			{
				pPhylogenyNode pnode = (pPhylogenyNode)nlist->array[i];
				if ( pnode->super.children )
				{
					ptree->nodelist[id++] = (pPamlNode)pnode;
				}
				else
				{
					int idx = atoi(pnode->name)-1;
					ptree->nodelist[idx] = (pPamlNode)pnode;
					free(pnode->name);
					pnode->name = gene_names[idx];
				}
			}
		}
		else if ( strncmp(pstr->buf, "dN/dS", 5) == 0 )
		{
			char* n = index(pstr->buf, '=' );
			*index(++n,')') = '\0';
			ptree->model = atoi(n);
			if( ptree->model == 4 )
			{
				int i;
				ptree->omega = MEMORY_NEW( ptree->model*2, double );
				ptree->prob = MEMORY_NEW( ptree->model, double );
				while(1)
				{
					file_read_line(pstr, fp);
					if ( strncmp(pstr->buf, "prop", 4 ) == 0 )
					{
						pArrayList list = string_pchar_space_split(pstr->buf);
						for( i = 1 ; i <= 4 ; i++ )
						{
							sscanf( (char*)list->array[i], "%lf", &ptree->prob[i-1] );
						}
						arraylist_free(list, NULL);
						file_read_line(pstr, fp);
						list = string_pchar_space_split(pstr->buf);
						for( i = 2 ; i <= 5 ; i++ )
						{
							sscanf( (char*)list->array[i], "%lf", &ptree->omega[i-2] );
						}
						arraylist_free(list, NULL);
						file_read_line(pstr, fp);
						list = string_pchar_space_split(pstr->buf);
						for( i = 2 ; i <= 5 ; i++ )
						{
							sscanf( (char*)list->array[i], "%lf", &ptree->omega[4+i-2] );
						}
						arraylist_free(list, NULL);
						break;
					}
				}
			}
			else
			{
				ptree->omega = MEMORY_NEW( ptree->model, double );
				ptree->prob = MEMORY_NEW( ptree->model, double );
				file_read_line(pstr, fp);
	
				file_read_line(pstr, fp);
				pArrayList list = string_pchar_space_split(pstr->buf);
				int i;
				for ( i = 1 ; i < list->size ; i++ )
				{
					ptree->prob[i-1] = atof((char*)list->array[i]);
				}
				arraylist_free(list, NULL);
	
				file_read_line(pstr, fp);
				int j;
				for ( i = 5 , j = 0; i < pstr->length ; i+=8 )
				{
					char t = pstr->buf[i+8];
					pstr->buf[i+8] = '\0'; 
					ptree->omega[j++] = atof(&pstr->buf[i]);
					pstr->buf[i+8] = t;
				}
			}
		}
		else if ( strncmp(pstr->buf, " branch", 6 ) == 0 )
		{
			file_read_line(pstr, fp);
			do
			{
				file_read_line(pstr, fp);
				if( strlen(pstr->buf) < 2 ) break;
				pstr->buf[64] = '\0';
				pArrayList list = string_pchar_space_split(pstr->buf);
				pPamlNode pnode;
				int from, to;
				sscanf( (char*)list->array[0], "%d..%d", &from,&to);
				from--; to--;
				if ( from < num_genes ) pnode = ptree->nodelist[from];
				else if ( to < num_genes || from < to ) pnode = ptree->nodelist[to];
				else pnode = ptree->nodelist[from];
				pnode->N = atof((char*)list->array[2]);
				pnode->S = atof((char*)list->array[3]);
				pnode->omega = atof((char*)list->array[4]);
				pnode->dN = atof((char*)list->array[5]);
				pnode->dS = atof((char*)list->array[6]);
				ptree->meanOmega = pnode->omega;
			}while( 1 );
		}
	}
	string_free(pstr);
	fclose(fp);
	return ptree;
}


double paml_tree_mp_remark(pString pstr, pTree ptree, pMetapostConfig pmc, va_list ap)
{
	pPamlTree pptree = (pPamlTree)ptree;
	char* title = va_arg(ap,char*);
	string_add( pstr, "\n% annotation\n");
	string_fadd( pstr, "label( btex %s : %s etex, (0.1u, %fu));\n", title,  (pptree->model == 2 ? "Netural" : "Positive"), pmc->height + 1 ) ;
	string_fadd( pstr, "label( btex $\\ln L$ : %f etex, (0.1u, %fu));\n", pptree->lnL, pmc->height + 0.8 ) ;
	string_fadd( pstr, "label( btex $\\bar{\\omega}$ : %f etex, (0.1u, %fu));\n", pptree->meanOmega, pmc->height + 0.60  ) ;
	static char* omega[] = { "\\omega < 1" , "\\omega = 1", "\\omega > 1" };

	int i;
	for ( i = 0 ; i < pptree->model ; i++ )
	{
		string_fadd( pstr, "label( btex $%s$ : %f(%3.2f) etex, (0.1u, %fu));\n", omega[i],pptree->omega[i], pptree->prob[i], pmc->height + 0.45 - i * 0.15  ) ;
	}
	

/*
	string_fadd( pstr, "label( btex \\begin{tabular}{|c|c|c|} \\hline \
				            & \\omega & P(\\omega) \\\\ \\hline \
				\\omega < 1 &  %f & %f%  \\\\ \
				\\omega == 1 &  %f & %f% \\\\ ", 
				pptree->omega[0], pptree->prob[0],
				pptree->omega[1], pptree->prob[1] );
	if ( pptree->model == 3 )
	{
		string_fadd( pstr, "\\omega > 1 &  %f & %f% \\\\ ", pptree->omega[2], pptree->prob[2] );
	}
	string_fadd( pstr ,"\\hline\n\\end{taular} etex, (0.1u, %fu));\n", title, pmc->height ) ;
*/
	return 0;
}

double paml_tree_mp_annotation(pString pstr, pTreeNode ptnode, pMetapostConfig pmc, va_list ap)
{
	pPamlNode pnode = (pPamlNode)ptnode; 

	double last = 0;
    string_add( pstr, ";\n");
	if ( ptnode->parent )
	{
		string_fadd( pstr, "ypart mid[%d] = ypart(p[%d]);\n", ptnode->id, ptnode->id );
		string_fadd( pstr, "xpart mid[%d] = (xpart(p[%d])+xpart(p[%d]))/2;\n", ptnode->id, ptnode->id, ptnode->parent->id );
		if ( pnode->super.branchlength > 0 )
		{
			string_fadd( pstr, "label.rt( btex $\\small{l = %g}$ etex, mid[%d] + (-0.2u,%fu));\n", pnode->super.branchlength, ptnode->id, 0.50 );
		}
		if ( pnode->N != -1 )
		{
			string_fadd( pstr, "label.rt( btex $\\small{dN = %g}$ etex, mid[%d] + (-0.2u,%fu));\n",  pnode->dN, ptnode->id, 0.35 );
			string_fadd( pstr, "label.rt( btex $\\small{dS = %g}$ etex, mid[%d] + (-0.2u,%fu));\n",  pnode->dS, ptnode->id, 0.20 );
			string_fadd( pstr, "label.rt( btex $\\small{accuS = %g}$ etex, mid[%d] + (-0.2u,%fu));\n",  pnode->accudS, ptnode->id, 0.05 );
		}
	}
	return last;
}

void paml_tree_free(pPamlTree ptree)
{
	phylogeny_free((pTree)ptree);
	free(ptree->nei_gojobori);
	free(ptree->gene_names);
	free(ptree->omega);
	free(ptree->prob);
	free(ptree->nodelist);
}

pString paml_tree_metapost(pPamlTree ptree, int id, char* title, double width, double height)
{
	pMetapostConfig pmc = MEMORY_NEW(1,MetapostConfig);
	pmc->id = id;
	pmc->unit = MP_UNIT_IN;
	pmc->dir = MP_DIR_HORIZONTAL;
	pmc->shape = MP_SHAPE_RECT | MP_SHAPE_MOST_CENTER;
	pmc->fmod = paml_tree_mp_annotation;
	pmc->fremark = paml_tree_mp_remark;
	pmc->width = width;
	pmc->height = height;
	pString pstr = phylogeny_to_mp( (pTree)ptree, pmc, title);
	memory_free(pmc);
	return pstr;
}

void __paml_tree_string(pString pstr, pPhylogenyNode ptnode)
{
	pPamlNode pnode = (pPamlNode)ptnode;
	if ( ptnode->name )
	{
		string_fadd( pstr, "%s", ptnode->name );
	}
	string_fadd( pstr, "<%f/%f>", pnode->dN, pnode->dS );
}

void paml_tree_print_info(pPamlTree ptree)
{
	switch( ptree->model )
	{
		case 2:
			printf("Neutral:\n");
			break;
		case 3:
			printf("Positive:\n");
			break;
	}		
	int i;
	for ( i = 0 ; i < ptree->model ; i++ )
	{
		printf("%4.3f(%4.3f%%) ", ptree->omega[i], ptree->prob[i]*100 );
	}
	printf("\n");
	pString pstr = paml_tree_string(ptree);
	printf(pstr->buf);
	string_free(pstr);
}

pString paml_tree_string(pPamlTree ptree)
{
	return phylogeny_string((pTree)ptree,__paml_tree_string);
}

void paml_tree_accu_dS_from_bottom(pPamlTree ptree)
{
	pArrayList postfix = ptree->super.postfix;
	int i;
	for ( i = 0 ; i < postfix->size-1 ; i++ )
	{
		pPamlNode pnode = (pPamlNode)postfix->array[i];
		pPamlNode parent = (pPamlNode)(((pTreeNode)pnode)->parent);
		double accu = pnode->accudS + pnode->dS;
		if ( parent->accudS < accu )
		{
			parent->accudS = accu;
		}
	}
}

int paml_tree_get_nodes_over_threshold(pPamlTree ptree, double threshold, int* rtn)
{
	paml_tree_accu_dS_from_bottom(ptree);
	pArrayList nlist = ptree->super.nlist;
	int i;
	for( i = 1 ; i < nlist->size ; i+=2 )
	{
		pPamlNode pnode = (pPamlNode)nlist->array[i];
		if ( pnode->accudS < threshold )
		{
			((pTreeNode)pnode)->reg = 1;
		}
		else
		{
			((pTreeNode)pnode)->reg = 0;
		}
	}
	int cnt = 0;
	for( i = 1 ; i < nlist->size ; i+=2 )
	{
		pTreeNode pnode = (pTreeNode)nlist->array[i];
		if ( pnode->reg == 1 )
		{
			pTreeNode parent = pnode->parent;	
			while( parent && parent->reg == 1 )
			{
				pnode = parent;
				parent = pnode->parent;	
			}
			rtn[cnt++] = pnode->id;	
		}
	}
	return cnt;
}

int paml_tree_check_significant(pPamlTree ptree, pPamlTree parg, double t)
{
   return 2 * ( ptree->lnL - parg->lnL ) > t;
}

double paml_tree_average_w(pPamlTree ptree)
{
	int i;
	double avg = 0;
	for ( i = 0 ; i < ptree->model ; i++ )
	{
		avg += ptree->omega[i] * ptree->prob[i];
	}	
	return avg;
}

double paml_tree_average_nei_gojobori(pPamlTree ptree)
{
	int i;
	double sum = 0;
	if ( ptree->nei_gojobori == NULL ) return -1;
	int cnt = (int)(ptree->num_genes * ( ptree->num_genes - 1 ) / 2.0);
	int realcnt = 0;
	for ( i = 0 ; i < cnt ; i++ )
	{
		if ( ptree->nei_gojobori[i] != -1 )
		{
			sum += ptree->nei_gojobori[i];
			realcnt++;
		}
	}
	return realcnt == 0 ? -1 : sum/realcnt; 
}
