#include<utils_string.h>
#include<stdlib.h>

#define STRING_STEP_SIZE 2048

pString string_new_step(int step)
{
	pString pstr = (pString)memory_new(1,sizeof(String));
	pstr->buf = (char*)memory_new(step+1,sizeof(char));
	pstr->buf[0] = '\0';
	pstr->length = 0;
	pstr->alloc_size = step;
	return pstr;
}

pString string_new()
{
	return string_new_step(STRING_STEP_SIZE);
}

pString string_new_with_string(char* newstr )
{
	int len = strlen(newstr);
	pString pstr = string_new_step(len+1);		
	strcpy(pstr->buf,newstr);
	pstr->alloc_size = len;
	pstr->length = len;
	return pstr;
}

void string_free(pString pstr)
{
	free(pstr->buf);
	free(pstr);
}

void string_free_without_data(pString pstr)
{
	free(pstr);
}

void string_reset(pString pstr)
{
	pstr->length = 0;
	pstr->buf[0] = '\0';
}

void string_add(pString pstr, char* add)
{
	int len = strlen(add);
	if ( pstr->alloc_size < len + pstr->length )	
	{
		pstr->alloc_size += len + STRING_STEP_SIZE;
		pstr->buf = (char*)realloc(pstr->buf, pstr->alloc_size + 1 );
		if ( pstr->buf == NULL )
		{
			print_error(__FILE__,(char*)__FUNCTION__,__LINE__,
					    "memory reallocation of String->buf");		
		}
	}
	strcat(pstr->buf,add);
	pstr->length += len;
}

void string_fadd(pString pstr, char* msg, ... )
{
	char buf[BUFSIZ];
	va_list ap;
	va_start(ap,msg);
	vsprintf(buf,msg,ap);
	va_end(ap);

	string_add(pstr,buf);
}

void string_trim(pString pstr)
{
	pstr->buf = (char*)realloc(pstr->buf, pstr->length + 1 );
	if ( pstr->buf == NULL )
	{
		print_error(__FILE__,(char*)__FUNCTION__,__LINE__,
					 "memory reallocation of String->buf");		
	}
	pstr->alloc_size = pstr->length;
}

void string_chomp(pString pstr)
{
	string_pchar_chomp(pstr->buf);	
	int newlen = strlen(pstr->buf);
	pstr->length = newlen;
}

char* string_get(pString pstr)
{
	return pstr->buf;
}

int is_letter(char c)
{
	return ( 'A' <= c && 'Z' >= c ) || ( 'a' <= c && 'z' >= c );
}

int is_space(char c)
{
	return c == ' ' || c == '\t'  || c == '\n'  || c == '\r';
}

char* string_pchar_chomp(char* pstr)
{
	int from, end;
	for ( from = 0 ; pstr[from] ; from++ )
	{
		if ( !is_space(pstr[from]) ) break;;
	}
	for ( end = strlen(pstr) - 1 ; end >= 0 ; end-- )
	{
		if ( !is_space(pstr[end]) ) break;;
	}
	int i, j;
	for ( i = from, j = 0 ; i <= end ; i++, j++ )
	{
		pstr[j] = pstr[i];
	}
	pstr[j] = '\0';
	return pstr;
}

int string_pchar_cmp_ignore_case(char* cmp1, char* cmp2)
{
	int i;
	int len = strlen(cmp2);
	for ( i = 0 ; i < len ; i++ )
	{
		if ( is_letter(cmp1[i]) && is_letter(cmp2[i]) )
		{
			if ( (cmp1[i] & 0x5F) != (cmp2[i] & 0x5F) ) return 0;
		}
		else
		{
			if ( cmp1[i] != cmp2[i]) return 0;
		}
	}
	return 1;
}

pArrayList string_pchar_split(char* buf, char delim)
{
	pArrayList psplit = arraylist_new(20);
	char* begin = buf;
	char* str;
	int len;
	while( *buf )
	{
		if ( *buf == delim )
		{
			*buf++ = '\0';
			len = strlen(begin);
			str = NULL;
			if ( len )
			{
				str = (char*)memory_new( len+1, sizeof(char) );
				strcpy(str,begin);
			}
			arraylist_add(psplit,str);
			begin = buf;
		}
		else
		{
			buf++;
		}
	}
	len = strlen(begin);
	str = NULL;
	if ( len )
	{
		str = (char*)memory_new( len+1, sizeof(char) );
		strcpy(str,begin);
	}
	arraylist_add(psplit,str);
	return psplit;
}

pArrayList string_pchar_space_split(char* buf)
{
	pArrayList parg = arraylist_new(10);

    int i = 0;
	while( *buf == ' ' || *buf == '\t' ) buf++;
	char* start = buf;
	while( *buf )
	{
		if ( *buf == ' ' || *buf == '\t' || *buf == '\n' )
		{
			*buf++ = '\0';
			if ( i > 0 )
			{
				arraylist_add( parg, start );
			}
			start = buf;
			i = 0;
		}
		else
		{
			i++;
			buf++;
		}
	}
	if ( i > 0 )
	{
		arraylist_add( parg, start );
	}
	return parg;
}

void string_pchar_join_double(char* rtn, char* sp, int argc, double* values)
{
	int i;
	char buf[1024];
	for ( i = 0 ; i < argc ; i++ )
	{
		sprintf(buf,"%15.14lf", values[i] );
		strcat(rtn,buf);
		if ( i < argc - 1 ) strcat(rtn,sp);
	}
}

void string_pchar_join(char* buf, char* stuff, int num, char** list)
{
	int i;
	strcpy(buf,list[0]);
	for ( i = 1 ; i < num ; i++ )
	{
		if ( stuff ) strcat(buf, stuff);
		strcat(buf, list[i]);	
	}
}

pString string_join(char* stuff, int num, char** list)
{
	int i;
	pString pstr = string_new_with_string(list[0]);
	for( i = 1 ; i < num ; i++ )
	{
		string_add(pstr,stuff);
		string_add(pstr,list[i]);
	}
	return pstr;
}
